Package: batchmix
Type: Package
Title: Semi-Supervised Bayesian Mixture Models Incorporating Batch
        Correction
License: GPL-3
Version: 2.2.2
Authors@R: c(person(given = "Stephen", family = "Coleman",
                    email = "stcolema@tcd.ie",
                    role = c("aut","cre")),
             person(given = "Paul", family ="Kirk",
                    email = "paul.kirk@mrc-bsu.cam.ac.uk",
                    role = "aut"),
             person(given = "Chris", family ="Wallace",
                    email = "cew54@cam.ac.uk",
                    role = "aut"))
Description: Semi-supervised and unsupervised Bayesian mixture models that
  simultaneously infer the cluster/class structure and a batch correction.
  Densities available are the multivariate normal and the multivariate t.
  The model sampler is implemented in C++. This package is aimed at analysis of
  low-dimensional data generated across several batches. See Coleman et al.
  (2022) <doi:10.1101/2022.01.14.476352> for details of the model.
Imports: Rcpp (>= 1.0.5), tidyr, ggplot2, salso
LinkingTo: Rcpp, RcppArmadillo
Suggests: xml2, knitr, rmarkdown
SystemRequirements: GNU make
Encoding: UTF-8
URL: https://github.com/stcolema/batchmix
BugReports: https://github.com/stcolema/batchmix/issues
RoxygenNote: 7.3.3
VignetteBuilder: knitr, rmarkdown
NeedsCompilation: yes
Packaged: 2026-06-04 19:47:15 UTC; colemste
Author: Stephen Coleman [aut, cre],
  Paul Kirk [aut],
  Chris Wallace [aut]
Maintainer: Stephen Coleman <stcolema@tcd.ie>
Repository: CRAN
Date/Publication: 2026-06-04 20:30:07 UTC

useDynLib(batchmix)

importFrom(ggplot2,
  aes_string, 
  facet_grid, 
  facet_wrap, 
  geom_boxplot,
  geom_line, 
  geom_point, 
  ggplot,
  label_both,
  labeller,
  labs
)

importFrom(stats, 
  cutree,
  as.dist,
  hclust,
  median,
  rbeta,
  rchisq,
  rpois,
  rnorm
)

importFrom(tidyr, 
  any_of,
  contains, 
  pivot_longer
)

importFrom(Rcpp, evalCpp, sourceCpp)

export(
  calcAllocProb,
  collectAcceptanceRates,
  continueChain,
  continueChains,
  createSimilarityMat,
  generateBatchData,
  generateBatchDataLogPoisson,
  generateBatchDataMVT,
  generateBatchDataVaryingRepresentation,
  generateInitialLabels,
  getLikelihood,
  getSampledBatchScale,
  getSampledBatchShift,
  getSampledClusterMeans,
  minVI,
  plotAcceptanceRates,
  plotLikelihoods,
  plotSampledBatchMeans,
  plotSampledBatchScales,
  plotSampledClusterMeans,
  plotSampledParameter,
  predictClass,
  predictFromMultipleChains,
  processMCMCChain,
  processMCMCChains,
  rStickBreakingPrior,
  runBatchMix,
  runMCMCChains,
  samplePriorLabels
)

exportPattern("^[[:alpha:]]+")

# batchmix

Downloads from CRAN in the past month:

[![](https://cranlogs.r-pkg.org/badges/batchmix)](https://cran.r-project.org/package=batchmix)

Semi-supervised and unsupervised Bayesian mixture models that 
simultaneously infer the cluster/class structure and a batch correction. 
Densities available are the multivariate normal and the multivariate t. 
The model sampler is implemented in C++. This package is aimed at analysis of 
low-dimensional data generated across several batches. See 
[Coleman et al. (2022)](https://doi.org/10.1101/2022.01.14.476352) for details
of the model.

## Advice on using the package

The main functions a user should be aware of are ``runMCMCChains``, 
``plotLikelihoods``, ``plotAcceptanceRates``, ``continueChains`` and 
``processChains``. 

Parameters are sampled using Metropolis-Hastings so checking that the 
acceptance rate is important. We recommend aiming for acceptance
rates between 0.1 and 0.5 for the class and batch means and batch 
scales ($\mu_k$, $m_b$ and $S_b$ respectively). In our testing, an 
acceptance rate of at least 0.4 for the class covariance matrices
tended to suggest the sampler is exploring well, but smaller values 
were frequently associated with poor behaviour. The degrees of freedom tend 
to have very high acceptance rates in our testing regardless of the 
sampling window.

We recommend running a small number of chains for a small number of 
iterations to assess the acceptance rates before committing the 
computational resourcces to run a full analysis.

For an example of a workflow please see the short vignette.

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(echo = TRUE)

set.seed(1)

## ----dataGen------------------------------------------------------------------
library(ggplot2)
library(batchmix)

# Data dimensions
N <- 200
P <- 4
K <- 3
B <- 5

# Generating model parameters
mean_dist <- 2.25
batch_dist <- 0.3
group_means <- seq(1, K) * mean_dist
batch_shift <- rnorm(B, mean = batch_dist, sd = batch_dist)
std_dev <- rep(2, K)
batch_var <- rep(1.2, B)
group_weights <- rep(1 / K, K)
batch_weights <- rep(1 / B, B)
dfs <- c(4, 15, 120)

my_data <- generateBatchData(
  N,
  P,
  group_means,
  std_dev,
  batch_shift,
  batch_var,
  group_weights,
  batch_weights,
  type = "MVT",
  group_dfs = dfs
)

## ----dataClean----------------------------------------------------------------
X <- my_data$observed_data

true_labels <- my_data$group_IDs
fixed <- my_data$fixed
batch_vec <- my_data$batch_IDs

alpha <- 1
initial_labels <- generateInitialLabels(alpha, K, fixed, true_labels)

## ----runMCMCChains------------------------------------------------------------
# Sampling parameters
R <- 1000
thin <- 50
n_chains <- 3

# Density choice
type <- "MVT"

# MCMC samples and BIC vector
mcmc_output <- runMCMCChains(
  X,
  n_chains,
  R,
  thin,
  batch_vec,
  type,
  initial_labels = initial_labels,
  fixed = fixed
)

## ----plotAcceptanceRatesEarly-------------------------------------------------
plotAcceptanceRates(mcmc_output)

## ----likelihood---------------------------------------------------------------
plotLikelihoods(mcmc_output)

## ----continueChains-----------------------------------------------------------
R_new <- 9000

# Given an initial value for the parameters
new_output <- continueChains(
  mcmc_output,
  X,
  fixed,
  batch_vec,
  R_new,
  keep_old_samples = TRUE
)

## ----continuedLikelihood------------------------------------------------------
plotLikelihoods(new_output)

## ----plotAcceptanceRates------------------------------------------------------
plotAcceptanceRates(new_output)

## ----processChains------------------------------------------------------------
# Burn in
burn <- 5000

# Process the MCMC samples
processed_samples <- processMCMCChains(new_output, burn)

## ----pca----------------------------------------------------------------------
chain_used <- processed_samples[[1]]

pc <- prcomp(X, scale = T)
pc_batch_corrected <- prcomp(chain_used$inferred_dataset)

plot_df <- data.frame(
  PC1 = pc$x[, 1],
  PC2 = pc$x[, 2],
  PC1_bf = pc_batch_corrected$x[, 1],
  PC2_bf = pc_batch_corrected$x[, 2],
  pred_labels = factor(chain_used$pred),
  true_labels = factor(true_labels),
  prob = chain_used$prob,
  batch = factor(batch_vec)
)

plot_df |>
  ggplot(aes(
    x = PC1,
    y = PC2,
    colour = true_labels,
    alpha = prob
  )) +
  geom_point()

plot_df |>
  ggplot(aes(
    x = PC1_bf,
    y = PC2_bf,
    colour = pred_labels,
    alpha = prob
  )) +
  geom_point()

test_inds <- which(fixed == 0)

sum(true_labels[test_inds] == chain_used$pred[test_inds]) / length(test_inds)

#!/usr/bin/Rscript
#' @title Generate batch data
#' @description Generate data from groups across batches. Assumes independence
#' across columns. In each column the parameters are randomly permuted for both
#' the groups and batches.
#' @param N The number of items (rows) to generate.
#' @param P The number of columns in the generated dataset.
#' @param group_means A vector of the group means for a column.
#' @param group_std_dev A vector of group standard deviations for a column.
#' @param batch_shift A vector of batch means in a column.
#' @param batch_scale A vector of batch standard deviations within a column.
#' @param group_weights A K x B matrix of the expected proportion of N in each group in each batch.
#' @param batch_weights A vector of the expected proportion of N in each batch.
#' @param frac_known The expected fraction of observed labels. Used to generate
#' a ``fixed`` vector to feed into the ``batchSemiSupervisedMixtureModel`` function.
#' @return A list of 4 objects; the data generated from the groups with and
#' without batch effects, the label indicating the generating group and the
#' batch label.
#' @export
#' @examples
#' N <- 500
#' P <- 2
#' K <- 2
#' B <- 5
#' mean_dist <- 4
#' batch_dist <- 0.3
#' group_means <- seq(1, K) * mean_dist
#' batch_shift <- rnorm(B, mean = batch_dist, sd = batch_dist)
#' std_dev <- rep(2, K)
#' batch_var <- rep(1.2, B)
#' group_weights <- matrix(
#'   c(
#'     0.8, 0.6, 0.4, 0.2, 0.2,
#'     0.2, 0.4, 0.6, 0.8, 0.8
#'   ),
#'   nrow = K, ncol = B, byrow = TRUE
#' )
#' batch_weights <- rep(1 / B, B)
#'
#' my_data <- generateBatchDataVaryingRepresentation(
#'   N,
#'   P,
#'   group_means,
#'   std_dev,
#'   batch_shift,
#'   batch_var,
#'   group_weights,
#'   batch_weights
#' )
#' @importFrom stats rnorm
generateBatchDataVaryingRepresentation <- function(N,
                                                   P,
                                                   group_means,
                                                   group_std_dev,
                                                   batch_shift,
                                                   batch_scale,
                                                   group_weights,
                                                   batch_weights,
                                                   frac_known = 0.2) {
  # The number of groups to generate
  K <- length(group_means)

  # The number of batches to generate
  B <- length(batch_shift)

  if (ncol(group_weights) != B) {
    stop("Number of columns in group weight matrix does not match the number of batches.")
  }

  if (nrow(group_weights) != K) {
    stop("Number of rows in group weight matrix does not match the number of groupes.")
  }

  # The membership vector for the N points, currently empty
  labels <- rep(0, N)

  # The batch labels for the N points
  batches <- sample(seq(1, B), N, replace = TRUE, prob = batch_weights)

  # The fixed labels for the semi-supervised case
  fixed <- sample(seq(0, 1), N,
    replace = TRUE,
    prob = c(1 - frac_known, frac_known)
  )

  # The data matrices
  true_data <- observed_data <- matrix(0, nrow = N, ncol = P)

  # Iterate over the batches to sample appropriate labels
  for (b in seq(1, B)) {
    batch_ind <- which(batches == b)
    N_b <- length(batch_ind)

    labels[batch_ind] <- sample(seq(1, K), N_b,
      replace = TRUE,
      prob = group_weights[, b]
    )
  }

  # Generate the data
  for (p in seq(1, P)) {
    # To provide different information in each column, randomly sample the
    # parameters with each group and batch
    reordered_group_means <- sample(group_means)
    reordered_group_std_devs <- sample(group_std_dev)

    reordered_batch_shift <- sample(batch_shift)
    reordered_batch_scale <- sample(batch_scale)

    for (n in seq(1, N)) {
      # Find the group and batch
      b <- batches[n]
      k <- labels[n]

      # Random data
      x <- stats::rnorm(1)

      # For ease of reading the following lines, create group and batch parameters
      .mu <- reordered_group_means[k]
      .sd <- reordered_group_std_devs[k]
      .m <- reordered_batch_shift[b]
      .s <- reordered_batch_scale[b]

      # Corrected and observed data point
      true_data[n, p] <- x * .sd + .mu
      observed_data[n, p] <- x * .sd * .s + .mu + .m
    }
  }

  # Return a list of the group labels, batch labels, fixed points and the
  # observed and true datasets.
  list(
    observed_data = observed_data,
    corrected_data = true_data,
    group_IDs = labels,
    batch_IDs = batches,
    fixed = fixed
  )
}

#!/usr/bin/Rscript
#' @title Check proposal windows
#' @description Checks the proposal windows are acceptable.
#' @param mu_proposal_window The proposal window for the cluster mean proposal
#' kernel. The proposal density is a Gaussian distribution, the window is the
#' variance.
#' @param cov_proposal_window The proposal window for the cluster covariance
#' proposal kernel. The proposal density is a Wishart distribution, this
#' argument is the reciprocal of the degree of freedom.
#' @param m_proposal_window The proposal window for the batch mean proposal
#'  kernel. The proposal density is a Gaussian distribution, the window is the
#' variance.
#' @param S_proposal_window The proposal window for the batch standard deviation
#'  proposal kernel. The proposal density is a Gamma distribution, this
#' argument is the reciprocal of the rate.
#' @param t_df_proposal_window The proposal window for the degrees of freedom
#' for the multivariate t distribution (not used if type is not 'MVT'). The
#' proposal density is a Gamma distribution, this argument is the reciprocal of
#' the rate.
#' @param verbose Logical indicating if a warning should be printed if proposal
#' windows are outside their expected scale.
#' @return No return value, called for side effects
#' @examples
#' checkProposalWindows(0.1, 0.2, 0.3, 0.1, 0.4, 0.3)
checkProposalWindows <- function(mu_proposal_window,
                                 cov_proposal_window,
                                 m_proposal_window,
                                 S_proposal_window,
                                 t_df_proposal_window,
                                 verbose = TRUE) {
  windows <- c(
    mu_proposal_window,
    m_proposal_window,
    cov_proposal_window,
    S_proposal_window,
    t_df_proposal_window
  )

  windows_reciprocals_used <- c(
    cov_proposal_window,
    S_proposal_window,
    t_df_proposal_window
  )

  all_windows_positive <- all(windows > 0)
  if (!all_windows_positive) {
    stop(paste0(
      "All proposal windowws must be positive numbers. Normally they",
      " take values between 0 and 1."
    ))
  }

  unexpected_value_taken <- any(windows_reciprocals_used >= 1.0)
  if (unexpected_value_taken && verbose) {
    warning(paste0(
      "The proposal windows for the covariance matrix, the batch scale, the ",
      "degrees of freedom and the skew are normally expected to be less than 1.0."
    ))
  }
  NULL
}

#' @title Run Batch Mixture Model
#' @description Runs a MCMC chain for a Bayesian mixture model which models
#' both batch effects and class/cluster structure.
#' @param X Data to cluster as a matrix with the items to cluster held in rows.
#' @param initial_labels Initial clustering.
#' @param batch_vec Labels identifying which batch each item being clustered is
#' from.
#' @param R The number of iterations in the sampler.
#' @param thin The factor by which the samples generated are thinned, e.g. if
#' ``thin=50`` only every 50th sample is kept.
#' @param type Character indicating density type to use. One of 'MVN'
#' (multivariate normal distribution) or 'MVT' (multivariate t distribution).
#' @param K_max The number of components to include (the upper bound on the
#' number of clusters in each sample). Defaults to the number of unique labels
#' in ``initial_labels``.
#' @param fixed Which items are fixed in their initial label. If not given,
#' defaults to a vector of 0 meaning the model is run unsupervised.
#' @param alpha The concentration parameter for the stick-breaking prior and the
#' weights in the model.
#' @param mu_proposal_window The proposal window for the cluster mean proposal
#' kernel. Making this smaller will normally increase the acceptance rate for
#' the proposed values in the Metropolis-Hastings sampler. The proposal density
#' is a Gaussian distribution, the window is the variance.
#' @param cov_proposal_window The proposal window for the cluster covariance
#' proposal kernel. The proposal density is a Wishart distribution, this
#' argument is the reciprocal of the degree of freedom. It is recommended to
#' set this aiming for accpetance rates of greater than 0.5 for the covariance
#' matrices (e.g., between 2e-03 and 1e-04 is a good range to consider
#' initially). As the entire covariance matrix is sampled at once exploration is
#' difficult.
#' @param m_proposal_window The proposal window for the batch mean proposal
#'  kernel. The proposal density is a Gaussian distribution, the window is the
#' variance.
#' @param S_proposal_window The proposal window for the batch standard deviation
#'  proposal kernel. The proposal density is a Gamma distribution, this
#' argument is the reciprocal of the rate. Recommended range to initially
#' consider is 0.015 to 2e-03, though smaller values might be necessary
#' particularly in higher dimensional data.
#' @param t_df_proposal_window The proposal window for the degrees of freedom
#' for the multivariate t distribution (not used if type is not 'MVT'). The
#' proposal density is a Gamma distribution, this argument is the reciprocal of
#' the rate. If the data is more Gaussian than the degrees of freedom might have
#' high acceptance rates regardless of the value chosen.
#' @param m_scale The scale hyperparameter for the batch shift prior
#' distribution. This defines the scale of the batch effect upon the mean and
#' should be in (0, 1].
#' @param rho The shape of the prior distribution for the batch scale.
#' @param theta The scale of the prior distribution for the batch scale.
#' @param initial_class_means A $P x K$ matrix of initial values for the class
#' means. Defaults to draws from the prior distribution.
#' @param initial_class_covariance A $P x P x K$ array of initial values for
#' the class covariance matrices. Defaults to draws from the prior distribution.
#' @param initial_batch_shift A $P x B$ matrix of initial values for the batch
#' shift effect Defaults to draws from the prior distribution.
#' @param initial_batch_scale A $P x B$ matrix of initial values for the batch
#' scales Defaults to draws from the prior distribution.
#' @param initial_class_df A $K$ vector of initial values for the class degrees
#' of freedom. Defaults to draws from the prior distribution.
#' @param verbose Logiccal indicating if warning about proposal windows should
#' be printed.
#' @return A named list containing the sampled partitions, cluster and batch
#' parameters, model fit measures and some details on the model call.
#' @export
#' @examples
#'
#' # Data in a matrix format
#' X <- matrix(c(rnorm(100, 0, 1), rnorm(100, 3, 1)), ncol = 2, byrow = TRUE)
#'
#' # Initial labelling
#' labels <- c(
#'   rep(1, 10),
#'   sample(c(1, 2), size = 40, replace = TRUE),
#'   rep(2, 10),
#'   sample(c(1, 2), size = 40, replace = TRUE)
#' )
#'
#' fixed <- c(rep(1, 10), rep(0, 40), rep(1, 10), rep(0, 40))
#'
#' # Batch
#' batch_vec <- sample(seq(1, 5), replace = TRUE, size = 100)
#'
#' # Density choice
#' type <- "MVN"
#'
#' # Sampling parameters
#' R <- 1000
#' thin <- 50
#'
#' # MCMC samples
#' mcmc_out <- runBatchMix(
#'   X,
#'   R,
#'   thin,
#'   batch_vec,
#'   type,
#'   initial_labels = labels,
#'   fixed = fixed
#' )
#'
#' # Given an initial value for the parameters
#' initial_class_means <- matrix(c(1, 1, 3, 4), nrow = 2)
#' initial_class_covariance <- array(c(1, 0, 0, 1, 1, 0, 0, 1),
#'   dim = c(2, 2, 2)
#' )
#'
#' # We can use values from a previous chain
#' initial_batch_shift <- mcmc_out$batch_shift[, , R / thin]
#' initial_batch_scale <- matrix(
#'   c(1.2, 1.3, 1.7, 1.1, 1.4, 1.3, 1.2, 1.2, 1.1, 2.0),
#'   nrow = 2
#' )
#'
#' mcmc_out <- runBatchMix(X,
#'   R,
#'   thin,
#'   batch_vec,
#'   type,
#'   initial_labels = labels,
#'   fixed = fixed,
#'   initial_class_means = initial_class_means,
#'   initial_class_covariance = initial_class_covariance,
#'   initial_batch_shift = initial_batch_shift,
#'   initial_batch_scale = initial_batch_scale
#' )
#'
runBatchMix <- function(X,
                        R,
                        thin,
                        batch_vec,
                        type,
                        K_max = NULL,
                        initial_labels = NULL,
                        fixed = NULL,
                        alpha = 1,
                        mu_proposal_window = 0.5**2,
                        cov_proposal_window = 0.002,
                        m_proposal_window = 0.3**2,
                        S_proposal_window = 0.01,
                        t_df_proposal_window = 0.015,
                        m_scale = NULL,
                        rho = 3.0,
                        theta = 1.0,
                        initial_class_means = NULL,
                        initial_class_covariance = NULL,
                        initial_batch_shift = NULL,
                        initial_batch_scale = NULL,
                        initial_class_df = NULL,
                        verbose = TRUE) {
  unsupervised <- is.null(fixed)
  no_initial_partition_given <- is.null(initial_labels)
  if (!is.matrix(X)) {
    stop("X is not a matrix. Data should be in matrix format.")
  }

  semisupervised_with_no_initial_labels <- (!unsupervised) && no_initial_partition_given

  if (semisupervised_with_no_initial_labels) {
    .err <- paste0(
      "If running a semi-supervised model (fixed has any entries ",
      "equal to 1), then an initial labelling must be given."
    )
    stop(.err)
  }

  N <- nrow(X)
  if (unsupervised) {
    fixed <- rep(0, N)
  }

  no_k_passed <- is.null(K_max)
  if (no_k_passed) {
    if (no_initial_partition_given) {
      K_max <- min(floor(N / 2), 50)
    } else {
      K_max <- length(unique(initial_labels))
    }
  }

  if (no_initial_partition_given) {
    initial_labels <- generateInitialLabels(alpha, K_max, fixed)
  }

  mcmc_out <- batchSemiSupervisedMixtureModel(X,
    R,
    thin,
    initial_labels,
    fixed,
    batch_vec,
    type,
    K_max = K_max,
    alpha = alpha,
    mu_proposal_window = mu_proposal_window,
    cov_proposal_window = cov_proposal_window,
    m_proposal_window = m_proposal_window,
    S_proposal_window = S_proposal_window,
    t_df_proposal_window = t_df_proposal_window,
    m_scale = m_scale,
    rho = rho,
    theta = theta,
    initial_class_means = initial_class_means,
    initial_class_covariance = initial_class_covariance,
    initial_batch_shift = initial_batch_shift,
    initial_batch_scale = initial_batch_scale,
    initial_class_df = initial_class_df,
    verbose = verbose
  )

  mcmc_out
}

#!/usr/bin/Rscript
#' @title Sample prior labels
#' @description Generate labels from the stick-breaking prior.
#' @param alpha The concentration parameter for the stick-breaking prior.
#' @param K The number of components to include (the upper bound on the number of unique labels generated).
#' @param N The number of labels to generate.
#' @return A vector of labels.
#' @export
#' @examples
#' initial_labels <- samplePriorLabels(1, 50, 100)
samplePriorLabels <- function(alpha, K, N) {
  w <- rStickBreakingPrior(alpha, K)
  initial_labels <- sample(seq(1, K), N, replace = TRUE, prob = w)
}

#!/usr/bin/Rscript
#' @title Prepare initial values
#' @description Prepares user given values for input into the C++ function.
#' @param initial_class_means A $P x K$ matrix of initial values for the class
#' means. Defaults to draws from the prior distribution.
#' @param initial_class_covariance A $P x P x K$ array of initial values for the
#' class covariance matrices. Defaults to draws from the prior distribution.
#' @param initial_batch_shift A $P x B$ matrix of initial values for the batch
#' shift effect Defaults to draws from the prior distribution.
#' @param initial_batch_scale A $P x B$ matrix of initial values for the batch
#' scales Defaults to draws from the prior distribution.
#' @param initial_class_df A $K$ vector of initial values for the class degrees
#' of freedom. Defaults to draws from the prior distribution.
#' @param P Integer. The number of measurements for each sample in the dataset
#' being modelled.
#' @param K The number of classes/clusters being modelled.
#' @param B The number of batches being modelled.
#' @param type The type of mixture model used; one of "MVN" or "MVT".
#' @return A named list containing the different parameters.
prepareInitialParameters <- function(initial_class_means,
                                     initial_class_covariance,
                                     initial_batch_shift,
                                     initial_batch_scale,
                                     initial_class_df,
                                     P,
                                     K,
                                     B,
                                     type) {
  # Define placeholders. These will be used to pass to C++ if no initial values
  # are given. C++ isn't great for optional arguments, so this bypasses the
  # problem
  class_means <- matrix(-Inf, nrow = P, ncol = K)
  class_cov <- array(-Inf, dim = c(P, P, K))
  batch_shift <- matrix(0, nrow = P, ncol = B)
  batch_scale <- matrix(-Inf, nrow = P, ncol = B)
  class_df <- rep(-Inf, K)

  class_mean_passed <- !is.null(initial_class_means)
  class_covariance_passed <- !is.null(initial_class_covariance)
  batch_shift_passed <- !is.null(initial_batch_shift)
  batch_scale_passed <- !is.null(initial_batch_scale)
  class_df_passed <- !is.null(initial_class_df)

  if (class_mean_passed) {
    not_a_matrix <- !is.matrix(initial_class_means)
    if (not_a_matrix) {
      stop("Initial class means must be either ``NULL`` or a P x K matrix.")
    }

    incorrect_dimensions <- ((nrow(initial_class_means) != P) &
      (ncol(initial_class_means) != K)
    )

    if (incorrect_dimensions) {
      stop("Initial class means must be either ``NULL`` or a P x K matrix.")
    }

    class_means <- initial_class_means
  }

  if (class_covariance_passed) {
    error_msg <- paste0(
      "Initial class covariances must be either ``NULL`` or ",
      "a P x P x K array"
    )

    not_an_array <- !is.array(initial_class_covariance)
    if (not_an_array) {
      stop(error_msg)
    }

    incorrect_dimensions <- any(dim(initial_class_covariance) != c(P, P, K))

    if (not_a_matrix | incorrect_dimensions) {
      stop(error_msg)
    }

    class_cov <- initial_class_covariance
  }

  if (class_df_passed) {
    wrong_type <- type != "MVT"
    if (wrong_type) {
      warning("Initial class degrees of freedom passed but model is MVN. Please check.")
    }

    error_msg <- paste0(
      "Initial class degrees of freedom must be either ",
      "``NULL`` or a K vector (such as ``c(3, 5)``."
    )

    not_a_vector <- !is.vector(initial_class_df)
    if (not_a_vector) {
      stop(error_msg)
    }

    incorrect_dimensions <- length(initial_class_df) != K

    if (incorrect_dimensions) {
      stop(error_msg)
    }

    class_df <- initial_class_df
  }


  if (batch_shift_passed) {
    error_msg <- paste0(
      "Initial batch shifts must be either ``NULL`` or a ",
      "P x B matrix."
    )

    not_a_matrix <- !is.matrix(initial_batch_shift)
    if (not_a_matrix) {
      stop(error_msg)
    }

    incorrect_dimensions <- ((nrow(initial_batch_shift) != P) &
      (ncol(initial_batch_shift) != B)
    )

    if (incorrect_dimensions) {
      stop(error_msg)
    }

    batch_shift <- initial_batch_shift
  }

  if (batch_scale_passed) {
    error_msg <- paste0(
      "Initial batch scales must be either ``NULL`` or a ",
      "P x B matrix."
    )

    not_a_matrix <- !is.matrix(initial_batch_scale)
    if (not_a_matrix) {
      stop(error_msg)
    }

    incorrect_dimensions <- ((nrow(initial_batch_scale) != P) &
      (ncol(initial_batch_scale) != B)
    )

    if (incorrect_dimensions) {
      stop(error_msg)
    }

    batch_scale <- initial_batch_scale
  }

  list(
    class_means = class_means,
    class_cov = class_cov,
    batch_shift = batch_shift,
    batch_scale = batch_scale,
    class_df = class_df
  )
}

#' @title Minimum VI lower bound
#' @description Local implementation of S. Wade's `minVI` function from their
#' `mcclust.ext` package (available from github).
#' Reimplemented here to avoid dependency on a non-CRAN package. Computes the
#' lower bound to the posterior expected Variation of Information. For full
#' details please see the aforementioned package and Wade and Ghahramani, 2018,
#' 'Bayesian Cluster Analysis: Point Estimation and Credible Balls (with
#' Discussion)'.
#' @param cls A clustering for which the lower bound of the Variation of
#' Information is calculated.
#' @param psm The posterior similarity matrix which `cls` is a summary thereof.
#' @returns A vector of the the lower bound of the Variation of Information
#' for
#' @examples
#' \dontrun{
#' # MCMC samples and BIC vector
#' mcmc_outputs <- runMCMCChains(
#'   X,
#'   n_chains,
#'   R,
#'   thin,
#'   batch_vec,
#'   type
#' )
#'
#' # Note that in this toy example we have not applied a burn in
#' psm <- createSimilarityMat(mcmc_outputs[[1]]$samples)
#' VI.lb(mcmc_outputs[[1]]$samples[1, ], psm)
#' }
VI.lb <- function(cls, psm) {
  if (is.vector(cls)) {
    cls <- t(cls)
  }
  n <- nrow(psm)
  n_inds <- seq(1, n)

  VI.lb.compute <- function(c) {
    f <- 0
    n_inds <- seq(1, n)
    for (i in n_inds) {
      ind <- (c == c[i])
      f <- f + (log2(sum(ind))
      + log2(sum(psm[i, ]))
        - 2 * log2(sum(ind * psm[i, ]))
      ) / n
    }
    return(f)
  }
  output <- apply(cls, 1, VI.lb.compute)
  output
}

#' @title Minimium VI
#' @description Local implementation of S. Wade's `minVI` function from their
#' `mcclust.ext` package (available from github).
#' Reimplemented here to avoid dependency on a non-CRAN package and we have
#' dropped the `greedy` method. Finds the optimal partition by minimising the
#' lower bound to the Variation of Information obtained from Jensen's inequality
#' where the expectation and log are reversed. For full details please see the
#' aforementioned package and Wade and Ghahramani, 2018, 'Bayesian Cluster
#' Analysis: Point Estimation and Credible Balls (with Discussion)'.
#' @param psm The posterior similarity matrix for a set of clustering MCMC
#' samples such as is returned by the `createSimilarityMat` function.
#' @param cls.draw The set of clustering MCMC samples used to generate `psm`.
#' Only required if `method` is one of `'draws'` or `'all'`.
#' @param method String indicating which method is used to find the point
#' estimate clustering. Must be one of `'avg'`, `'comp'`, `'draws'` or `'all'`.
#' Defaults to `'avg'`. If `'all'` is passed the three methods are all applied
#' to return different choices of point clustering.
#' @param max.k The maximum number of clusters to consider. Only used by the
#' `'comp'` and `'avg'` methods. Defaults to one-quarter the number of data
#' points rounded up.
#' @returns If `method` is `'all'` returns a matrix of four clusterings, one for
#' each method and a repeat of that which performs best based on minimising the
#' Variation of Information between the clustering and the PSM. Otherwise
#' returns a vector. This is annotated with the attribute `"info"`, a named list
#' describing:
#'
#' * `.$loss`: the loss score used (Variation of Information)
#'
#' * `.$maxNClusters`: the `max.k` value used by the `'comp'` and `'avg'` methods
#'
#' * `.$expectedLoss`: the estimated minimum Variation of Information for the point
#' clustering(s)
#'
#' * `.$method`: the point method used to infer the clustering(s)
#'
#' Names are due to legacy reasons - this function is replacing the
#' `salso::salso` function and name choices are to minimise workflow damage.
#' @examples
#' \dontrun{
#' # MCMC samples and BIC vector
#' mcmc_outputs <- runMCMCChains(
#'   X,
#'   n_chains,
#'   R,
#'   thin,
#'   batch_vec,
#'   type
#' )
#'
#' # Note that in this toy example we have not applied a burn in
#' psm <- createSimilarityMat(mcmc_outputs[[1]]$samples)
#' cl_est <- minVI(psm, mcmc_outputs[[1]]$samples)
#' }
#' @importFrom stats hclust as.dist cutree
#' @export
minVI <- function(psm,
                  cls.draw = NULL,
                  method = "avg",
                  max.k = NULL) {
  wrong_method <- !(method %in% c("avg", "comp", "draws", "all"))
  if (wrong_method) {
    stop("method must be one of 'avg', 'comp', 'draws' or 'all'.")
  }

  method <- match.arg(method, choices = method)
  if (method %in% c("draws", "all") & is.null(cls.draw)) {
    stop("cls.draw must be provided if method=''draws''")
  }

  # If no maximum number of clusters is passed, set this to quarter of the
  # number of datapoints - note this is only used by 'avg' and 'comp' methods
  if (is.null(max.k)) {
    max.k <- ceiling(dim(psm)[1] / 4)
    k_inds <- seq(1, max.k)
  }

  # If using the 'average' method
  if (method == "avg" | method == "all") {
    hclust.avg <- hclust(as.dist(1 - psm), method = "average")
    cls.avg <- t(apply(matrix(k_inds), 1, function(x) cutree(hclust.avg, k = x)))
    VI.avg <- VI.lb(cls.avg, psm)
    val.avg <- min(VI.avg)
    cl.avg <- cls.avg[which.min(VI.avg), ]
    if (method == "avg") {
      avg_estimate <- cl.avg
      avg_info <- list(
        loss = "VI",
        maxNClusters = max.k,
        expectedLoss = val.avg,
        method = "avg"
      )
      attr(avg_estimate, "info") <- avg_info
      return(avg_estimate)
    }
  }

  if (method == "comp" | method == "all") {
    # if (is.null(max.k)) max.k <- ceiling(dim(psm)[1] / 8)
    hclust.comp <- hclust(as.dist(1 - psm), method = "complete")
    cls.comp <- t(apply(matrix(k_inds), 1, function(x) cutree(hclust.comp, k = x)))
    VI.comp <- VI.lb(cls.comp, psm)
    val.comp <- min(VI.comp)
    cl.comp <- cls.comp[which.min(VI.comp), ]
    if (method == "comp") {
      comp_estimate <- cl.comp
      comp_info <- list(
        loss = "VI",
        maxNClusters = max.k,
        expectedLoss = val.comp,
        method = "comp"
      )
      attr(comp_estimate, "info") <- comp_info
      return(comp_estimate)
    }
  }

  if (method == "draws" | method == "all") {
    n <- ncol(psm)
    EVI_lb_local <- function(c) {
      f <- 0
      n_inds <- seq(1, n)
      for (i in n_inds) {
        ind <- (c == c[i])
        f <- f + (log2(sum(ind))
        + log2(sum(psm[i, ]))
          - 2 * log2(sum(ind * psm[i, ]))
        ) / n
      }
      return(f)
    }
    VI.draws <- apply(cls.draw, 1, EVI_lb_local)
    val.draws <- min(VI.draws)
    cl.draw <- cls.draw[which.min(VI.draws), ]
    names(cl.draw) <- NULL
    if (method == "draws") {
      draws_estimate <- cl.draw
      draws_info <- list(
        loss = "VI",
        maxNClusters = max.k,
        expectedLoss = val.draws,
        method = "draws"
      )
      attr(draws_estimate, "info") <- draws_info
      return(draws_estimate)
    }
  }

  # If using "all" compile the information
  vals <- c(val.avg, val.comp, val.draws)
  cls <- rbind(cl.avg, cl.comp, cl.draw)
  cls <- rbind(cls[which.min(vals), ], cls)
  vals <- c(min(vals), vals)
  rownames(cls) <- names(vals) <- c("best", "avg", "comp", "draws")
  colnames(cls) <- NULL

  estimate <- cls
  info <- list(
    loss = "VI",
    maxNClusters = max.k,
    expectedLoss = vals,
    method = "all"
  )
  attr(estimate, "info") <- info
  estimate
}

#' @title Batch semisupervised mixture model
#' @description A Bayesian mixture model with batch effects.
#' @param X Data to cluster as a matrix with the items to cluster held in rows.
#' @param initial_labels Initial clustering.
#' @param fixed Which items are fixed in their initial label.
#' @param batch_vec Labels identifying which batch each item being clustered is
#' from.
#' @param R The number of iterations in the sampler.
#' @param thin The factor by which the samples generated are thinned, e.g. if
#' ``thin=50`` only every 50th sample is kept.
#' @param type Character indicating density type to use. One of 'MVN'
#' (multivariate normal distribution) or 'MVT' (multivariate t distribution).
#' @param K_max The number of components to include (the upper bound on the
#' number of clusters in each sample). Defaults to the number of unique labels
#' in ``initial_labels``.
#' @param alpha The concentration parameter for the stick-breaking prior and the
#' weights in the model.
#' @param concentration Initial concentration vector for component weights.
#' @param mu_proposal_window The proposal window for the cluster mean proposal
#' kernel. The proposal density is a Gaussian distribution, the window is the
#' variance.
#' @param cov_proposal_window The proposal window for the cluster covariance
#' proposal kernel. The proposal density is a Wishart distribution, this
#' argument is the reciprocal of the degree of freedom.
#' @param m_proposal_window The proposal window for the batch mean proposal
#'  kernel. The proposal density is a Gaussian distribution, the window is the
#' variance.
#' @param S_proposal_window The proposal window for the batch standard deviation
#'  proposal kernel. The proposal density is a Gamma distribution, this
#' argument is the reciprocal of the rate.
#' @param t_df_proposal_window The proposal window for the degrees of freedom
#' for the multivariate t distribution (not used if type is not 'MVT'). The
#' proposal density is a Gamma distribution, this argument is the reciprocal of
#' the rate.
#' @param m_scale The scale hyperparameter for the batch shift prior
#' distribution. This defines the scale of the batch effect upon the mean and
#' should be in (0, 1].
#' @param rho The shape of the prior distribution for the batch scale.
#' @param theta The scale of the prior distribution for the batch scale.
#' @param initial_class_means A $P x K$ matrix of initial values for the class
#' means. Defaults to draws from the prior distribution.
#' @param initial_class_covariance A $P x P x K$ array of initial values for
#' the class covariance matrices. Defaults to draws from the prior distribution.
#' @param initial_batch_shift A $P x B$ matrix of initial values for the batch
#' shift effect Defaults to draws from the prior distribution.
#' @param initial_batch_scale A $P x B$ matrix of initial values for the batch
#' scales Defaults to draws from the prior distribution.
#' @param initial_class_df A $K$ vector of initial values for the class degrees
#' of freedom. Defaults to draws from the prior distribution.
#' @param verbose Logiccal indicating if warning about proposal windows should
#' be printed.
#' @return A named list containing the sampled partitions, cluster and batch
#' parameters, model fit measures and some details on the model call.
#' @examples
#'
#' # Data in a matrix format
#' X <- matrix(c(rnorm(100, 0, 1), rnorm(100, 3, 1)), ncol = 2, byrow = TRUE)
#'
#' # Initial labelling
#' labels <- c(
#'   rep(1, 10),
#'   sample(c(1, 2), size = 40, replace = TRUE),
#'   rep(2, 10),
#'   sample(c(1, 2), size = 40, replace = TRUE)
#' )
#'
#' fixed <- c(rep(1, 10), rep(0, 40), rep(1, 10), rep(0, 40))
#'
#' # Batch
#' batch_vec <- sample(seq(1, 5), replace = TRUE, size = 100)
#'
#' # Density choice
#' type <- "MVN"
#'
#' # Sampling parameters
#' R <- 1000
#' thin <- 50
#'
#' # MCMC samples and BIC vector
#' samples <- batchSemiSupervisedMixtureModel(
#'   X,
#'   R,
#'   thin,
#'   labels,
#'   fixed,
#'   batch_vec,
#'   type
#' )
#'
#' # Given an initial value for the parameters
#' initial_class_means <- matrix(c(1, 1, 3, 4), nrow = 2)
#' initial_class_covariance <- array(c(1, 0, 0, 1, 1, 0, 0, 1),
#'   dim = c(2, 2, 2)
#' )
#'
#' # We can use values from a previous chain
#' initial_batch_shift <- samples$batch_shift[, , R / thin]
#' initial_batch_scale <- matrix(
#'   c(1.2, 1.3, 1.7, 1.1, 1.4, 1.3, 1.2, 1.2, 1.1, 2.0),
#'   nrow = 2
#' )
#'
#' samples <- batchSemiSupervisedMixtureModel(X,
#'   R,
#'   thin,
#'   labels,
#'   fixed,
#'   batch_vec,
#'   type,
#'   initial_class_means = initial_class_means,
#'   initial_class_covariance = initial_class_covariance,
#'   initial_batch_shift = initial_batch_shift,
#'   initial_batch_scale = initial_batch_scale
#' )
#'
batchSemiSupervisedMixtureModel <- function(X,
                                            R,
                                            thin,
                                            initial_labels,
                                            fixed,
                                            batch_vec,
                                            type,
                                            K_max = length(unique(initial_labels)),
                                            alpha = NULL,
                                            concentration = NULL,
                                            mu_proposal_window = 0.5**2,
                                            cov_proposal_window = 0.002,
                                            m_proposal_window = 0.3**2,
                                            S_proposal_window = 0.01,
                                            t_df_proposal_window = 0.015,
                                            m_scale = NULL,
                                            rho = 3.0,
                                            theta = 1.0,
                                            initial_class_means = NULL,
                                            initial_class_covariance = NULL,
                                            initial_batch_shift = NULL,
                                            initial_batch_scale = NULL,
                                            initial_class_df = NULL,
                                            verbose = TRUE) {
  if (!is.matrix(X)) {
    stop("X is not a matrix. Data should be in matrix format.")
  }

  if (length(batch_vec) != nrow(X)) {
    stop("The number of rows in X and the number of batch labels are not equal.")
  }

  if (R < thin) {
    warning("Iterations to run less than thinning factor. No samples recorded.")
  }

  # Check that the initial labels starts at 0, if not remedy this.
  if (!any(initial_labels == 0)) {
    initial_labels <- as.numeric(as.factor(initial_labels)) - 1
  }

  if (max(initial_labels) != (length(unique(initial_labels)) - 1)) {
    stop("initial labels are not all contiguous integers.")
  }

  # Check that the batch labels starts at 0, if not remedy this.
  if (!any(batch_vec == 0)) {
    batch_vec <- as.numeric(as.factor(batch_vec)) - 1
  }

  if (max(batch_vec) != (length(unique(batch_vec)) - 1)) {
    stop("batch labels are not all contiguous integers.")
  }

  # The number of batches present
  B <- length(unique(batch_vec))

  # The concentration parameter for the prior Dirichlet distribution of the
  # component weights.
  alpha_not_passed <- is.null(alpha)
  concentration_not_passed <- is.null(concentration)
  if ((!alpha_not_passed) && (!concentration_not_passed)) {
    stop("Only one of ``concentration`` or ``alpha`` should be passed.")
  }
  if (concentration_not_passed) {
    if (alpha_not_passed) {
      alpha <- 1.0 / K_max
    }
    concentration <- rep(alpha, K_max)
  }

  # Check the proposal windows are all strictly positive
  checkProposalWindows(
    mu_proposal_window,
    cov_proposal_window,
    m_proposal_window,
    S_proposal_window,
    t_df_proposal_window,
    verbose
  )

  # The proposal windows for these objects are narrower for larger quantities,
  # so we use the reciprocal to ensure that the relationship between acceptance
  # rates is the same for all parameters, namely that smaller windows increases
  # the acceptance rate
  actual_cov_proposal_window <- 1.0 / cov_proposal_window
  actual_S_proposal_window <- 1.0 / S_proposal_window
  actual_t_df_proposal_window <- 1.0 / t_df_proposal_window

  P <- ncol(X)

  # Check if an initial value is passed for any of the parameters. Prepare the
  # parameters to be passed to C++.
  class_mean_passed <- !is.null(initial_class_means)
  class_covariance_passed <- !is.null(initial_class_covariance)
  batch_shift_passed <- !is.null(initial_batch_shift)
  batch_scale_passed <- !is.null(initial_batch_scale)
  class_df_passed <- !is.null(initial_class_df)

  sample_m_scale <- is.null(m_scale)
  if(sample_m_scale) {
    m_scale <- 0.01
  }
  
  initial_parameters <- prepareInitialParameters(
    initial_class_means,
    initial_class_covariance,
    initial_batch_shift,
    initial_batch_scale,
    initial_class_df,
    P,
    K_max,
    B,
    type
  )

  class_means <- initial_parameters$class_means
  class_cov <- initial_parameters$class_cov
  batch_shift <- initial_parameters$batch_shift
  batch_scale <- initial_parameters$batch_scale
  class_df <- initial_parameters$class_df

  # Pull samples from the mixture model
  if (type == "MVN") {
    mcmc_output <- sampleSemisupervisedMVN(
      X,
      K_max,
      B,
      initial_labels,
      batch_vec,
      fixed,
      mu_proposal_window,
      actual_cov_proposal_window,
      m_proposal_window,
      actual_S_proposal_window,
      R,
      thin,
      concentration,
      m_scale,
      rho,
      theta,
      class_means,
      class_cov,
      batch_shift,
      batch_scale,
      class_mean_passed,
      class_covariance_passed,
      TRUE, # batch_shift_passed,
      batch_scale_passed,
      sample_m_scale
    )
  }

  if (type == "MVT") {
    mcmc_output <- sampleSemisupervisedMVT(
      X,
      K_max,
      B,
      initial_labels,
      batch_vec,
      fixed,
      mu_proposal_window,
      actual_cov_proposal_window,
      m_proposal_window,
      actual_S_proposal_window,
      actual_t_df_proposal_window,
      R,
      thin,
      concentration,
      m_scale,
      rho,
      theta,
      class_means,
      class_cov,
      class_df,
      batch_shift,
      batch_scale,
      class_mean_passed,
      class_covariance_passed,
      class_df_passed,
      TRUE, # batch_shift_passed,
      batch_scale_passed,
      sample_m_scale
    )
  }

  mcmc_output$concentration <- matrix(concentration, nrow = 1)


  if (!type %in% c("MVN", "MVT")) {
    stop("Type not recognised. Please use one of 'MVN' or 'MVT'.")
  }

  # Record details of model run to output
  # MCMC details
  mcmc_output$thin <- thin
  mcmc_output$R <- R
  mcmc_output$burn <- 0

  # Density choice
  mcmc_output$type <- type

  # Dimensions of data
  mcmc_output$P <- P
  mcmc_output$N <- nrow(X)

  # Number of components and batches modelled
  mcmc_output$K_max <- K_max
  mcmc_output$B <- B

  # Record hyperparameter choice
  mcmc_output$alpha <- alpha
  mcmc_output$m_scale <- m_scale
  mcmc_output$rho <- rho
  mcmc_output$theta <- theta

  # Proposal windows
  mcmc_output$mu_proposal_window <- mu_proposal_window
  mcmc_output$cov_proposal_window <- cov_proposal_window
  mcmc_output$m_proposal_window <- m_proposal_window
  mcmc_output$S_proposal_window <- S_proposal_window
  mcmc_output$t_df_proposal_window <- t_df_proposal_window

  # Indicate if the model was semi-supervised or unsupervised
  mcmc_output$Semisupervised <- TRUE

  # Correct this if we were effectively unsupervised
  actually_unsupervised <- all(fixed == 0)
  if (actually_unsupervised) {
    mcmc_output$Semisupervised <- FALSE
  }
  
  # Indicate if lambda^2 was sampled
  mcmc_output$sample_m_scale <- sample_m_scale
  
  mcmc_output
}

#!/usr/bin/Rscript
#' @title Plot acceptance rates
#' @description Plot the acceptance rates for the parameters sampled in a
#' Metropolis-Hastings step. Aiming for acceptance rates in [0.2, 0.5] for the
#' class means, the batch effect on location and scale is a good rule of thumb.
#' The class covariance should be in [0.35, 0.8] based on the authors'
#' experience. The class degree of freedom appears to be prone to high
#' acceptance rates, but aim to keep this above 0.2 at a minimum.
#' @param mcmc_lst The output of the ``runMCMCChains`` function.
#' @return A ggplot object of the boxplots of acceptance rates for each
#' parameter across chains.
#' @export
#' @examples
#' # Data in a matrix format
#' X <- matrix(c(rnorm(100, 0, 1), rnorm(100, 3, 1)), ncol = 2, byrow = TRUE)
#'
#' # Initial labelling
#' labels <- c(
#'   rep(1, 10),
#'   sample(c(1, 2), size = 40, replace = TRUE),
#'   rep(2, 10),
#'   sample(c(1, 2), size = 40, replace = TRUE)
#' )
#'
#' fixed <- c(rep(1, 10), rep(0, 40), rep(1, 10), rep(0, 40))
#'
#' # Batch
#' batch_vec <- sample(seq(1, 5), replace = TRUE, size = 100)
#'
#' # Sampling parameters
#' R <- 500
#' thin <- 10
#' n_chains <- 4
#'
#' # MCMC samples and BIC vector
#' mcmc_lst <- runMCMCChains(X, n_chains, R, thin, batch_vec, "MVN",
#'   initial_labels = labels,
#'   fixed = fixed
#' )
#'
#' # Plot the acceptance rate of each parameter in the 4 chains
#' plotAcceptanceRates(mcmc_lst)
#'
#' @importFrom ggplot2 ggplot aes_string geom_boxplot
#' @importFrom tidyr pivot_longer any_of
plotAcceptanceRates <- function(mcmc_lst) {
  type <- mcmc_lst[[1]]$type

  acceptance_rates <- do.call(
    rbind,
    lapply(mcmc_lst, collectAcceptanceRates)
  )

  acceptance_rates$Chain <- seq(1, length(mcmc_lst))

  plot_df <- tidyr::pivot_longer(acceptance_rates, -tidyr::any_of("Chain"),
    names_to = "Parameter",
    values_to = "Acceptance_rate"
  )

  p_out <- ggplot2::ggplot(
    plot_df,
    ggplot2::aes_string(x = "Parameter", y = "Acceptance_rate")
  ) +
    ggplot2::geom_boxplot() +
    ggplot2::ylim(0, 1)

  p_out
}

#!/usr/bin/Rscript
#' @title Collect acceptance rate
#' @description Collects the acceptance rates for each parameter into a data.frame
#' @param samples The output of ``runBatchMix``.
#' @return A wide data.frame of all the sampled parameters and the iteration.
#' @export
#' @examples
#'
#' # Data in a matrix format
#' X <- matrix(c(rnorm(100, 0, 1), rnorm(100, 3, 1)), ncol = 2, byrow = TRUE)
#'
#' # Initial labelling
#' labels <- c(
#'   rep(1, 10),
#'   sample(c(1, 2), size = 40, replace = TRUE),
#'   rep(2, 10),
#'   sample(c(1, 2), size = 40, replace = TRUE)
#' )
#'
#' fixed <- c(rep(1, 10), rep(0, 40), rep(1, 10), rep(0, 40))
#'
#' # Batch
#' batch_vec <- sample(seq(1, 5), replace = TRUE, size = 100)
#'
#' # Sampling parameters
#' R <- 1000
#' thin <- 50
#'
#' # MCMC samples
#' samples <- runBatchMix(X, R, thin, batch_vec, "MVN",
#'   initial_labels = labels,
#'   fixed = fixed
#' )
#'
#' # Acceptance rates
#' collectAcceptanceRates(samples)
#'
collectAcceptanceRates <- function(samples) {
  # Number of classes and batches
  K <- samples$K_max # ncol(samples$means[, , 1])
  B <- samples$B # ncol(samples$batch_shift[, , 1])
  P <- samples$P # nrow(samples$means[, , 1])
  N <- samples$N # ncol(samples$samples)
  type <- samples$type

  # Stack the sampled matrices on top of each other
  mean_rate <- t(samples$mu_acceptance_rate)
  colnames(mean_rate) <- paste0("Mu_", seq(1, K))
  cov_rate <- t(samples$cov_acceptance_rate)
  colnames(cov_rate) <- paste0("Sigma_", seq(1, K))

  batch_shift_rate <- t(samples$m_acceptance_rate)
  colnames(batch_shift_rate) <- paste0("m_", seq(1, B))
  batch_scale_rate <- t(samples$S_acceptance_rate)
  colnames(batch_scale_rate) <- paste0("S_", seq(1, B))

  output_df <- as.data.frame(
    cbind(
      mean_rate,
      cov_rate,
      batch_shift_rate,
      batch_scale_rate
    )
  )

  if (type == "MVT") {
    t_df_rate <- t(samples$t_df_acceptance_rate)
    colnames(t_df_rate) <- paste0("nu_", seq(1, K))
    output_df <- cbind(output_df, t_df_rate)
  }

  if (type == "MSN") {
    phi_rate <- t(samples$phi_acceptance_rate)
    colnames(phi_rate) <- paste0("phi_", seq(1, K))
    output_df <- cbind(output_df, phi_rate)
  }

  output_df
}

#!/usr/bin/Rscript
#' @title Plot likelihoods
#' @description Plots the model fit for multiple chains.
#' @param mcmc_outputs The output from ``runMCMCChains``.
#' @param choice The model fit score to use. Must be one of
#' ``'observed_likelihood'``, ``'complete_likelihood'`` or ``'BIC'``. Defaults
#' to ``'complete_likelihood'``.
#' @param colour_by_chain Logical indcating if plots should be coloured by chain
#' or all the same colour. Defaults to ``TRUE``.
#' @return A ggplot2 object. Line plot of likelihood across iteration.
#' @importFrom ggplot2 ggplot aes_string geom_line
#' @export
#' @examples
#'
#' # Data in a matrix format
#' X <- matrix(c(rnorm(100, 0, 1), rnorm(100, 3, 1)), ncol = 2, byrow = TRUE)
#'
#' # Initial labelling
#' labels <- c(
#'   rep(1, 10),
#'   sample(c(1, 2), size = 40, replace = TRUE),
#'   rep(2, 10),
#'   sample(c(1, 2), size = 40, replace = TRUE)
#' )
#'
#' fixed <- c(rep(1, 10), rep(0, 40), rep(1, 10), rep(0, 40))
#'
#' # Batch
#' batch_vec <- sample(seq(1, 5), replace = TRUE, size = 100)
#'
#' # Sampling parameters
#' R <- 1000
#' thin <- 50
#' n_chains <- 4
#'
#' # MCMC samples
#' samples <- runMCMCChains(X, n_chains, R, thin, batch_vec, "MVN",
#'   initial_labels = labels,
#'   fixed = fixed
#' )
#'
#' p <- plotLikelihoods(samples)
#'
plotLikelihoods <- function(mcmc_outputs,
                            choice = "complete_likelihood",
                            colour_by_chain = TRUE) {
  lkl_lst <- lapply(mcmc_outputs, getLikelihood, choice)

  n_chains <- length(lkl_lst)
  for (ii in seq(1, n_chains)) {
    lkl_lst[[ii]]$Chain <- mcmc_outputs[[ii]]$Chain
  }

  lkl_df <- do.call(rbind, lkl_lst)
  lkl_df$Chain <- factor(lkl_df$Chain)

  if (colour_by_chain) {
    p <- ggplot2::ggplot(
      data = lkl_df,
      mapping = ggplot2::aes_string(
        x = "iteration",
        y = choice,
        colour = "Chain"
      )
    ) +
      ggplot2::geom_line()
  } else {
    p <- ggplot2::ggplot(
      data = lkl_df,
      mapping = ggplot2::aes_string(
        x = "iteration",
        y = choice,
        group = "Chain"
      )
    ) +
      ggplot2::geom_line()
  }
  p
}

#!/usr/bin/Rscript
#
#' @title Check data generation inputs
#' @description Checks that the inputs for the ``generateBatchData`` function
#' are correct. For internal use only.
#' @param N The number of items (rows) to generate.
#' @param P The number of columns in the generated dataset.
#' @param group_means A vector of the group means for a column.
#' @param group_std_devs A vector of group standard deviations for a column.
#' @param batch_shift A vector of batch means in a column.
#' @param batch_scale A vector of batch standard deviations within a column.
#' @param group_weights A K x B matrix of the expected proportion of N in each group in each batch.
#' @param batch_weights A vector of the expected proportion of N in each batch.
#' @param type A string indicating if data should be generated from
#' multivariate normal ("MVN") or multivariate t ("MVT") densities.
#' @param group_dfs A K-vector of the group specific degrees of freedom.
#' @param frac_known The number of items with known labels.
#' @param permute_variables Logical indicating if group and batch means and
#' standard deviations should be permuted in each column or not.
#' @param scale_data Logical indicating if data should be mean centred and
#' standardised.
#' @return No return value, called for side effects.
#' @examples
#' N <- 500
#' P <- 2
#' K <- 2
#' B <- 5
#' mean_dist <- 4
#' batch_dist <- 0.3
#' group_means <- seq(1, K) * mean_dist
#' batch_shift <- rnorm(B, mean = batch_dist, sd = batch_dist)
#' group_std_devs <- rep(2, K)
#' batch_scale <- rep(1.2, B)
#' group_weights <- rep(1 / K, K)
#' batch_weights <- rep(1 / B, B)
#' type <- "MVT"
#' group_dfs <- c(4, 7)
#' frac_known <- 0.3
#' permute_variables <- TRUE
#' scale_data <- FALSE
#'
#' checkDataGenerationInputs(
#'   N,
#'   P,
#'   group_means,
#'   group_std_devs,
#'   batch_shift,
#'   batch_scale,
#'   group_weights,
#'   batch_weights,
#'   type,
#'   group_dfs,
#'   frac_known,
#'   permute_variables,
#'   scale_data
#' )
checkDataGenerationInputs <- function(N,
                                      P,
                                      group_means,
                                      group_std_devs,
                                      batch_shift,
                                      batch_scale,
                                      group_weights,
                                      batch_weights,
                                      type,
                                      group_dfs,
                                      frac_known,
                                      permute_variables,
                                      scale_data) {
  N_not_positive <- (N <= 0)
  N_not_whole_number <- (N %% 1) != 0
  if (N_not_positive || N_not_whole_number) {
    stop("N must be a positive whole number.")
  }

  P_not_positive <- (P <= 0)
  P_not_whole_number <- (P %% 1) != 0
  if (P_not_positive || P_not_whole_number) {
    stop("P must be a positive whole number.")
  }

  valid_density_passed <- (type == "MVN" || type == "MVT")
  if (!valid_density_passed) {
    stop("``type`` must be one of 'MVN' or 'MVT'.")
  }

  mvn_generated <- type == "MVN"
  mvt_generated <- type == "MVT"

  if (mvt_generated) {
    dfs_not_passed <- is.null(group_dfs)
    if (dfs_not_passed) {
      .err <- paste0(
        "If generating from a MVT, please pass values for the group degrees ",
        "of freedom (``group_dfs``)."
      )
      stop(.err)
    }
    dfs_too_small <- any(group_dfs < 1.0)
    if (dfs_too_small) {
      stop("Group degrees of freedom must be greater than or equal to 1.")
    }
  }

  # The number of batches to generate
  B <- length(batch_weights)

  wrong_length_batch_shift <- length(batch_shift) != B
  wrong_length_batch_scale <- length(batch_scale) != B

  # Allow for varying groups within batches and then record the number of groups
  # to generate
  varying_group_within_batch <- is.matrix(group_weights)
  if (varying_group_within_batch) {
    K <- nrow(group_weights)
    if (ncol(group_weights) != B) {
      stop("Number of columns in group weight matrix does not match the number of batches.")
    }
  } else {
    K <- length(group_weights)
  }

  wrong_length_group_means <- length(group_means) != K
  wrong_length_group_std_devs <- length(group_std_devs) != K


  if (any(
    wrong_length_group_means,
    wrong_length_group_std_devs,
    wrong_length_batch_shift,
    wrong_length_batch_scale
  )) {
    .err <- paste0(
      "Number of group and batch weights must match the number of ",
      "parameters in each of ``group_means``, ``group_std_devs``, ",
      "``batch_shift`` and ``batch_scale``."
    )
    stop(.err)
  }

  frac_known_not_in_0_1 <- (frac_known < 0.0 || frac_known > 1.0)
  if (frac_known_not_in_0_1) {
    stop("``frac_known`` must be a value between 0 and 1 (inclusive).")
  }

  batch_scale_too_small <- any(batch_scale < 1.0)
  if (batch_scale_too_small) {
    stop("All entries in ``batch_scale`` must be greater than or equal to 1.0.")
  }
  non_positive_std_devs <- any(group_std_devs <= 0.0)
  if (non_positive_std_devs) {
    stop("All entries of ``group_std_devs`` must be strictly positive.")
  }

  permute_variables_not_logical <- !is.logical(permute_variables)
  if (permute_variables_not_logical) {
    stop("``permute_variable`` must be logical.")
  }

  scale_data_not_logical <- !is.logical(scale_data)
  if (scale_data_not_logical) {
    stop("``scale_data`` must be logical.")
  }
}

#!/usr/bin/Rscript
#
#' @title Generate group IDs
#' @description Generate group IDs within ``generateBatchData``.
#' @param N The number of items (rows) to generate.
#' @param K The number of groups to genetare.
#' @param B The number of batches present in ``batch_IDs``.
#' @param batch_IDs The batch membership of each item.
#' @param group_weights One of either a K x B matrix of the expected proportion
#' of each batch in each group or a K-vector of the expected proportion of the
#' entire dataset in each group.
#' @param varying_group_within_batch Flag indicating if the groups are vvarying
#' across batches.
#' @return A N-vector of group membership.
#' @examples
#' N <- 500
#' K <- 2
#' B <- 5
#' group_weights <- rep(1 / K, K)
#' batch_weights <- rep(1 / B, B)
#' batch_IDs <- sample(seq(1, B), N, replace = TRUE, prob = batch_weights)
#' varying_group_within_batch <- FALSE
#' group_IDs <- generateGroupIDsInSimulator(
#'   N,
#'   K,
#'   B,
#'   batch_IDs,
#'   group_weights,
#'   varying_group_within_batch
#' )
generateGroupIDsInSimulator <- function(N,
                                        K,
                                        B,
                                        batch_IDs,
                                        group_weights,
                                        varying_group_within_batch) {
  # Generate group membership, potentially allowing imbalance across batches
  group_IDs <- rep(0, N)
  if (varying_group_within_batch) {
    # Iterate over the batches to sample appropriate labels
    for (b in seq(1, B)) {
      batch_ind <- which(batch_IDs == b)
      N_b <- length(batch_ind)

      group_IDs[batch_ind] <- sample(seq(1, K), N_b,
        replace = TRUE,
        prob = group_weights[, b]
      )
    }
  } else {
    # The membership vector for the N points
    group_IDs <- sample(seq(1, K), N, replace = TRUE, prob = group_weights)
  }
  group_IDs
}

#!/usr/bin/Rscript
#' @title Process MCMC chain
#' @description Applies a burn in to and finds a point estimate for the output
#' of ``batchSemiSupervisedMixtureModel``.
#' @param mcmc_output Output from ``batchSemiSupervisedMixtureModel``
#' @param burn The number of MCMC samples to drop as part of a burn in.
#' @param point_estimate_method Summary statistic used to define the point
#' estimate. Must be ``'mean'`` or ``'median'``. ``'median'`` is the default.
#' @returns A named list similar to the output of
#' ``batchSemiSupervisedMixtureModel`` with some additional entries:
#'
#'  * ``mean_est``: $(P x K)$ matrix. The point estimate of the cluster
#'  means with columns  corresponding to clusters.
#'
#'  * ``cov_est``: $(P x P x K)$ array. The point estimate of the
#'  cluster covariance matrices with slices corresponding to clusters.
#'
#'  * ``shift_est``: $(P x B)$ matrix. The point estimate of the batch
#'  shift effect with columns  corresponding to batches.
#'
#'  * ``scale_est``: $(P x B)$ matrix. The point estimate of the batch
#'  scale effects. The $bth$ column contains the diagonal entries of the scaling
#'  matrix for the $bth£ batch.
#'
#'  * ``mean_sum_est``: $(P x K x B)$ array. The point estimate of the
#'  sum of the cluster  means and the batch shift effect with columns
#'  corresponding to clusters and slices to batches.
#'
#'  * ``cov_comb_est``: List of length $B$, with each entry being a
#'  $(P x P x K)$ array. The point estimate of the combination of the
#'  cluster covariance matrices and the batch scale effect with list entries
#'  corresponding to batches and slices of each array corresponding to clusters.
#'
#'  * ``inferred_dataset``: $(N x P)$ matrix. The inferred ``batch-free''
#'  dataset.
#'
#'  * ``allocation_probability``: $(N x K)$ matrix. The point estimate of
#'  the allocation probabilities for each data point to each class.
#'
#'  * ``prob``: $N$ vector. The point estimate of the probability of being
#'  allocated to the class with the highest probability.
#'
#'  * ``pred``: $N$ vector. The predicted class for each sample.
#'
#' @export
#' @examples
#'
#' # Data in a matrix format
#' X <- matrix(c(rnorm(100, 0, 1), rnorm(100, 3, 1)), ncol = 2, byrow = TRUE)
#'
#' # Initial labelling
#' labels <- c(
#'   rep(1, 10),
#'   sample(c(1, 2), size = 40, replace = TRUE),
#'   rep(2, 10),
#'   sample(c(1, 2), size = 40, replace = TRUE)
#' )
#'
#' fixed <- c(rep(1, 10), rep(0, 40), rep(1, 10), rep(0, 40))
#'
#' # Batch
#' batch_vec <- sample(seq(1, 5), replace = TRUE, size = 100)
#'
#' # Sampling parameters
#' R <- 1000
#' burn <- 250
#' thin <- 50
#'
#' # MCMC samples
#' samples <- runBatchMix(X, R, thin, batch_vec, "MVN",
#'   initial_labels = labels,
#'   fixed = fixed
#' )
#'
#' # Process the MCMC samples
#' processed_samples <- processMCMCChain(samples, burn)
#'
#' @importFrom stats median
processMCMCChain <- function(mcmc_output, burn, point_estimate_method = "median") {
  # Dimensions of the dataset
  N <- mcmc_output$N
  P <- mcmc_output$P
  K_max <- mcmc_output$K_max
  B <- mcmc_output$B

  # The type of mixture model used
  type <- mcmc_output$type

  # Indices for clusters and batches
  batch_inds <- seq(1, B)
  cluster_inds <- seq(1, K_max)

  # MCMC iterations and thinning
  R <- mcmc_output$R
  thin <- mcmc_output$thin

  # Is the output semisupervised
  is_semisupervised <- mcmc_output$Semisupervised

  # What summary statistic is used to define our point estimates
  use_median <- point_estimate_method == "median"
  use_mean <- point_estimate_method == "mean"
  wrong_method <- !(use_median | use_mean)
  if (wrong_method) {
    stop("Wrong point estimate method given. Must be one of 'mean' or 'median'")
  }

  # We burn the floor of burn / thin of these
  eff_burn <- floor(burn / thin)

  # We record only the floor of R / thin samples
  eff_R <- floor(R / thin) - eff_burn

  # The indices dropped as part of the burn in
  dropped_indices <- seq(1, eff_burn)

  new_output <- mcmc_output

  # First apply a burn in to all quantities
  new_output$batch_corrected_data <- mcmc_output$batch_corrected_data[, , -dropped_indices]

  # The model fit measurements
  new_output$observed_likelihood <- mcmc_output$observed_likelihood[-dropped_indices, ]
  new_output$complete_likelihood <- mcmc_output$complete_likelihood[-dropped_indices, ]
  new_output$BIC <- mcmc_output$BIC[-dropped_indices, ]
  
  # The allocations and allocation probabilities
  new_output$samples <- mcmc_output$samples[-dropped_indices, ]

  if (is_semisupervised) {
    new_output$alloc <- mcmc_output$alloc[, , -dropped_indices]
  }

  # The sampled parameters
  new_output$means <- mcmc_output$means[, , -dropped_indices, drop = FALSE]
  new_output$covariance <- mcmc_output$covariance[, , -dropped_indices, drop = FALSE]
  new_output$batch_shift <- mcmc_output$batch_shift[, , -dropped_indices, drop = FALSE]
  new_output$batch_scale <- mcmc_output$batch_scale[, , -dropped_indices, drop = FALSE]
  new_output$mean_sum <- mcmc_output$mean_sum[, , -dropped_indices, drop = FALSE]
  new_output$cov_comb <- mcmc_output$cov_comb[, , -dropped_indices, drop = FALSE]

  new_output$weights <- mcmc_output$weights[-dropped_indices, , drop = FALSE]

  m_scale_sampled <- mcmc_output$sample_m_scale
  if(m_scale_sampled) {
    new_output$lambda_2 <- mcmc_output$lambda_2[-dropped_indices]
  }

  # The mean of the posterior samples for the parameters
  if (use_mean) {
    mean_est <- rowMeans(new_output$means, dims = 2L)
    shift_est <- rowMeans(new_output$batch_shift, dims = 2L)
    scale_est <- rowMeans(new_output$batch_scale, dims = 2L)
  }

  if (use_median) {
    mean_est <- apply(new_output$means, c(1, 2), stats::median)
    shift_est <- apply(new_output$batch_shift, c(1, 2), stats::median)
    scale_est <- apply(new_output$batch_scale, c(1, 2), stats::median)
  }

  if (type == "MVT") {
    new_output$t_df <- mcmc_output$t_df[-dropped_indices, , drop = FALSE]
    if (use_mean) {
      new_output$t_df_est <- colMeans(new_output$t_df)
    }
    if (use_median) {
      new_output$t_df_est <- apply(new_output$t_df, 2, stats::median)
    }
  }

  # The covariance is represented as a matrix for reasons, but is more naturally
  # thought of as a 3D array
  if (use_mean) {
    cov_est <- rowMeans(new_output$covariance, dims = 2L)
  }
  if (use_median) {
    cov_est <- apply(new_output$covariance, c(1, 2), stats::median)
  }

  cov_est_better_format <- array(0, c(P, P, K_max))

  # The indices for the columns corresponding to the first column for each
  # clusters' covariance matrix, with one trailing index that is used as a bound
  cov_comb_cluster_inds <- cov_inds <- seq(1, P * (K_max + 1), by = P)

  for (k in cluster_inds) {
    lb <- cov_inds[k]
    ub <- cov_inds[k + 1] - 1
    columns_selected <- seq(lb, ub)
    cov_est_better_format[, , k] <- cov_est[, columns_selected]
  }

  # The combinations of the batch and group parameters are a little awkward
  # Do a nicer a format and record point estimates
  if (use_mean) {
    mean_sum_est <- rowMeans(new_output$mean_sum, dims = 2L)
  }
  if (use_median) {
    mean_sum_est <- apply(new_output$mean_sum, c(1, 2), stats::median)
  }

  mean_sum_better_format <- array(0, c(P, K_max, B))

  if (use_mean) {
    cov_comb_est <- rowMeans(new_output$cov_comb, dims = 2L)
  }
  if (use_median) {
    cov_comb_est <- apply(new_output$cov_comb, c(1, 2), stats::median)
  }

  cov_comb_better_format <- vector("list", B)
  cov_comb_better_format_entry <- array(0, c(P, P, K_max))

  cov_comb_batch_inds <- seq(0, (P + 1) * K_max * B, by = P * K_max)

  # Iterate over batches saving the mean sums and covariance combinations in a
  # more obvious fashion
  for (b in batch_inds) {
    mean_sum_better_format[, , b] <- mean_sum_est[, seq(b, b + K_max - 1)]

    rel_inds <- cov_comb_batch_inds[b] + cov_comb_cluster_inds
    for (k in cluster_inds) {
      lb <- rel_inds[k]
      ub <- rel_inds[k + 1] - 1
      columns_selected <- seq(lb, ub)

      cov_comb_better_format_entry[, , k] <- cov_comb_est[, columns_selected]
    }
    cov_comb_better_format[[b]] <- cov_comb_better_format_entry
  }

  # Save hte estimated parameters to the output object
  new_output$mean_est <- mean_est
  new_output$shift_est <- shift_est
  new_output$scale_est <- scale_est
  new_output$cov_est <- cov_est_better_format
  new_output$mean_sum_est <- mean_sum_better_format
  new_output$cov_comb_est <- cov_comb_better_format

  # The estimate of the inferred dataset
  if (use_mean) {
    inferred_dataset <- rowMeans(new_output$batch_corrected_data, dims = 2L)
  }
  if (use_median) {
    inferred_dataset <- apply(new_output$batch_corrected_data, c(1, 2), stats::median)
  }

  new_output$inferred_dataset <- inferred_dataset

  if (is_semisupervised) {
    # The estimate of the allocation probability matrix, the probability of the
    # most probable class and the predicted class
    new_output$allocation_probability <- .alloc_prob <- calcAllocProb(new_output,
      method = point_estimate_method
    )

    new_output$prob <- apply(.alloc_prob, 1, max)
    new_output$pred <- apply(.alloc_prob, 1, which.max)
  }

  # Record the applied burn in
  new_output$burn <- burn

  # Return the MCMC object with burn in applied and point estimates found
  new_output
}

#!/usr/bin/Rscript
#' @title Plot sampled cluster means
#' @description Plot the sampled values for the cluster means in each
#' dimension from the output of the mixture model functions. Not recommended
#' for large K or P.
#' @param samples The output of the ``batchUnsupervisedMixtureModel`` or
#' ``batchSemiSupervisedMixtureModel`` functions.
#' @param burn_in The samples at the beginning of the chain to drop. Defaults to 0.
#' @return A ggplot object of the values in each sampled cluster mean per iteration.
#' @export
#' @examples
#'
#' # Data in matrix format
#' X <- matrix(c(rnorm(100, 0, 1), rnorm(100, 3, 1)), ncol = 2, byrow = TRUE)
#'
#' # Observed batches represented by integers
#' batch_vec <- sample(seq(1, 5), size = 100, replace = TRUE)
#'
#' # MCMC iterations (this is too low for real use)
#' R <- 100
#' thin <- 5
#'
#' # MCMC samples and BIC vector
#' samples <- runBatchMix(X, R, thin, batch_vec, "MVN")
#'
#' # Plot the sampled value of the cluster means against MCMC iteration
#' plotSampledClusterMeans(samples)
#'
#' @importFrom ggplot2 ggplot aes_string geom_point facet_grid labs labeller label_both
plotSampledClusterMeans <- function(samples, burn_in = 0) {
  K <- samples$K_max
  P <- samples$P

  R <- samples$R
  thin <- samples$thin

  # Check that the values of R and thin make sense
  if (floor(R / thin) != nrow(samples$samples)) {
    stop("The ratio of R to thin does not match the number of samples present.")
  }

  sampled_cluster_means <- getSampledClusterMeans(samples$means, K, P,
    R = R,
    thin = thin
  )

  # Remove warm-up samples
  sampled_cluster_means <- sampled_cluster_means[
    sampled_cluster_means$Iteration > burn_in,
  ]

  # Create the plot as aggplot2 object
  p <- ggplot2::ggplot(
    sampled_cluster_means,
    ggplot2::aes_string(x = "Iteration", y = "value")
  ) +
    ggplot2::geom_point() +
    ggplot2::facet_grid(Cluster ~ Dimension,
      labeller = ggplot2::labeller(
        Cluster = ggplot2::label_both,
        Dimension = ggplot2::label_both
      )
    ) +
    ggplot2::labs(
      title = "Cluster means",
      x = "MCMC iteration",
      y = "Sampled value"
    )

  p
}

#' @title Predict from multiple MCMC chains
#' @description Applies a burn in to and finds a point estimate by combining
#' multiple chains of ``callMDI``.
#' @param mcmc_outputs Output from ``runMCMCChains``
#' @param burn The number of MCMC samples to drop as part of a burn in.
#' @param point_estimate_method Summary statistic used to define the point
#' estimate. Must be ``'mean'`` or ``'median'``. ``'median'`` is the default.
#' @param chains_already_processed Logical indicating if the chains have already
#' had a burn-in applied.
#' @param nCores Integer used by salso to parallelize point estimate prediction.
#' @returns A named list of quantities related to prediction/clustering:
#'
#'  * ``allocation_probability``: List with an $(N x K)$ matrix if the model is
#'    semi-supervised. The point estimate of the allocation probabilities for
#'    each data point to each class.
#'
#'  * ``prob``: $N$ vector of the point estimate of the probability of being
#'    allocated to the class with the highest probability.
#'
#'  * ``pred``: $N$ vector of the predicted class for each sample. If the model
#'    is unsupervised then the ``salso`` function from Dahl et al. (2021) is
#'    used on the sampled partitions using the default settings.
#'
#'  * ``samples``: List of sampled allocations for each view. Columns
#'    correspond to items being clustered, rows to MCMC samples.
#' @examples
#' \donttest{
#' # Data dimensions
#' N <- 600
#' P <- 4
#' K <- 5
#' B <- 7
#'
#' # Generating model parameters
#' mean_dist <- 2.25
#' batch_dist <- 0.3
#' group_means <- seq(1, K) * mean_dist
#' batch_shift <- rnorm(B, mean = batch_dist, sd = batch_dist)
#' std_dev <- rep(2, K)
#' batch_var <- rep(1.2, B)
#' group_weights <- rep(1 / K, K)
#' batch_weights <- rep(1 / B, B)
#' dfs <- c(4, 7, 15, 60, 120)
#'
#' my_data <- generateBatchData(
#'   N,
#'   P,
#'   group_means,
#'   std_dev,
#'   batch_shift,
#'   batch_var,
#'   group_weights,
#'   batch_weights,
#'   type = "MVT",
#'   group_dfs = dfs
#' )
#'
#'
#' X <- my_data$observed_data
#'
#' true_labels <- my_data$group_IDs
#' fixed <- my_data$fixed
#' batch_vec <- my_data$batch_IDs
#'
#' alpha <- 1
#' initial_labels <- generateInitialLabels(alpha, K, fixed, true_labels)
#'
#' # Sampling parameters
#' R <- 1000
#' thin <- 25
#' burn <- 100
#' n_chains <- 2
#'
#' # Density choice
#' type <- "MVT"
#'
#' # MCMC samples and BIC vector
#' mcmc_outputs <- runMCMCChains(
#'   X,
#'   n_chains,
#'   R,
#'   thin,
#'   batch_vec,
#'   type,
#'   initial_labels = initial_labels,
#'   fixed = fixed
#' )
#' ensemble_mod <- predictFromMultipleChains(mcmc_outputs, burn)
#' }
#' @export
predictFromMultipleChains <- function(mcmc_outputs,
                                      burn,
                                      point_estimate_method = "median",
                                      chains_already_processed = FALSE,
                                      nCores = 1L) {
  if (chains_already_processed) {
    processed_chains <- mcmc_outputs
  } else {
    # Process the chains, making point estimates and applying a burn-in
    processed_chains <- processMCMCChains(mcmc_outputs, burn, point_estimate_method)
  }

  # The number of chains
  n_chains <- length(processed_chains)
  chain_indices <- seq(1, n_chains)

  # This is used to derive some characteristics of the model run
  first_chain <- processed_chains[[1]]

  # Dimensions of the dataset
  N <- first_chain$N
  K <- first_chain$K

  # MCMC call
  R <- first_chain$R
  thin <- first_chain$thin

  # The type of mixture model used
  type <- first_chain$type

  # Is the output semisupervised
  is_semisupervised <- first_chain$Semisupervised

  # What summary statistic is used to define our point estimates
  use_median <- point_estimate_method == "median"
  use_mean <- point_estimate_method == "mean"
  wrong_method <- !(use_median | use_mean)
  if (wrong_method) {
    stop("Wrong point estimate method given. Must be one of 'mean' or 'median'")
  }

  # We burn the floor of burn / thin of these
  eff_burn <- floor(burn / thin)

  # We record only the floor of R / thin samples
  eff_R <- floor(R / thin) - eff_burn

  # The indices dropped as part of the burn in
  dropped_indices <- seq(1, eff_burn)

  # Setup the output list
  merged_outputs <- list()

  merged_outputs$R <- R
  merged_outputs$thin <- thin
  merged_outputs$burn <- burn
  merged_outputs$n_chains <- n_chains
  merged_outputs$Semisupervised <- is_semisupervised

  merged_outputs$Point_estimate <- point_estimate_method

  merged_outputs$N <- N
  merged_outputs$K <- K

  merged_outputs$weights <- do.call(rbind, lapply(processed_chains, function(x) x$weights))
  merged_outputs$concentration <- do.call(rbind, lapply(processed_chains, function(x) x$concentration))

  first_chain <- TRUE

  merged_outputs$allocation_probability <- .alloc_prob <- matrix(
    0,
    N,
    K
  )
  for (ii in chain_indices) {
    .curr_chain <- processed_chains[[ii]]
    in_first_chain <- (ii == 1)

    if (in_first_chain) {
      merged_outputs$samples <- .curr_chain$samples[, , drop = TRUE]
    } else {
      .prev <- merged_outputs$samples
      .current <- .curr_chain$samples[, , drop = TRUE]
      merged_outputs$samples <- rbind(.prev, .current)
    }

    if (is_semisupervised) {
      .prev <- .alloc_prob
      .curr <- .curr_chain$allocation_probability

      .alloc_prob <- .prev + .curr
    }
  }

  if (is_semisupervised) {
    # Normalise the probabilities
    .alloc_prob <- .alloc_prob / n_chains

    merged_outputs$allocation_probability <- .alloc_prob

    merged_outputs$prob <- .prob <- apply(.alloc_prob, 1, max)
    merged_outputs$pred <- apply(.alloc_prob, 1, which.max)
  } else {
    # merged_outputs$psm <- .psm <- createSimilarityMat(merged_outputs$samples)
    # merged_outputs$pred <- minVI(.psm, merged_outputs$samples, method = "avg")
    merged_outputs$pred <- suppressWarnings(salso::salso(merged_outputs$samples, nCores = nCores))
  }
  merged_outputs
}

#' # This dummy function definition is included with the package to ensure that
#' # 'tools::package_native_routine_registration_skeleton()' generates the required
#' # registration info for the 'run_testthat_tests' symbol.
#' #' @title Run testthat tests
#' #' @description This dummy function definition is included with the package to ensure that
#' #' ``tools::package_native_routine_registration_skeleton()`` generates the required
#' #' registration info for the 'run_testthat_tests' symbol. This documentation is
#' #' solely to overcome ``devtools::check()``.
#' (function() {
#'   .Call("run_testthat_tests", FALSE, PACKAGE = "batchmix")
#' })

#!/usr/bin/Rscript
#
#' @title Generate batch data
#' @description Generate data from K multivaraite normal or multivariate t
#' distributions with additional noise from batches. Assumes independence across
#' columns. In each column the parameters are randomly permuted for both the
#' groups and batches.
#' @param N The number of items (rows) to generate.
#' @param P The number of columns in the generated dataset.
#' @param group_means A vector of the group means for a column.
#' @param group_std_devs A vector of group standard deviations for a column.
#' @param batch_shift A vector of batch means in a column.
#' @param batch_scale A vector of batch standard deviations within a column.
#' @param group_weights One of either a K x B matrix of the expected proportion
#' of each batch in each group or a K-vector of the expected proportion of the
#' entire dataset in each group.
#' @param batch_weights A vector of the expected proportion of N in each batch.
#' @param type A string indicating if data should be generated from
#' multivariate normal ("MVN") or multivariate t ("MVT") densities (defaults to
#' "MVN").
#' @param group_dfs A K-vector of the group specific degrees of freedom.
#' @param frac_known The number of items with known labels.
#' @param permute_variables Logical indicating if group and batch means and
#' standard deviations should be permuted in each column or not (defaults to
#' ``TRUE``).
#' @param scale_data Logical indicating if data should be mean centred and
#' standardised (defaults to ``FALSE``).
#' @return A list of 5 objects; the data generated from the groups with and
#' without batch effects, the label indicating the generating group, the
#' batch label and the vector indicating training versus test.
#' @export
#' @examples
#' N <- 500
#' P <- 2
#' K <- 2
#' B <- 5
#' mean_dist <- 4
#' batch_dist <- 0.3
#' group_means <- seq(1, K) * mean_dist
#' batch_shift <- rnorm(B, mean = batch_dist, sd = batch_dist)
#' std_dev <- rep(2, K)
#' batch_var <- rep(1.2, B)
#' group_weights <- rep(1 / K, K)
#' batch_weights <- rep(1 / B, B)
#' dfs <- c(4, 7)
#' my_data <- generateBatchData(
#'   N,
#'   P,
#'   group_means,
#'   std_dev,
#'   batch_shift,
#'   batch_var,
#'   group_weights,
#'   batch_weights,
#'   type = "MVT",
#'   group_dfs = dfs
#' )
#' @importFrom stats rnorm rchisq
generateBatchData <- function(N,
                              P,
                              group_means,
                              group_std_devs,
                              batch_shift,
                              batch_scale,
                              group_weights,
                              batch_weights,
                              type = "MVN",
                              group_dfs = NULL,
                              frac_known = 0.2,
                              permute_variables = TRUE,
                              scale_data = FALSE) {
  checkDataGenerationInputs(
    N,
    P,
    group_means,
    group_std_devs,
    batch_shift,
    batch_scale,
    group_weights,
    batch_weights,
    type,
    group_dfs,
    frac_known,
    permute_variables,
    scale_data
  )

  mvn_generated <- type == "MVN"
  mvt_generated <- type == "MVT"

  # The number of batches and groups to generate
  B <- length(batch_weights)
  K <- length(group_means)

  # Allow for varying groups within batches
  varying_group_within_batch <- is.matrix(group_weights)

  # The batch labels for the N points
  batch_IDs <- sample(seq(1, B), N, replace = TRUE, prob = batch_weights)

  # Generate group membership, potentially allowing imbalance across batches
  group_IDs <- generateGroupIDsInSimulator(
    N,
    K,
    B,
    batch_IDs,
    group_weights,
    varying_group_within_batch
  )

  # Fixed labels
  fixed <- sample(seq(0, 1), N,
    replace = TRUE,
    prob = c(1 - frac_known, frac_known)
  )

  # The data matrices
  observed_data <- true_data <- matrix(nrow = N, ncol = P)


  # If not permuting variables across features, set them outside of the loop
  reordered_group_means <- group_means
  reordered_group_std_devs <- group_std_devs

  reordered_batch_shift <- batch_shift
  reordered_batch_scale <- batch_scale

  # Iterate over each of the columns permuting the means associated with each
  # label.
  for (p in seq(1, P))
  {
    if (permute_variables) {
      # To provide different information in each column, randomly sample the
      # parameters with each group and batch
      reordered_group_means <- sample(group_means)
      reordered_group_std_devs <- sample(group_std_devs)

      reordered_batch_shift <- sample(batch_shift)
      reordered_batch_scale <- sample(batch_scale)
    }

    # Draw n points from the K univariate Gaussians defined by the permuted means.
    for (n in seq(1, N)) {
      # Draw a point from a standard normal
      x <- stats::rnorm(1)
      k <- group_IDs[n]
      b <- batch_IDs[n]

      # For ease of reading the following lines, create group and batch parameters
      .mu <- reordered_group_means[k]
      .sd <- reordered_group_std_devs[k]
      .m <- reordered_batch_shift[b]
      .s <- reordered_batch_scale[b]

      if (mvn_generated) {
        # Adjust to the group distribution
        true_data[n, p] <- x * .sd + .mu

        # Adjust to the batched group distribution
        observed_data[n, p] <- x * .sd * .s + .mu + .m
      }

      if (mvt_generated) {
        chi_draw <- stats::rchisq(1, group_dfs[k])

        # Adjust to the group distribution
        true_data[n, p] <- x * .sd * sqrt(group_dfs[k] / chi_draw) + .mu

        # Adjust to the batched group distribution
        observed_data[n, p] <- x * .sd * .s * sqrt(group_dfs[k] / chi_draw) + .mu + .m
      }
    }
  }

  row.names(observed_data) <- row.names(true_data) <- paste0("Item_", seq(1, N))
  if (scale_data) {
    observed_data <- scale(observed_data)
    true_data <- scale(true_data)
  }

  # Return the data, the data without batch effects, the allocation labels and
  # the batch labels.
  list(
    observed_data = observed_data,
    corrected_data = true_data,
    group_IDs = group_IDs,
    batch_IDs = batch_IDs,
    fixed = fixed
  )
}

#!/usr/bin/Rscript
#' @title Generate initial labels
#' @description For simulated data, generates an initial labelling for sampling.
#' @param alpha The mass in the stick breaking prior
#' @param K The number of classes available.
#' @param fixed The vector of 0s and 1s indicating which labels are to be held
#' fixed.
#' @param labels The initial labelling. Defaults to NULL.
#' @return An N vector of labels.
#' @export
#' @examples
#' N <- 500
#' P <- 2
#' K <- 2
#' B <- 5
#' mean_dist <- 4
#' batch_dist <- 0.3
#' cluster_means <- seq(1, K) * mean_dist
#' batch_shift <- rnorm(B, mean = batch_dist, sd = batch_dist)
#' std_dev <- rep(2, K)
#' batch_var <- rep(1.2, B)
#' cluster_weights <- rep(1 / K, K)
#' batch_weights <- rep(1 / B, B)
#'
#' my_data <- generateBatchData(
#'   N,
#'   P,
#'   cluster_means,
#'   std_dev,
#'   batch_shift,
#'   batch_var,
#'   cluster_weights,
#'   batch_weights
#' )
#'
#' initial_labels <- generateInitialLabels(1, K, my_data$fixed)
generateInitialLabels <- function(alpha, K, fixed, labels = NULL) {
  N <- length(fixed)
  if (is.null(labels)) {
    labels <- rep(0, N)
  }

  N_fixed <- sum(fixed)
  N_unfixed <- N - N_fixed
  labels_available <- seq(1, K)

  # Draw prior weights from the stick breaking prior
  w <- rStickBreakingPrior(alpha, K)

  labels[fixed == 0] <- sample(labels_available, N_unfixed,
    replace = TRUE,
    prob = w
  )

  labels
}

#' @title Continue chain
#' @description Continues sampling from a previous position for a given chain.
#' @param mcmc_output Chain to be continued.
#' @param X Data to cluster as a matrix with the items to cluster held in rows.
#' @param fixed The indicator vector for which labels are observed.
#' @param batch_vec The vector of the batch labels for the data.
#' @param R The number of iterations to run in this continuation (thinning
#' factor is the same as initial chain).
#' @param keep_old_samples Logical indicating if the original samples should be
#' kept or only the new samples returned. Defaults to TRUE.
#' @return A named list containing the sampled partitions, cluster and batch
#' parameters, model fit measures and some details on the model call.
#' @export
#' @examples
#'
#' # Data in a matrix format
#' X <- matrix(c(rnorm(100, 0, 1), rnorm(100, 3, 1)), ncol = 2, byrow = TRUE)
#'
#' # Initial labelling
#' labels <- c(
#'   rep(1, 10),
#'   sample(c(1, 2), size = 40, replace = TRUE),
#'   rep(2, 10),
#'   sample(c(1, 2), size = 40, replace = TRUE)
#' )
#'
#' fixed <- c(rep(1, 10), rep(0, 40), rep(1, 10), rep(0, 40))
#'
#' # Batch
#' batch_vec <- sample(seq(1, 5), replace = TRUE, size = 100)
#'
#' # Density choice
#' type <- "MVT"
#'
#' # Sampling parameters
#' R <- 1000
#' thin <- 50
#'
#' # MCMC samples and BIC vector
#' mcmc_output <- runBatchMix(
#'   X,
#'   R,
#'   thin,
#'   batch_vec,
#'   type,
#'   initial_labels = labels,
#'   fixed = fixed
#' )
#'
#' # Given an initial value for the parameters
#' mcmc_output <- continueChain(
#'   mcmc_output,
#'   X,
#'   fixed,
#'   batch_vec,
#'   R,
#' )
#'
continueChain <- function(mcmc_output,
                          X,
                          fixed,
                          batch_vec,
                          R,
                          keep_old_samples = TRUE) {
  # The relevant aspects of the previous chain
  R_old <- mcmc_output$R
  thin <- mcmc_output$thin
  last_sample <- R_eff_old <- floor(R_old / thin)

  B <- mcmc_output$B
  K_max <- mcmc_output$K_max
  N <- mcmc_output$N
  P <- mcmc_output$P

  type <- mcmc_output$type

  alpha <- mcmc_output$alpha
  m_scale <- mcmc_output$m_scale
  rho <- mcmc_output$rho
  theta <- mcmc_output$theta

  mu_proposal_window <- mcmc_output$mu_proposal_window
  cov_proposal_window <- mcmc_output$cov_proposal_window
  m_proposal_window <- mcmc_output$m_proposal_window
  S_proposal_window <- mcmc_output$S_proposal_window
  t_df_proposal_window <- mcmc_output$t_df_proposal_window

  initial_class_means <- mcmc_output$means[, , last_sample]

  initial_class_covariance <- array(0, dim = c(P, P, K_max))
  for (k in seq(1, K_max)) {
    col_i <- (k - 1) * P + 1
    col_j <- k * P
    rel_cols <- seq(col_i, col_j)
    initial_class_covariance[, , k] <- mcmc_output$covariance[, rel_cols, last_sample]
  }

  initial_batch_shift <- mcmc_output$batch_shift[, , last_sample]
  initial_batch_scale <- mcmc_output$batch_scale[, , last_sample]

  is_mvt <- type == "MVT"
  is_semisupervised <- mcmc_output$Semisupervised
  initial_class_df <- NULL
  if (is_mvt) {
    initial_class_df <- mcmc_output$t_df[last_sample, ]
  }

  initial_concentration <- mcmc_output$concentration
  
  sample_m_scale <- is.null(m_scale)
  if(sample_m_scale) {
    old_lambda2 <- c(mcmc_output$lambda_2)
  }

  labels <- mcmc_output$samples[last_sample, ]

  new_samples <- batchSemiSupervisedMixtureModel(X,
    R,
    thin,
    labels,
    fixed,
    batch_vec,
    type,
    K_max = K_max,
    concentration = initial_concentration,
    mu_proposal_window = mu_proposal_window,
    cov_proposal_window = cov_proposal_window,
    m_proposal_window = m_proposal_window,
    S_proposal_window = S_proposal_window,
    t_df_proposal_window = t_df_proposal_window,
    m_scale = m_scale,
    rho = rho,
    theta = theta,
    initial_class_means = initial_class_means,
    initial_class_covariance = initial_class_covariance,
    initial_class_df = initial_class_df,
    initial_batch_shift = initial_batch_shift,
    initial_batch_scale = initial_batch_scale
  )

  if (keep_old_samples) {
    R_eff <- floor(R / thin)

    R_comb_eff <- R_eff + R_eff_old

    # R_comb is has to be calculated this way. Reason, consider R_old = 107,
    # R = 113, thin = 10; then R_old + R = 220, 220 / 10 = 22, but we only
    # record R_eff_old = floor(107 / 10) = 10, R_eff = floor(113 / 10) = 11,
    # R_comb_eff = 21.
    R_comb <- R_comb_eff * thin

    # Let's combine the sampled parameters
    combined_allocation_samples <- rbind(
      mcmc_output$samples,
      new_samples$samples
    )

    combined_means <- array(c(mcmc_output$means, new_samples$means),
      dim = c(P, K_max, R_comb_eff)
    )

    combined_shifts <- array(
      c(
        mcmc_output$batch_shift,
        new_samples$batch_shift
      ),
      dim = c(P, B, R_comb_eff)
    )

    combined_scales <- array(
      c(
        mcmc_output$batch_scale,
        new_samples$batch_scale
      ),
      dim = c(P, B, R_comb_eff)
    )

    combined_mean_sums <- array(
      c(
        mcmc_output$mean_sum,
        new_samples$mean_sum
      ),
      dim = c(P, K_max * B, R_comb_eff)
    )

    combined_covariances <- array(
      c(
        mcmc_output$covariance,
        new_samples$covariance
      ),
      dim = c(P, P * K_max, R_comb_eff)
    )

    combined_covariance_comb <- array(
      c(
        mcmc_output$cov_comb,
        new_samples$cov_comb
      ),
      dim = c(P, P * K_max * B, R_comb_eff)
    )

    combined_weights <- rbind(mcmc_output$weights, new_samples$weights)

    comb_cov_acceptance_rate <- ((mcmc_output$cov_acceptance_rate * R_old +
      new_samples$cov_acceptance_rate * R)
    / (R_old + R)
    )

    comb_mu_acceptance_rate <- ((mcmc_output$mu_acceptance_rate * R_old +
      new_samples$mu_acceptance_rate * R)
    / (R_old + R)
    )

    comb_m_acceptance_rate <- ((mcmc_output$m_acceptance_rate * R_old +
      new_samples$m_acceptance_rate * R)
    / (R_old + R)
    )

    comb_S_acceptance_rate <- ((mcmc_output$S_acceptance_rate * R_old +
      new_samples$S_acceptance_rate * R)
    / (R_old + R)
    )
    
    if(sample_m_scale) {
      comb_lambda2 <- c(old_lambda2, new_samples$lambda_2)
    }
    

    if (type == "MVT") {
      comb_t_df_acceptance_rate <- ((mcmc_output$t_df_acceptance_rate * R_old +
        new_samples$t_df_acceptance_rate * R)
      / (R_old + R)
      )

      comb_t_df <- rbind(mcmc_output$t_df, new_samples$t_df)
    }

    # if(is_semisupervised) {
    comb_allocation_probs <- array(
      c(
        mcmc_output$alloc,
        new_samples$alloc
      ),
      dim = c(N, K_max, R_comb_eff)
    )

    # }

    comb_complete_likelihood <- rbind(
      mcmc_output$complete_likelihood,
      new_samples$complete_likelihood
    )

    comb_observed_likelihood <- rbind(
      mcmc_output$observed_likelihood,
      new_samples$observed_likelihood
    )

    comb_bic <- rbind(
      mcmc_output$BIC,
      new_samples$BIC
    )


    comb_inferred_data <- array(
      c(
        mcmc_output$batch_corrected_data,
        new_samples$batch_corrected_data
      ),
      dim = c(N, P, R_comb_eff)
    )

    new_samples$R <- R_comb

    new_samples$means <- combined_means
    new_samples$covariance <- combined_covariances
    new_samples$batch_shift <- combined_shifts
    new_samples$batch_scale <- combined_scales
    new_samples$mean_sum <- combined_mean_sums
    new_samples$cov_comb <- combined_covariance_comb

    new_samples$weights <- combined_weights
    new_samples$cov_acceptance_rate <- comb_cov_acceptance_rate
    new_samples$mu_acceptance_rate <- comb_mu_acceptance_rate
    new_samples$m_acceptance_rate <- comb_m_acceptance_rate
    new_samples$S_acceptance_rate <- comb_S_acceptance_rate

    new_samples$observed_likelihood <- comb_observed_likelihood
    new_samples$complete_likelihood <- comb_complete_likelihood
    new_samples$BIC <- comb_bic

    new_samples$samples <- combined_allocation_samples
    new_samples$alloc <- comb_allocation_probs
    new_samples$batch_corrected_data <- comb_inferred_data

    if(sample_m_scale) {
      new_samples$lambda_2 <- comb_lambda2
    }
    
    if (is_mvt) {
      new_samples$t_df <- comb_t_df
      new_samples$t_df_acceptance_rate <- comb_t_df_acceptance_rate
    }

  }

  new_samples
}

#!/usr/bin/Rscript
#' @title Plot sampled vector parameter
#' @description Plot the sampled values for a sampled vector from the output of
#' the ``mixtureModel`` function. Not recommended for large B or P.
#' @param samples The output of the ``mixtureModel`` function.
#' @param parameter The name of the parameter to be plotted (a string).
#' @param R The number of iterations run. Defaults to the number of samples for
#' the cluster membership.
#' @param thin The thinning factor of the sampler. Defaults to 1.
#' @param burn_in The samples at the beginning of the chain to drop. Defaults to 0.
#' @return A ggplot object of the values in each sampled batch mean per iteration.
#' @export
#' @examples
#' # Data in matrix format
#' X <- matrix(c(rnorm(100, 0, 1), rnorm(100, 3, 1)), ncol = 2, byrow = TRUE)
#'
#' # Observed batches represented by integers
#' batch_vec <- sample(seq(1, 3), size = 100, replace = TRUE)
#'
#' # MCMC iterations (this is too low for real use)
#' R <- 50
#' thin <- 1
#'
#' # MCMC samples and BIC vector
#' samples <- runBatchMix(X, R, thin, batch_vec, "MVN", K_max = 8)
#'
#' # Plot the sampled value of the cluster means against MCMC iteration
#' parameter <- "means"
#' plotSampledParameter(samples, parameter, R, thin)
#' @importFrom ggplot2 ggplot aes_string geom_point facet_wrap labs
plotSampledParameter <- function(samples, parameter, R = NULL, thin = 1, burn_in = 0) {
  n_param <- dim(samples[[parameter]])[2]
  P <- dim(samples[[parameter]])[1]

  if (is.null(R)) {
    R <- nrow(samples$samples)
  }

  # Check that the values of R and thin make sense
  if (floor(R / thin) != nrow(samples$samples)) {
    stop("The ratio of R to thin does not match the number of samples present.")
  }

  sampled_parameter <- getSampledBatchShift(samples[[parameter]], n_param, P, R = R, thin = thin)

  sampled_parameter <- sampled_parameter[sampled_parameter$Iteration > burn_in, ]

  p <- ggplot2::ggplot(
    sampled_parameter,
    ggplot2::aes_string(x = "Iteration", y = "value")
  ) +
    ggplot2::geom_point() +
    ggplot2::facet_wrap(~name, ncol = P) +
    ggplot2::labs(
      title = parameter,
      x = "MCMC iteration",
      y = "Sampled value"
    )

  p
}

#!/usr/bin/Rscript
#' @title Predict class
#' @description Predicts a final class for each item given a matrix of
#' allocation probabilities.
#' @param prob Output from the ``calcAllocProb`` function, a N x K matrix of
#' allocation probabilities.
#' @return An N vector of class allocations.
#' @export
#' @examples
#'
#' # Data in a matrix format
#' X <- matrix(c(rnorm(100, 0, 1), rnorm(100, 3, 1)), ncol = 2, byrow = TRUE)
#'
#' # Initial labelling
#' labels <- c(
#'   rep(1, 10),
#'   sample(c(1, 2), size = 40, replace = TRUE),
#'   rep(2, 10),
#'   sample(c(1, 2), size = 40, replace = TRUE)
#' )
#'
#' fixed <- c(rep(1, 10), rep(0, 40), rep(1, 10), rep(0, 40))
#'
#' # Batch
#' batch_vec <- sample(seq(1, 5), replace = TRUE, size = 100)
#'
#' # Sampling parameters
#' R <- 1000
#' thin <- 50
#'
#' # MCMC samples and BIC vector
#' samples <- batchSemiSupervisedMixtureModel(
#'   X,
#'   R,
#'   thin,
#'   labels,
#'   fixed,
#'   batch_vec,
#'   "MVN"
#' )
#'
#' # Burn in
#' burn <- 200
#' eff_burn <- burn / thin
#'
#' # Probability across classes
#' probs <- calcAllocProb(samples, burn = burn)
#'
#' # Predict the class
#' preds <- predictClass(probs)
#'
predictClass <- function(prob) {
  pred_cl <- apply(prob, 1, which.max)

  pred_cl
}

#!/usr/bin/Rscript
#
#' @title Generate batch data from a multivariate t distribution
#' @description Generate data from K multivariate t distributions with
#' additional noise from batches. Assumes independence across columns. In each
#' column the parameters are randomly permuted for both the groups and batches.
#' @param N The number of items (rows) to generate.
#' @param P The number of columns in the generated dataset.
#' @param group_means A vector of the group means for a column.
#' @param group_std_devs A vector of group standard deviations for a column.
#' @param batch_shift A vector of batch means in a column.
#' @param batch_scale A vector of batch standard deviations within a column.
#' @param group_weights A K x B matrix of the expected proportion of N in each group in each batch.
#' @param batch_weights A vector of the expected proportion of N in each batch.
#' @param frac_known The number of items with known labels.
#' @param dfs A K-vector of the group specific degrees of freedom.
#' @return A list of 5 objects; the data generated from the groups with and
#' without batch effects, the label indicating the generating group, the
#' batch label and the vector indicating training versus test.
#' @export
#' @examples
#' N <- 500
#' P <- 2
#' K <- 2
#' B <- 5
#' mean_dist <- 4
#' batch_dist <- 0.3
#' group_means <- seq(1, K) * mean_dist
#' batch_shift <- rnorm(B, mean = batch_dist, sd = batch_dist)
#' std_dev <- rep(2, K)
#' batch_var <- rep(1.2, B)
#' group_weights <- rep(1 / K, K)
#' batch_weights <- rep(1 / B, B)
#' dfs <- c(4, 7)
#' my_data <- generateBatchDataMVT(
#'   N,
#'   P,
#'   group_means,
#'   std_dev,
#'   batch_shift,
#'   batch_var,
#'   group_weights,
#'   batch_weights,
#'   dfs
#' )
#' @importFrom stats rnorm rchisq
generateBatchDataMVT <- function(N,
                                 P,
                                 group_means,
                                 group_std_devs,
                                 batch_shift,
                                 batch_scale,
                                 group_weights,
                                 batch_weights,
                                 dfs,
                                 frac_known = 0.2) {
  # The number of groups to generate
  K <- length(group_means)

  # The number of batches to generate
  B <- length(batch_shift)

  # The membership vector for the N points
  group_IDs <- sample(seq(1, K), N, replace = TRUE, prob = group_weights)

  # The batch labels for the N points
  batch_IDs <- sample(seq(1, B), N, replace = TRUE, prob = batch_weights)

  # Fixed labels
  fixed <- sample(seq(0, 1), N,
    replace = TRUE,
    prob = c(1 - frac_known, frac_known)
  )

  # The data matrices
  observed_data <- true_data <- matrix(nrow = N, ncol = P)

  # Iterate over each of the columns permuting the means associated with each
  # label.
  for (p in seq(1, P))
  {
    # To provide different information in each column, randomly sample the
    # parameters with each group and batch
    reordered_group_means <- sample(group_means)
    reordered_group_std_devs <- sample(group_std_devs)

    reordered_batch_shift <- sample(batch_shift)
    reordered_batch_scale <- sample(batch_scale)

    # Draw n points from the K univariate Gaussians defined by the permuted means.
    for (n in seq(1, N)) {
      # Draw a point from a standard normal
      x <- stats::rnorm(1)
      k <- group_IDs[n]
      b <- batch_IDs[n]

      chi_draw <- stats::rchisq(1, dfs[k])

      # For ease of reading the following lines, create group and batch parameters
      .mu <- reordered_group_means[k]
      .sd <- reordered_group_std_devs[k]
      .m <- reordered_batch_shift[b]
      .s <- reordered_batch_scale[b]

      # Adjust to the group distribution
      true_data[n, p] <- x * .sd * sqrt(dfs[k] / chi_draw) + .mu

      # Adjust to the batched group distribution
      observed_data[n, p] <- x * .sd * .s * sqrt(dfs[k] / chi_draw) + .mu + .m
    }
  }

  # Return the data, the data without batch effects, the allocation labels and
  # the batch labels.
  list(
    observed_data = observed_data,
    corrected_data = true_data,
    group_IDs = group_IDs,
    batch_IDs = batch_IDs,
    fixed = fixed
  )
}

#!/usr/bin/Rscript
#' @title Get sampled cluster means
#' @description Given an array of sampled cluster means from the
#' ``mixtureModel`` function, acquire a tidy version ready for ``ggplot2`` use.
#' @param sampled_cluster_means A 3D array of sampled cluster means.
#' @param K The number of clusters present. Defaults to the number of columns in
#' the batch mean matrix from the first sample.
#' @param P The dimension of the batch mean shifts. Defaults to the number of
#' rows in the batch mean matrix from the first sample.
#' @param R The number of iterations run. Defaults to the number of slices in
#' the sampled batch mean array.
#' @param thin The thinning factor of the sampler. Defaults to 1.
#' @return A data.frame of three columns; the parameter, the sampled value and the iteration.
#' @export
#' @examples
#'
#' # Data in matrix format
#' X <- matrix(c(rnorm(100, 0, 1), rnorm(100, 3, 1)), ncol = 2, byrow = TRUE)
#'
#' # Observed batches represented by integers
#' batch_vec <- sample(seq(1, 5), size = 100, replace = TRUE)
#'
#' # MCMC iterations (this is too low for real use)
#' R <- 100
#' thin <- 5
#'
#' # MCMC samples
#' samples <- runBatchMix(X, R, thin, batch_vec, "MVN")
#'
#' batch_shift_df <- getSampledClusterMeans(samples$means, R = R, thin = thin)
#'
#' @importFrom tidyr pivot_longer contains
getSampledClusterMeans <- function(sampled_cluster_means,
                                   K = dim(sampled_cluster_means)[2],
                                   P = dim(sampled_cluster_means)[1],
                                   R = dim(sampled_cluster_means)[3],
                                   thin = 1) {
  # Check that the values of R and thin make sense
  if (floor(R / thin) != dim(sampled_cluster_means)[3]) {
    stop("The ratio of R to thin does not match the number of samples present.")
  }

  # Stack the sampled matrices on top of each other
  mean_df <- data.frame(t(apply(sampled_cluster_means, 3L, rbind)))

  # Indices over columns and batches
  col_inds <- seq(1, P)
  group_inds <- seq(1, K)

  # Give sensible column names
  colnames(mean_df) <- paste0(
    "Mu_",
    sort(as.numeric(levels(interaction(group_inds, col_inds, sep = ""))))
  )

  # Add a variable for the iteration the sample comes from
  iterations <- seq(1, R / thin) * thin
  mean_df$Iteration <- iterations

  # Pivot to a long format ready for ``ggplot2``
  mean_long_df <- tidyr::pivot_longer(mean_df, tidyr::contains("Mu_"))
  mean_long_df$Dimension <- rep(col_inds, nrow(mean_long_df) / P)
  mean_long_df$Cluster <- rep(group_inds, nrow(mean_long_df) / (K * P),
    each = P
  )

  mean_long_df
}

#' @title Continue chains
#' @description Continues sampling from a list of previous chains.
#' @param mcmc_output Chains to be continued.
#' @param X Data to cluster as a matrix with the items to cluster held in rows.
#' @param fixed The indicator vector for which labels are observed.
#' @param batch_vec The vector of the batch labels for the data.
#' @param R The number of iterations to run in this continuation (thinning
#' factor is the same as initial chain).
#' @param keep_old_samples Logical indicating if the original samples should be
#' kept or only the new samples returned. Defaults to TRUE.
#' @return A named list containing the sampled partitions, cluster and batch
#' parameters, model fit measures and some details on the model call.
#' @export
#' @examples
#'
#' # Data in a matrix format
#' X <- matrix(c(rnorm(100, 0, 1), rnorm(100, 3, 1)), ncol = 2, byrow = TRUE)
#'
#' # Initial labelling
#' labels <- c(
#'   rep(1, 10),
#'   sample(c(1, 2), size = 40, replace = TRUE),
#'   rep(2, 10),
#'   sample(c(1, 2), size = 40, replace = TRUE)
#' )
#'
#' fixed <- c(rep(1, 10), rep(0, 40), rep(1, 10), rep(0, 40))
#'
#' # Batch
#' batch_vec <- sample(seq(1, 5), replace = TRUE, size = 100)
#'
#' # Density choice
#' type <- "MVT"
#'
#' # Sampling parameters
#' R <- 1000
#' thin <- 50
#' n_chains <- 4
#'
#' # MCMC samples
#' mcmc_output <- runMCMCChains(
#'   X,
#'   n_chains,
#'   R,
#'   thin,
#'   batch_vec,
#'   type,
#'   initial_labels = labels,
#'   fixed = fixed
#' )
#'
#' # Given an initial value for the parameters
#' new_output <- continueChains(
#'   mcmc_output,
#'   X,
#'   fixed,
#'   batch_vec,
#'   R,
#'   keep_old_samples = TRUE
#' )
continueChains <- function(mcmc_output,
                           X,
                           fixed,
                           batch_vec,
                           R,
                           keep_old_samples = TRUE) {
  new_output <- lapply(
    mcmc_output,
    continueChain,
    X,
    fixed,
    batch_vec,
    R,
    keep_old_samples
  )

  n_chains <- length(mcmc_output)

  # Record chain number
  for (ii in seq(n_chains)) {
    new_output[[ii]]$Chain <- mcmc_output[[ii]]$Chain
  }


  new_output
}

#!/usr/bin/Rscript
#' @title Process MCMC chains
#' @description Applies a burn in to and finds a point estimate for each of the
#' chains outputted from ``runMCMCChains``.
#' @param mcmc_lst Output from ``runMCMCChains``
#' @param burn The number of MCMC samples to drop as part of a burn in.
#' @param point_estimate_method Summary statistic used to define the point
#' estimate. Must be ``'mean'`` or ``'median'``. ``'median'`` is the default.
#' @returns A named list similar to the output of
#' ``batchSemiSupervisedMixtureModel`` with some additional entries:
#'
#'  * ``mean_est``:  $(P x K)$ matrix. The point estimate of the cluster
#'  means with columns  corresponding to clusters.
#'
#'  * ``cov_est``:  $(P x P x K)$ array. The point estimate of the
#'  cluster covariance matrices with slices corresponding to clusters.
#'
#'  * ``shift_est``:  $(P x B)$ matrix. The point estimate of the batch
#'  shift effect with columns  corresponding to batches.
#'
#'  * ``scale_est``:  $(P x B)$ matrix. The point estimate of the batch
#'  scale effects. The $bth$ column contains the diagonal entries of the scaling
#'  matrix for the $bth£ batch.
#'
#'  * ``mean_sum_est``:  $(P x K x B)$ array. The point estimate of the
#'  sum of the cluster  means and the batch shift effect with columns
#'  corresponding to clusters and slices to batches.
#'
#'  * ``cov_comb_est``:  List of length $B$, with each entry being a
#'  $(P x P x K)$ array. The point estimate of the combination of the
#'  cluster covariance matrices and the batch scale effect with list entries
#'  corresponding to batches and slices of each array corresponding to clusters.
#'
#'  * ``inferred_dataset``:  $(N x P)$ matrix. The inferred ``batch-free''
#'  dataset.
#'
#'  * ``allocation_probability``:  $(N x K)$ matrix. The point estimate of
#'  the allocation probabilities for each data point to each class.
#'
#'  * ``prob``:  $N$ vector. The point estimate of the probability of being
#'  allocated to the class with the highest probability.
#'
#'  * ``pred``:  $N$ vector. The predicted class for each sample.
#'
#' @export
#' @examples
#'
#' # Data in a matrix format
#' X <- matrix(c(rnorm(100, 0, 1), rnorm(100, 3, 1)), ncol = 2, byrow = TRUE)
#'
#' # Initial labelling
#' labels <- c(
#'   rep(1, 10),
#'   sample(c(1, 2), size = 40, replace = TRUE),
#'   rep(2, 10),
#'   sample(c(1, 2), size = 40, replace = TRUE)
#' )
#'
#' fixed <- c(rep(1, 10), rep(0, 40), rep(1, 10), rep(0, 40))
#'
#' # Batch
#' batch_vec <- sample(seq(1, 5), replace = TRUE, size = 100)
#'
#' # Sampling parameters
#' R <- 1000
#' burn <- 250
#' thin <- 50
#' n_chains <- 4
#'
#' # MCMC samples
#' samples <- runMCMCChains(X, n_chains, R, thin, batch_vec, "MVN",
#'   initial_labels = labels,
#'   fixed = fixed
#' )
#'
#' # Process the MCMC samples
#' processed_samples <- processMCMCChains(samples, burn)
#'
processMCMCChains <- function(mcmc_lst, burn, point_estimate_method = "median") {
  new_output <- lapply(mcmc_lst, processMCMCChain, burn, point_estimate_method)

  # Return the MCMC object with burn in applied and point estimates found
  new_output
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

#' @title Create Similarity Matrix
#' @description Constructs a similarity matrix of the pairwise coclustering 
#' rate.
#' @param allocations Matrix of sampled partitions. Columns correspond to 
#' items/samples being clustered, each row is a sampled partition.//' 
#' @return A symmetric n x n matrix (for n rows in cluster record) describing 
#' the fraction of iterations for which each pairwise combination of points are
#' assigned the same label.
#' @export
createSimilarityMat <- function(allocations) {
    .Call('_batchmix_createSimilarityMat', PACKAGE = 'batchmix', allocations)
}

#' title Propose new non-negative value
#' description Propose new non-negative for sampling.
#' param x Current value to be proposed
#' param window The proposal window
#' return new double
NULL

#' title The Inverse Gamma Distribution
#' description Random generation from the inverse Gamma distribution.
#' param shape Shape parameter.
#' param rate Rate parameter.
#' return Sample from invGamma(shape, rate).
NULL

#' title The Inverse Gamma Distribution
#' description Random generation from the inverse Gamma distribution.
#' param N Number of samples to draw.
#' param shape Shape parameter.
#' param rate Rate parameter.
#' return Sample from invGamma(shape, rate).
NULL

#' title The Gamma Distribution
#' description Random generation from the Gamma distribution.
#' param shape Shape parameter.
#' param rate Rate parameter.
#' return Sample from Gamma(shape, rate).
NULL

#' title The Gamma Distribution
#' description Random generation from the Gamma distribution.
#' param N Number of samples to draw.
#' param shape Shape parameter.
#' param rate Rate parameter.
#' return N samples from Gamma(shape, rate).
NULL

#' title The Beta Distribution
#' description Random generation from the Beta distribution.
#' See https://en.wikipedia.org/wiki/Beta_distribution#Related_distributions.
#' Samples from a Beta distribution based using two independent gamma
#' distributions.
#' param a Shape parameter.
#' param b Shape parameter.
#' return Sample from Beta(a, b).
NULL

#' title The Beta Distribution
#' description Random generation from the Beta distribution.
#' See https://en.wikipedia.org/wiki/Beta_distribution#Related_distributions.
#' Samples from a Beta distribution based using two independent gamma
#' distributions.
#' param n The number of samples to draw.
#' param a Shape parameter.
#' param b Shape parameter.
#' return Sample from Beta(a, b).
NULL

#' title Metropolis acceptance step
#' description Given a probaility, randomly accepts by sampling from a uniform 
#' distribution.
#' param acceptance_prob Double between 0 and 1.
#' return Boolean indicating acceptance.
NULL

#' title Sample mean
#' description calculate the sample mean of a matrix X.
#' param X Matrix
#' return Vector of the column means of X.
NULL

#' title Calculate sample covariance
#' description Returns the unnormalised sample covariance. Required as
#' arma::cov() does not work for singletons.
#' param data Data in matrix format
#' param sample_mean Sample mean for data
#' param n The number of samples in data
#' param n_col The number of columns in data
#' return One of the parameters required to calculate the posterior of the
#'  Multivariate normal with uknown mean and covariance (the unnormalised
#'  sample covariance).
NULL

#' @title Gamma log-likelihood
#' @description Used in calculating model probability in Metropolis-Hastings 
#' algorithm when proposals are from the Gamma distribution.
#' @param x - double; the value to calculate the unnormalised likelihood of.
#' @param shape - double; the shape of the Gamma distribution.
#' @param rate - double; the rate of the Gamma distribution
#' @return the unnormalised log-likelihood of x in a Gamma with parameters shape 
#' and rate.
gammaLogLikelihood <- function(x, shape, rate) {
    .Call('_batchmix_gammaLogLikelihood', PACKAGE = 'batchmix', x, shape, rate)
}

#' @title Inverse gamma log-likelihood
#' @description Used in calculating model probability in Metropolis-Hastings 
#' algorithm when proposals are from the inverse-Gamma distribution.
#' @param x - double; the value to calculate the likelihood of.
#' @param shape - double; the shape of the inverse-Gamma distribution.
#' @param scale - double; the scale of the inverse-Gamma distribution
#' @return the unnormalised log-likelihood of x in a inverse-Gamma with parameters 
#' shape and scale.
invGammaLogLikelihood <- function(x, shape, scale) {
    .Call('_batchmix_invGammaLogLikelihood', PACKAGE = 'batchmix', x, shape, scale)
}

#' @title Wishart log-likelihood
#' @description Used in calculating model probability in Metropolis-Hastings 
#' algorithm when proposals are from the Wishart distribution.
#' @param X - matrix; the matrix to calculate the likelihood of.
#' @param V - matrix; the scale of the Wishart distribution.
#' @param n - double; the degrees of freedom for the Wishart distribution.
#' @param P - unsigned integer; the dimension of X.
#' @return the unnormalised log-likelihood of X in a Wishart with parameters V 
#' and n.
wishartLogLikelihood <- function(X, V, n, P) {
    .Call('_batchmix_wishartLogLikelihood', PACKAGE = 'batchmix', X, V, n, P)
}

#' @title Inverse-Wishart log-likelihood
#' @description Used in calculating model probability in Metropolis-Hastings 
#' algorithm when proposals are from the Wishart distribution.
#' @param X - matrix; the matrix to calculate the likelihood of.
#' @param Psi - matrix; the scale of the inverse-Wishart distribution.
#' @param nu - double; the degrees of freedom for the inverse-Wishart distribution.
#' @param P - unsigned integer; the dimension of X.
#' @return the unnormalised log-likelihood of X in a inverse-Wishart with parameters Psi 
#' and nu.
invWishartLogLikelihood <- function(X, Psi, nu, P) {
    .Call('_batchmix_invWishartLogLikelihood', PACKAGE = 'batchmix', X, Psi, nu, P)
}

#' @title Sample mixture of multivariate normal distributions with batch effects
#' @description Performs MCMC sampling for a mixture model with batch effects.
#' @param X The data matrix to perform clustering upon (items to cluster in rows).
#' @param K The number of components to model (upper limit on the number of 
#' clusters found).
#' @param B The number of batches to model.
#' @param labels Vector item labels to initialise from.
#' @param batch_vec Observed batch labels.
#' @param mu_proposal_window The standard deviation for the Gaussian proposal
#' density of the cluster means.
#' @param cov_proposal_window The degrees of freedom for the Wishart proposal
#' density of the cluster covariances.
#' @param m_proposal_window The standard deviation for the Gaussian proposal
#' density of the batch mean effects.
#' @param S_proposal_window The rate for the Gamma proposal density of the 
#' batch scale.
#' @param R The number of iterations to run for.
#' @param thin thinning factor for samples recorded.
#' @param concentration Vector of concentrations for mixture weights
#' (recommended to be symmetric).
#' @param m_scale The scale hyperparameter for the batch shift prior 
#' distribution.
#' @param rho The shape of the prior distribution for the batch scale.
#' @param theta The scale of the prior distribution for the batch scale.
#' @param initial_mu A P x K matrix of initial values for the class means.
#' @param initial_cov A P x P x K cube of initial values for the class 
#' covariance matrices.
#' @param initial_m A P x B matrix of initial values for the batch shift 
#' effects.
#' @param initial_S A P x B matrix of initial values for the batch scales.
#' @param mu_initialised Bool indicating if the class means are initialised by
#' the user. If ``false`` then initial values are drawn from the prior 
#' distribution.
#' @param cov_initialised Bool indicating if the class covariance matrices are 
#' initialised by the user. If ``false`` then initial values are drawn from the
#' prior distribution.
#' @param m_initialised Bool indicating if the batch shift effects are 
#' initialised by the user. If ``false`` then initial values are drawn from the
#' prior distribution.
#' @param S_initialised Bool indicating if the batch scales are initialised by 
#' the user. If ``false`` then initial values are drawn from the prior 
#' distribution.
#' @param sample_m_scale Bool indicating if the hyperparameter on the batch 
#' shift effect is sampled or given as fixed.
#' @return Named list of the different quantities drawn by the sampler.
sampleMVN <- function(X, K, B, labels, batch_vec, mu_proposal_window, cov_proposal_window, m_proposal_window, S_proposal_window, R, thin, concentration, m_scale, rho, theta, initial_mu, initial_cov, initial_m, initial_S, mu_initialised, cov_initialised, m_initialised, S_initialised, sample_m_scale) {
    .Call('_batchmix_sampleMVN', PACKAGE = 'batchmix', X, K, B, labels, batch_vec, mu_proposal_window, cov_proposal_window, m_proposal_window, S_proposal_window, R, thin, concentration, m_scale, rho, theta, initial_mu, initial_cov, initial_m, initial_S, mu_initialised, cov_initialised, m_initialised, S_initialised, sample_m_scale)
}

#' @title Sample mixture of multivariate t-distributions with batch effects
#' @description Performs MCMC sampling for a MVT mixture model with batch effects.
#' @param X The data matrix to perform clustering upon (items to cluster in rows).
#' @param K The number of components to model (upper limit on the number of 
#' clusters found).
#' @param B The number of batches to model.
#' @param labels Vector item labels to initialise from.
#' @param batch_vec Observed batch labels.
#' @param mu_proposal_window The standard deviation for the Gaussian proposal 
#' density of the cluster means.
#' @param cov_proposal_window The degrees of freedom for the Wishart proposal 
#' density of the cluster covariances.
#' @param m_proposal_window The standard deviation for the Gaussian proposal 
#' density of the batch mean effects.
#' @param S_proposal_window The rate for the Gamma proposal density of the 
#' batch scale.
#' @param t_df_proposal_window The rate for the Gamma proposal density of the 
#' cluster degrees of freedom.
#' @param R The number of iterations to run for.
#' @param thin thinning factor for samples recorded.
#' @param concentration Vector of concentrations for mixture weights 
#' (recommended to be symmetric).
#' @param m_scale The scale hyperparameter for the batch shift prior 
#' distribution.
#' @param rho The shape of the prior distribution for the batch scale.
#' @param theta The scale of the prior distribution for the batch scale.
#' @param initial_mu A P x K matrix of initial values for the class means.
#' @param initial_cov A P x P x K cube of initial values for the class 
#' covariance matrices.
#' @param initial_df A K vector of initial values for the class degrees of
#' freedom.
#' @param initial_m A P x B matrix of initial values for the batch shift 
#' effects.
#' @param initial_S A P x B matrix of initial values for the batch scales.
#' @param mu_initialised Bool indicating if the class means are initialised by
#' the user. If ``false`` then initial values are drawn from the prior 
#' distribution.
#' @param cov_initialised Bool indicating if the class covariance matrices are 
#' initialised by the user. If ``false`` then initial values are drawn from the
#' prior distribution.
#' @param df_initialised Bool indicating if the class degrees of freedom are 
#' initialised by the user. If ``false`` then initial values are drawn from the 
#' prior distribution.
#' @param m_initialised Bool indicating if the batch shift effects are 
#' initialised by the user. If ``false`` then initial values are drawn from the
#' prior distribution.
#' @param S_initialised Bool indicating if the batch scales are initialised by 
#' the user. If ``false`` then initial values are drawn from the prior 
#' distribution.
#' @param sample_m_scale Bool indicating if the hyperparameter on the batch 
#' shift effect is sampled or given as fixed.
#' @return Named list of the different quantities drawn by the sampler.
sampleMVT <- function(X, K, B, labels, batch_vec, mu_proposal_window, cov_proposal_window, m_proposal_window, S_proposal_window, t_df_proposal_window, R, thin, concentration, m_scale, rho, theta, initial_mu, initial_cov, initial_df, initial_m, initial_S, mu_initialised, cov_initialised, df_initialised, m_initialised, S_initialised, sample_m_scale) {
    .Call('_batchmix_sampleMVT', PACKAGE = 'batchmix', X, K, B, labels, batch_vec, mu_proposal_window, cov_proposal_window, m_proposal_window, S_proposal_window, t_df_proposal_window, R, thin, concentration, m_scale, rho, theta, initial_mu, initial_cov, initial_df, initial_m, initial_S, mu_initialised, cov_initialised, df_initialised, m_initialised, S_initialised, sample_m_scale)
}

#' @title Sample semi-supervised MVN Mixture model
#' @description Performs MCMC sampling for a mixture model.
#' @param X The data matrix to perform clustering upon (items to cluster in
#' rows).
#' @param K The number of components to model (upper limit on the number of
#' clusters found).
#' @param B The number of batches to model.
#' @param labels Vector item labels to initialise from.
#' @param batch_vec Observed batch labels.
#' @param fixed Binary vector of the items that are fixed in their initial
#' label.
#' @param mu_proposal_window The standard deviation for the Gaussian proposal
#'  density of the cluster means.
#' @param cov_proposal_window The degrees of freedom for the Wishart proposal
#'  density of the cluster covariances.
#' @param m_proposal_window The standard deviation for the Gaussian proposal
#'  density of the batch mean effects.
#' @param S_proposal_window The rate for the Gamma proposal density of the batch scale.
#' @param R The number of iterations to run for.
#' @param thin thinning factor for samples recorded.
#' @param concentration Vector of concentrations for mixture weights 
#' (recommended to be symmetric).
#' @param m_scale The scale hyperparameter for the batch shift prior 
#' distribution.
#' @param rho The shape of the prior distribution for the batch scale.
#' @param theta The scale of the prior distribution for the batch scale.
#' @param initial_mu A P x K matrix of initial values for the class means.
#' @param initial_cov A P x P x K cube of initial values for the class 
#' covariance matrices.
#' @param initial_m A P x B matrix of initial values for the batch shift 
#' effects.
#' @param initial_S A P x B matrix of initial values for the batch scales.
#' @param mu_initialised Bool indicating if the class means are initialised by
#' the user. If ``false`` then initial values are drawn from the prior 
#' distribution.
#' @param cov_initialised Bool indicating if the class covariance matrices are 
#' initialised by the user. If ``false`` then initial values are drawn from the
#' prior distribution.
#' @param m_initialised Bool indicating if the batch shift effects are 
#' initialised by the user. If ``false`` then initial values are drawn from the
#' prior distribution.
#' @param S_initialised Bool indicating if the batch scales are initialised by 
#' the user. If ``false`` then initial values are drawn from the prior 
#' distribution.
#' @param sample_m_scale Bool indicating if the hyperparameter on the batch 
#' shift effect is sampled or given as fixed.
#' @return Named list of the different quantities drawn by the sampler.
sampleSemisupervisedMVN <- function(X, K, B, labels, batch_vec, fixed, mu_proposal_window, cov_proposal_window, m_proposal_window, S_proposal_window, R, thin, concentration, m_scale, rho, theta, initial_mu, initial_cov, initial_m, initial_S, mu_initialised, cov_initialised, m_initialised, S_initialised, sample_m_scale) {
    .Call('_batchmix_sampleSemisupervisedMVN', PACKAGE = 'batchmix', X, K, B, labels, batch_vec, fixed, mu_proposal_window, cov_proposal_window, m_proposal_window, S_proposal_window, R, thin, concentration, m_scale, rho, theta, initial_mu, initial_cov, initial_m, initial_S, mu_initialised, cov_initialised, m_initialised, S_initialised, sample_m_scale)
}

#' @title Sample semi-supervised MVT Mixture model
#' @description Performs MCMC sampling for a mixture model.
#' @param X The data matrix to perform clustering upon (items to cluster in rows).
#' @param K The number of components to model (upper limit on the number of clusters found).
#' @param B The number of batches to model.
#' @param labels Vector item labels to initialise from.
#' @param batch_vec Observed batch labels.
#' @param fixed Binary vector of the items that are fixed in their initial label.
#' @param mu_proposal_window The standard deviation for the Gaussian proposal density of the cluster means.
#' @param cov_proposal_window The degrees of freedom for the Wishart proposal density of the cluster covariances.
#' @param m_proposal_window The standard deviation for the Gaussian proposal density of the batch mean effects.
#' @param S_proposal_window The rate for the Gamma proposal density of the batch scale.
#' @param t_df_proposal_window The rate for the Gamma proposal density of the cluster degrees of freedom.
#' @param R The number of iterations to run for.
#' @param thin thinning factor for samples recorded.
#' @param concentration Vector of concentrations for mixture weights (recommended to be symmetric).
#' @param m_scale The scale hyperparameter for the batch shift prior 
#' distribution.
#' @param rho The shape of the prior distribution for the batch scale.
#' @param theta The scale of the prior distribution for the batch scale.
#' @param initial_mu A P x K matrix of initial values for the class means.
#' @param initial_cov A P x P x K cube of initial values for the class 
#' covariance matrices.
#' @param initial_df A K vector of initial values for the class degrees of
#' freedom.
#' @param initial_m A P x B matrix of initial values for the batch shift 
#' effects.
#' @param initial_S A P x B matrix of initial values for the batch scales.
#' @param mu_initialised Bool indicating if the class means are initialised by
#' the user. If ``false`` then initial values are drawn from the prior 
#' distribution.
#' @param cov_initialised Bool indicating if the class covariance matrices are 
#' initialised by the user. If ``false`` then initial values are drawn from the
#' prior distribution.
#' @param df_initialised Bool indicating if the class degrees of freedom are 
#' initialised by the user. If ``false`` then initial values are drawn from the 
#' prior distribution.
#' @param m_initialised Bool indicating if the batch shift effects are 
#' initialised by the user. If ``false`` then initial values are drawn from the
#' prior distribution.
#' @param S_initialised Bool indicating if the batch scales are initialised by 
#' the user. If ``false`` then initial values are drawn from the prior 
#' distribution.
#' @param sample_m_scale Bool indicating if the hyperparameter on the batch 
#' shift effect is sampled or given as fixed.
#' @return Named list of the different quantities drawn by the sampler.
sampleSemisupervisedMVT <- function(X, K, B, labels, batch_vec, fixed, mu_proposal_window, cov_proposal_window, m_proposal_window, S_proposal_window, t_df_proposal_window, R, thin, concentration, m_scale, rho, theta, initial_mu, initial_cov, initial_df, initial_m, initial_S, mu_initialised, cov_initialised, df_initialised, m_initialised, S_initialised, sample_m_scale) {
    .Call('_batchmix_sampleSemisupervisedMVT', PACKAGE = 'batchmix', X, K, B, labels, batch_vec, fixed, mu_proposal_window, cov_proposal_window, m_proposal_window, S_proposal_window, t_df_proposal_window, R, thin, concentration, m_scale, rho, theta, initial_mu, initial_cov, initial_df, initial_m, initial_S, mu_initialised, cov_initialised, df_initialised, m_initialised, S_initialised, sample_m_scale)
}

#' @title Generate batch data
#' @description Generate data from K multivaraite normal or multivariate t
#' distributions with additional noise from batches. Assumes independence across
#' columns. In each column the parameters are randomly permuted for both the
#' groups and batches.
#' @param N The number of items (rows) to generate.
#' @param P The number of columns in the generated dataset.
#' @param group_rates A vector of the group rates for the classes within a column.
#' @param batch_rates A vector of the batch rates for the classes within a column.
#' This is used to create a variable which has the sum of the appropriate batch
#' and class rate, it might be better interpreted as the batch effect on the
#' observed rate.
#' @param group_weights One of either a K x B matrix of the expected proportion
#' of each batch in each group or a K-vector of the expected proportion of the
#' entire dataset in each group.
#' @param batch_weights A vector of the expected proportion of N in each batch.
#' @param frac_known The number of items with known labels.
#' @param permute_variables Logical indicating if group and batch means and
#' standard deviations should be permuted in each column or not (defaults to
#' ``TRUE``).
#' @param scale_data Logical indicating if data should be mean centred and
#' standardised (defaults to ``FALSE``).
#' @return A list of 5 objects; the data generated from the groups with and
#' without batch effects, the label indicating the generating group, the
#' batch label and the vector indicating training versus test.
#' @export
generateBatchDataLogPoisson <- function(N,
                                        P,
                                        group_rates,
                                        batch_rates,
                                        group_weights,
                                        batch_weights,
                                        frac_known = 0.2,
                                        permute_variables = TRUE,
                                        scale_data = FALSE) {
  # The number of batches and groups to generate
  B <- length(batch_rates)
  K <- length(group_rates)

  # Allow for varying groups within batches
  varying_group_within_batch <- is.matrix(group_weights)

  # The batch labels for the N points
  batch_IDs <- sample(seq(1, B), N, replace = TRUE, prob = batch_weights)

  # Generate group membership, potentially allowing imbalance across batches
  group_IDs <- generateGroupIDsInSimulator(
    N,
    K,
    B,
    batch_IDs,
    group_weights,
    varying_group_within_batch
  )

  # Fixed labels
  fixed <- sample(seq(0, 1), N,
    replace = TRUE,
    prob = c(1 - frac_known, frac_known)
  )

  classes <- unique(group_IDs)
  batches <- unique(batch_IDs)
  observed_data <- true_data <- matrix(nrow = N, ncol = P)

  lambdas_pk <- group_rates
  lambdas_pb <- batch_rates

  for (p in seq(1, P)) {
    if (permute_variables) {
      lambdas_pk <- sample(group_rates)
      lambdas_pb <- sample(batch_rates)
    }
    for (b in batches) {
      batch_inds <- which(batch_IDs == b)
      for (k in classes) {
        class_in_batch_inds <- which((group_IDs == k) & (batch_IDs == b))
        n_kb <- length(class_in_batch_inds)
        if (n_kb > 0) {
          batch_free_count <- rpois(n_kb, lambdas_pk[k])
          batch_effect_on_count <- rpois(n_kb, lambdas_pb[b])
          observed_count <- batch_free_count + batch_effect_on_count
          normal_noise <- rnorm(n_kb)
          observed_data[class_in_batch_inds, p] <- log(1.0 + observed_count) + normal_noise
          true_data[class_in_batch_inds, p] <- log(1.0 + batch_free_count) + normal_noise
        }
      }
    }
  }
  row.names(true_data) <- row.names(observed_data) <- paste0("Person_", seq(1, N))

  if (scale_data) {
    observed_data <- scale(observed_data)
    true_data <- scale(true_data)
  }

  # Return the data, the data without batch effects, the allocation labels and
  # the batch labels.
  list(
    observed_data = observed_data,
    corrected_data = true_data,
    group_IDs = group_IDs,
    batch_IDs = batch_IDs,
    fixed = fixed
  )
}

#!/usr/bin/Rscript
#' @title Calculate allocation probabilities
#' @description Calculate the empirical allocation probability for each class
#' based on the sampled allocation probabilities.
#' @param mcmc_samples Output from ``batchSemiSupervisedMixtureModel``.
#' @param burn The number of samples to discard.
#' @param method The point estimate to use. ``method = 'mean'`` or
#' ``method = 'median'``. ``'median'`` is the default.
#' @return An N x K matrix of class probabilities.
#' @export
#' @examples
#'
#' # Data in matrix format
#' X <- matrix(c(rnorm(100, 0, 1), rnorm(100, 3, 1)), ncol = 2, byrow = TRUE)
#'
#' # Initial labelling
#' labels <- c(
#'   rep(1, 10),
#'   sample(c(1, 2), size = 40, replace = TRUE),
#'   rep(2, 10),
#'   sample(c(1, 2), size = 40, replace = TRUE)
#' )
#'
#' fixed <- c(rep(1, 10), rep(0, 40), rep(1, 10), rep(0, 40))
#'
#' # Batch
#' batch_vec <- sample(seq(1, 5), replace = TRUE, size = 100)
#'
#' # Sampling parameters
#' R <- 1000
#' thin <- 50
#'
#' # MCMC samples and BIC vector
#' samples <- batchSemiSupervisedMixtureModel(X, R, thin, labels, fixed, batch_vec, "MVN")
#'
#' # Burn in
#' burn <- 20
#' eff_burn <- burn / thin
#'
#' # Probability across classes
#' probs <- calcAllocProb(samples, burn = burn)
#'
calcAllocProb <- function(mcmc_samples, burn = 0, method = "median") {
  R <- mcmc_samples$R
  thin <- mcmc_samples$thin
  .alloc <- mcmc_samples$alloc

  if (burn > 0) {
    if (burn > R) {
      stop("Burn in exceeds number of iterations run.")
    }

    eff_burn <- floor(burn / thin)
    dropped_samples <- seq(1, eff_burn)
    .alloc <- .alloc[, , -dropped_samples]
  }

  probs <- NULL

  if (method == "median") {
    probs <- apply(.alloc, c(1, 2), median)
  }
  if (method == "mean") {
    probs <- rowSums(.alloc, dims = 2) / dim(.alloc)[3]
  }
  if (length(probs) == 1) {
    if (is.null(probs)) {
      stop("``method`` must be one of 'mean' or 'median'")
    }
  }
  probs
}

#!/usr/bin/Rscript
#' @title Run MCMC Chains
#' @description Run multiple chains of the batch mixture model of the same type.
#' @param X Data to cluster as a matrix with the items to cluster held in rows.
#' @param n_chains Integer. Number of MCMC chains to run.
#' @param batch_vec Labels identifying which batch each item being clustered is
#' from.
#' @param R The number of iterations in the sampler.
#' @param thin The factor by which the samples generated are thinned, e.g. if
#' ``thin=50`` only every 50th sample is kept.
#' @param type Character indicating density type to use. One of 'MVN'
#' (multivariate normal distribution) or 'MVT' (multivariate t distribution).
#' ``weights`` which is a matrix with K x B columns. The columns are ordered by
#' batch, i.e. the first K columns contain the class weights in the first batch,
#' the second K are the class weights in the second batch, etc. If generic
#' weights are used then this matrix has K columns, one for each component weight.
#' @param K_max The number of components to include (the upper bound on the
#' number of clusters in each sample). Defaults to the number of unique labels
#' in ``initial_labels``.
#' @param initial_labels Initial clustering, if none given defaults to a random draw.
#' @param fixed Which items are fixed in their initial label. If not given,
#' defaults to a vector of 0 meaning the model is run unsupervised.
#' @param alpha The concentration parameter for the stick-breaking prior and the
#' weights in the model.
#' @param mu_proposal_window The proposal window for the cluster mean proposal
#' kernel. The proposal density is a Gaussian distribution, the window is the
#' variance.
#' @param cov_proposal_window The proposal window for the cluster covariance
#' proposal kernel. The proposal density is a Wishart distribution, this
#' argument is the reciprocal of the degree of freedom.
#' @param m_proposal_window The proposal window for the batch mean proposal
#'  kernel. The proposal density is a Gaussian distribution, the window is the
#' variance.
#' @param S_proposal_window The proposal window for the batch standard deviation
#'  proposal kernel. The proposal density is a Gamma distribution, this
#' argument is the reciprocal of the rate.
#' @param t_df_proposal_window The proposal window for the degrees of freedom
#' for the multivariate t distribution (not used if type is not 'MVT'). The
#' proposal density is a Gamma distribution, this argument is the reciprocal of
#' the rate.
#' @param m_scale The scale hyperparameter for the batch shift prior
#' distribution. This defines the scale of the batch effect upon the mean and
#' should be in (0, 1].
#' @param rho The shape of the prior distribution for the batch scale.
#' @param theta The scale of the prior distribution for the batch scale.
#' @param initial_class_means A $P x K$ matrix of initial values for the class
#' means. Defaults to draws from the prior distribution.
#' @param initial_class_covariance A $P x P x K$ array of initial values for
#' the class covariance matrices. Defaults to draws from the prior distribution.
#' @param initial_batch_shift A $P x B$ matrix of initial values for the batch
#' shift effect Defaults to draws from the prior distribution.
#' @param initial_batch_scale A $P x B$ matrix of initial values for the batch
#' scales Defaults to draws from the prior distribution.
#' @param initial_class_df A $K$ vector of initial values for the class degrees
#' of freedom. Defaults to draws from the prior distribution.
#' @param verbose Logiccal indicating if warning about proposal windows should
#' be printed.
#' @returns A list of named lists. Each entry is the output of
#' ``runBatchMix``.
#' @export
#' @examples
#'
#' # Data in a matrix format
#' X <- matrix(c(rnorm(100, 0, 1), rnorm(100, 3, 1)), ncol = 2, byrow = TRUE)
#'
#' # Initial labelling
#' labels <- c(
#'   rep(1, 10),
#'   sample(c(1, 2), size = 40, replace = TRUE),
#'   rep(2, 10),
#'   sample(c(1, 2), size = 40, replace = TRUE)
#' )
#'
#' fixed <- c(rep(1, 10), rep(0, 40), rep(1, 10), rep(0, 40))
#'
#' # Batch
#' batch_vec <- sample(seq(1, 5), replace = TRUE, size = 100)
#'
#' # Sampling parameters
#' R <- 1000
#' thin <- 50
#' n_chains <- 4
#'
#' # MCMC samples
#' samples <- runMCMCChains(X, n_chains, R, thin, batch_vec, "MVN",
#'   initial_labels = labels,
#'   fixed = fixed
#' )
#'
runMCMCChains <- function(X,
                          n_chains,
                          R,
                          thin,
                          batch_vec,
                          type,
                          K_max = NULL,
                          initial_labels = NULL,
                          fixed = NULL,
                          alpha = 1,
                          mu_proposal_window = 0.5**2,
                          cov_proposal_window = 0.002,
                          m_proposal_window = 0.3**2,
                          S_proposal_window = 0.01,
                          t_df_proposal_window = 0.015,
                          m_scale = 0.01,
                          rho = 3.0,
                          theta = 1.0,
                          initial_class_means = NULL,
                          initial_class_covariance = NULL,
                          initial_batch_shift = NULL,
                          initial_batch_scale = NULL,
                          initial_class_df = NULL,
                          verbose = TRUE) {
  mcmc_lst <- vector("list", n_chains)

  mcmc_lst <- lapply(mcmc_lst, function(x) {
    runBatchMix(X,
      R,
      thin,
      batch_vec,
      type,
      K_max = K_max,
      initial_labels = initial_labels,
      fixed = fixed,
      alpha = alpha,
      mu_proposal_window = mu_proposal_window,
      cov_proposal_window = cov_proposal_window,
      m_proposal_window = m_proposal_window,
      S_proposal_window = S_proposal_window,
      t_df_proposal_window = t_df_proposal_window,
      m_scale = m_scale,
      rho = rho,
      theta = theta,
      initial_class_means = initial_class_means,
      initial_class_covariance = initial_class_covariance,
      initial_batch_shift = initial_batch_shift,
      initial_batch_scale = initial_batch_scale,
      initial_class_df = initial_class_df,
      verbose = verbose
    )
  })

  # Record chain number
  for (ii in seq(n_chains)) {
    mcmc_lst[[ii]]$Chain <- ii
  }

  mcmc_lst
}

#!/usr/bin/Rscript
#' @title Get likelihood
#' @description Extracts the model fit score from the mixture model output.
#' @param mcmc_output The output from the mixture model.
#' @param choice The model fit score to use. Must be one of
#' ``'observed_likelihood'``, ``'complete_likelihood'`` or ``'BIC'``. Defaults
#' to ``'complete_likelihood'``.
#' @return A data.frame containing the model fit score of choice and the
#' iteration.
#' @export
#' @examples
#'
#' # Data in a matrix format
#' X <- matrix(c(rnorm(100, 0, 1), rnorm(100, 3, 1)), ncol = 2, byrow = TRUE)
#'
#' # Batch
#' batch_vec <- sample(seq(1, 5), replace = TRUE, size = 100)
#'
#' # Sampling parameters
#' R <- 100
#' thin <- 5
#'
#' # MCMC samples and BIC vector
#' samples <- runBatchMix(X, R, thin, batch_vec, "MVN")
#'
#' lkl_df <- getLikelihood(samples)
#'
getLikelihood <- function(mcmc_output, choice = "complete_likelihood") {
  R <- mcmc_output$R
  thin <- mcmc_output$thin
  burn <- mcmc_output$burn
  first_recorded_iter <- burn + thin

  iters <- seq(first_recorded_iter, R, by = thin)

  invalid_choice <- !(
    choice %in% c(
      "observed_likelihood",
      "complete_likelihood",
      "BIC"
    )
  )

  if (invalid_choice) {
    stop("Choice must be one of 'observed_likelihood', 'complete_likelihood' or 'BIC'.")
  }

  sampled_likelihoods <- mcmc_output[[choice]]

  lkl_df <- data.frame(score = sampled_likelihoods, iteration = iters)
  colnames(lkl_df) <- c(choice, "iteration")

  lkl_df
}

#' @title Bayesian Mixture Modelling for Joint Model-Based
#' Clustering/Classification and Batch Correction
#' @description Semi-supervised and unsupervised Bayesian mixture models that
#' simultaneously infer the cluster/class structure and a batch correction.
#' Densities available are the multivariate normal and the multivariate t.
#' The model sampler is implemented in C++. This package is aimed at analysis of
#' low-dimensional data generated across several batches. See
#' (Coleman et al. (2022))[https://doi.org/10.1101/2022.01.14.476352] for
#' details of the model.
#' @name batchmix-package
#' @aliases batchmix
#' @docType package
#' @author Stephen Coleman <stcolema@tcd.ie>, Paul D.W. Kirk, Chris Wallace
#' @keywords package
#'
#' @importFrom ggplot2 aes_string facet_grid facet_wrap geom_line geom_point
#' ggplot label_both labeller labs
#' @importFrom stats cutree as.dist hclust median rbeta rchisq rnorm
#' @importFrom tidyr contains pivot_longer
#' @importFrom Rcpp evalCpp sourceCpp
#' @importFrom RcppParallel setThreadOptions
#' @useDynLib batchmix
#' @examples
#'
#' # Data in a matrix format
#' X <- matrix(c(rnorm(100, 0, 1), rnorm(100, 3, 1)), ncol = 2, byrow = TRUE)
#'
#' # Initial labelling
#' labels <- c(
#'   rep(1, 10),
#'   sample(c(1, 2), size = 40, replace = TRUE),
#'   rep(2, 10),
#'   sample(c(1, 2), size = 40, replace = TRUE)
#' )
#'
#' # Which labels are observed
#' fixed <- c(rep(1, 10), rep(0, 40), rep(1, 10), rep(0, 40))
#'
#' # Batch
#' batch_vec <- sample(seq(1, 5), replace = TRUE, size = 100)
#'
#' # Sampling parameters
#' R <- 1000
#' thin <- 50
#'
#' # Classification
#' samples <- runBatchMix(X,
#'   R,
#'   thin,
#'   batch_vec,
#'   "MVN",
#'   initial_labels = labels,
#'   fixed = fixed,
#' )
#'
#' # Clustering
#' samples <- runBatchMix(X, R, thin, batch_vec, "MVT")
#'
NULL

#!/usr/bin/Rscript
#' @title Random Draw From Stick Breaking Prior
#' @description Draw weights from the stick-breaking prior.
#' @param alpha The concentration parameter.
#' @param K The number of weights to generate.
#' @return A vector of component weights.
#' @export
#' @examples
#' weights <- rStickBreakingPrior(1, 50)
#' @importFrom stats rbeta
rStickBreakingPrior <- function(alpha, K) {
  v <- stats::rbeta(K, 1, alpha)
  stick <- 1
  w <- rep(0, K)

  for (i in seq(1, K)) {
    w[i] <- v[i] * stick
    stick <- stick - w[i]
  }
  w
}

---
title: "batchmix workflow"
author: "Stephen Coleman"
output: rmarkdown::html_vignette
description: |
  Tutorial for clustering/classifying with batchmix.
vignette: >
  %\VignetteIndexEntry{Introduction to batchmix}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)

set.seed(1)
```

## Introduction

This document shows the basics of applying our Bayesian model-based 
clustering/classification with joint batch correction in ``R``. It shows how to
generate some toy data, apply the model, assess convergence and process outputs.

## Data generation

We simulate some data using the ``generateBatchData`` function.

```{r dataGen}
library(ggplot2)
library(batchmix)

# Data dimensions
N <- 200
P <- 4
K <- 3
B <- 5

# Generating model parameters
mean_dist <- 2.25
batch_dist <- 0.3
group_means <- seq(1, K) * mean_dist
batch_shift <- rnorm(B, mean = batch_dist, sd = batch_dist)
std_dev <- rep(2, K)
batch_var <- rep(1.2, B)
group_weights <- rep(1 / K, K)
batch_weights <- rep(1 / B, B)
dfs <- c(4, 15, 120)

my_data <- generateBatchData(
  N,
  P,
  group_means,
  std_dev,
  batch_shift,
  batch_var,
  group_weights,
  batch_weights,
  type = "MVT",
  group_dfs = dfs
)
```

This gives us a named list with two related datasets, the ``observed_data`` 
which includes batch effects and the ``corrected_data`` which is batch-free. It
also includes ``group_IDs``, a vector indicating class membership for each item,
``batch_IDs``, which indicates batch of origin for each item, and ``fixed``,
which indicates which labels are observed and fixed in the model. We pull these 
out of the names list in the format that the modelling functions desire them.

```{r dataClean}
X <- my_data$observed_data

true_labels <- my_data$group_IDs
fixed <- my_data$fixed
batch_vec <- my_data$batch_IDs

alpha <- 1
initial_labels <- generateInitialLabels(alpha, K, fixed, true_labels)
```

## Modelling

Given some data, we are interested in modelling it. We assume here that the set
of observed labels includes at least one example of each class in the data. 

```{r runMCMCChains}
# Sampling parameters
R <- 1000
thin <- 50
n_chains <- 3

# Density choice
type <- "MVT"

# MCMC samples and BIC vector
mcmc_output <- runMCMCChains(
  X,
  n_chains,
  R,
  thin,
  batch_vec,
  type,
  initial_labels = initial_labels,
  fixed = fixed
)
```

We want to assess two things. First, how frequently the proposed parameters in the Metropolis-Hastings step are accepted:

```{r plotAcceptanceRatesEarly}
plotAcceptanceRates(mcmc_output)
```

Secondly, we want to asses how well our chains have converged. To do this we plot the ``complete_likelihood`` of each chain. This is the quantity most relevant to a clustering/classification, being dependent on the labels. The ``observed_likelihood`` is independent of labels and more relevant for density estimation.

```{r likelihood}
plotLikelihoods(mcmc_output)
```

We see that our chains disagree. We have to run them for more iterations. We use the ``continueChains`` function for this.

```{r continueChains}
R_new <- 9000

# Given an initial value for the parameters
new_output <- continueChains(
  mcmc_output,
  X,
  fixed,
  batch_vec,
  R_new,
  keep_old_samples = TRUE
)
```

To see if the chains better agree we re-plot the likelihood.

```{r continuedLikelihood}
plotLikelihoods(new_output)
```

We also re-check the acceptance rates.

```{r plotAcceptanceRates}
plotAcceptanceRates(new_output)
```

This looks like several of the chains agree by the 5,000th iteration.

## Process chains

We process the chains, acquiring point estimates of different quantities.

```{r processChains}
# Burn in
burn <- 5000

# Process the MCMC samples
processed_samples <- processMCMCChains(new_output, burn)
```

## Visualisation

For multidimensional data we use a PCA plot.

```{r pca}
chain_used <- processed_samples[[1]]

pc <- prcomp(X, scale = T)
pc_batch_corrected <- prcomp(chain_used$inferred_dataset)

plot_df <- data.frame(
  PC1 = pc$x[, 1],
  PC2 = pc$x[, 2],
  PC1_bf = pc_batch_corrected$x[, 1],
  PC2_bf = pc_batch_corrected$x[, 2],
  pred_labels = factor(chain_used$pred),
  true_labels = factor(true_labels),
  prob = chain_used$prob,
  batch = factor(batch_vec)
)

plot_df |>
  ggplot(aes(
    x = PC1,
    y = PC2,
    colour = true_labels,
    alpha = prob
  )) +
  geom_point()

plot_df |>
  ggplot(aes(
    x = PC1_bf,
    y = PC2_bf,
    colour = pred_labels,
    alpha = prob
  )) +
  geom_point()

test_inds <- which(fixed == 0)

sum(true_labels[test_inds] == chain_used$pred[test_inds]) / length(test_inds)
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/samplePriorLabels.R
\name{samplePriorLabels}
\alias{samplePriorLabels}
\title{Sample prior labels}
\usage{
samplePriorLabels(alpha, K, N)
}
\arguments{
\item{alpha}{The concentration parameter for the stick-breaking prior.}

\item{K}{The number of components to include (the upper bound on the number of unique labels generated).}

\item{N}{The number of labels to generate.}
}
\value{
A vector of labels.
}
\description{
Generate labels from the stick-breaking prior.
}
\examples{
initial_labels <- samplePriorLabels(1, 50, 100)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generateBatchData.R
\name{generateBatchData}
\alias{generateBatchData}
\title{Generate batch data}
\usage{
generateBatchData(
  N,
  P,
  group_means,
  group_std_devs,
  batch_shift,
  batch_scale,
  group_weights,
  batch_weights,
  type = "MVN",
  group_dfs = NULL,
  frac_known = 0.2,
  permute_variables = TRUE,
  scale_data = FALSE
)
}
\arguments{
\item{N}{The number of items (rows) to generate.}

\item{P}{The number of columns in the generated dataset.}

\item{group_means}{A vector of the group means for a column.}

\item{group_std_devs}{A vector of group standard deviations for a column.}

\item{batch_shift}{A vector of batch means in a column.}

\item{batch_scale}{A vector of batch standard deviations within a column.}

\item{group_weights}{One of either a K x B matrix of the expected proportion
of each batch in each group or a K-vector of the expected proportion of the
entire dataset in each group.}

\item{batch_weights}{A vector of the expected proportion of N in each batch.}

\item{type}{A string indicating if data should be generated from
multivariate normal ("MVN") or multivariate t ("MVT") densities (defaults to
"MVN").}

\item{group_dfs}{A K-vector of the group specific degrees of freedom.}

\item{frac_known}{The number of items with known labels.}

\item{permute_variables}{Logical indicating if group and batch means and
standard deviations should be permuted in each column or not (defaults to
``TRUE``).}

\item{scale_data}{Logical indicating if data should be mean centred and
standardised (defaults to ``FALSE``).}
}
\value{
A list of 5 objects; the data generated from the groups with and
without batch effects, the label indicating the generating group, the
batch label and the vector indicating training versus test.
}
\description{
Generate data from K multivaraite normal or multivariate t
distributions with additional noise from batches. Assumes independence across
columns. In each column the parameters are randomly permuted for both the
groups and batches.
}
\examples{
N <- 500
P <- 2
K <- 2
B <- 5
mean_dist <- 4
batch_dist <- 0.3
group_means <- seq(1, K) * mean_dist
batch_shift <- rnorm(B, mean = batch_dist, sd = batch_dist)
std_dev <- rep(2, K)
batch_var <- rep(1.2, B)
group_weights <- rep(1 / K, K)
batch_weights <- rep(1 / B, B)
dfs <- c(4, 7)
my_data <- generateBatchData(
  N,
  P,
  group_means,
  std_dev,
  batch_shift,
  batch_var,
  group_weights,
  batch_weights,
  type = "MVT",
  group_dfs = dfs
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predictClass.R
\name{predictClass}
\alias{predictClass}
\title{Predict class}
\usage{
predictClass(prob)
}
\arguments{
\item{prob}{Output from the ``calcAllocProb`` function, a N x K matrix of
allocation probabilities.}
}
\value{
An N vector of class allocations.
}
\description{
Predicts a final class for each item given a matrix of
allocation probabilities.
}
\examples{

# Data in a matrix format
X <- matrix(c(rnorm(100, 0, 1), rnorm(100, 3, 1)), ncol = 2, byrow = TRUE)

# Initial labelling
labels <- c(
  rep(1, 10),
  sample(c(1, 2), size = 40, replace = TRUE),
  rep(2, 10),
  sample(c(1, 2), size = 40, replace = TRUE)
)

fixed <- c(rep(1, 10), rep(0, 40), rep(1, 10), rep(0, 40))

# Batch
batch_vec <- sample(seq(1, 5), replace = TRUE, size = 100)

# Sampling parameters
R <- 1000
thin <- 50

# MCMC samples and BIC vector
samples <- batchSemiSupervisedMixtureModel(
  X,
  R,
  thin,
  labels,
  fixed,
  batch_vec,
  "MVN"
)

# Burn in
burn <- 200
eff_burn <- burn / thin

# Probability across classes
probs <- calcAllocProb(samples, burn = burn)

# Predict the class
preds <- predictClass(probs)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{gammaLogLikelihood}
\alias{gammaLogLikelihood}
\title{Gamma log-likelihood}
\usage{
gammaLogLikelihood(x, shape, rate)
}
\arguments{
\item{x}{- double; the value to calculate the unnormalised likelihood of.}

\item{shape}{- double; the shape of the Gamma distribution.}

\item{rate}{- double; the rate of the Gamma distribution}
}
\value{
the unnormalised log-likelihood of x in a Gamma with parameters shape 
and rate.
}
\description{
Used in calculating model probability in Metropolis-Hastings 
algorithm when proposals are from the Gamma distribution.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{invGammaLogLikelihood}
\alias{invGammaLogLikelihood}
\title{Inverse gamma log-likelihood}
\usage{
invGammaLogLikelihood(x, shape, scale)
}
\arguments{
\item{x}{- double; the value to calculate the likelihood of.}

\item{shape}{- double; the shape of the inverse-Gamma distribution.}

\item{scale}{- double; the scale of the inverse-Gamma distribution}
}
\value{
the unnormalised log-likelihood of x in a inverse-Gamma with parameters 
shape and scale.
}
\description{
Used in calculating model probability in Metropolis-Hastings 
algorithm when proposals are from the inverse-Gamma distribution.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getLikelihood.R
\name{getLikelihood}
\alias{getLikelihood}
\title{Get likelihood}
\usage{
getLikelihood(mcmc_output, choice = "complete_likelihood")
}
\arguments{
\item{mcmc_output}{The output from the mixture model.}

\item{choice}{The model fit score to use. Must be one of
``'observed_likelihood'``, ``'complete_likelihood'`` or ``'BIC'``. Defaults
to ``'complete_likelihood'``.}
}
\value{
A data.frame containing the model fit score of choice and the
iteration.
}
\description{
Extracts the model fit score from the mixture model output.
}
\examples{

# Data in a matrix format
X <- matrix(c(rnorm(100, 0, 1), rnorm(100, 3, 1)), ncol = 2, byrow = TRUE)

# Batch
batch_vec <- sample(seq(1, 5), replace = TRUE, size = 100)

# Sampling parameters
R <- 100
thin <- 5

# MCMC samples and BIC vector
samples <- runBatchMix(X, R, thin, batch_vec, "MVN")

lkl_df <- getLikelihood(samples)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rStickBreakingPrior.R
\name{rStickBreakingPrior}
\alias{rStickBreakingPrior}
\title{Random Draw From Stick Breaking Prior}
\usage{
rStickBreakingPrior(alpha, K)
}
\arguments{
\item{alpha}{The concentration parameter.}

\item{K}{The number of weights to generate.}
}
\value{
A vector of component weights.
}
\description{
Draw weights from the stick-breaking prior.
}
\examples{
weights <- rStickBreakingPrior(1, 50)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calcAllocProb.R
\name{calcAllocProb}
\alias{calcAllocProb}
\title{Calculate allocation probabilities}
\usage{
calcAllocProb(mcmc_samples, burn = 0, method = "median")
}
\arguments{
\item{mcmc_samples}{Output from ``batchSemiSupervisedMixtureModel``.}

\item{burn}{The number of samples to discard.}

\item{method}{The point estimate to use. ``method = 'mean'`` or
``method = 'median'``. ``'median'`` is the default.}
}
\value{
An N x K matrix of class probabilities.
}
\description{
Calculate the empirical allocation probability for each class
based on the sampled allocation probabilities.
}
\examples{

# Data in matrix format
X <- matrix(c(rnorm(100, 0, 1), rnorm(100, 3, 1)), ncol = 2, byrow = TRUE)

# Initial labelling
labels <- c(
  rep(1, 10),
  sample(c(1, 2), size = 40, replace = TRUE),
  rep(2, 10),
  sample(c(1, 2), size = 40, replace = TRUE)
)

fixed <- c(rep(1, 10), rep(0, 40), rep(1, 10), rep(0, 40))

# Batch
batch_vec <- sample(seq(1, 5), replace = TRUE, size = 100)

# Sampling parameters
R <- 1000
thin <- 50

# MCMC samples and BIC vector
samples <- batchSemiSupervisedMixtureModel(X, R, thin, labels, fixed, batch_vec, "MVN")

# Burn in
burn <- 20
eff_burn <- burn / thin

# Probability across classes
probs <- calcAllocProb(samples, burn = burn)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/VI.lb.R
\name{VI.lb}
\alias{VI.lb}
\title{Minimum VI lower bound}
\usage{
VI.lb(cls, psm)
}
\arguments{
\item{cls}{A clustering for which the lower bound of the Variation of
Information is calculated.}

\item{psm}{The posterior similarity matrix which `cls` is a summary thereof.}
}
\value{
A vector of the the lower bound of the Variation of Information
for
}
\description{
Local implementation of S. Wade's `minVI` function from their
`mcclust.ext` package (available from github).
Reimplemented here to avoid dependency on a non-CRAN package. Computes the
lower bound to the posterior expected Variation of Information. For full
details please see the aforementioned package and Wade and Ghahramani, 2018,
'Bayesian Cluster Analysis: Point Estimation and Credible Balls (with
Discussion)'.
}
\examples{
\dontrun{
# MCMC samples and BIC vector
mcmc_outputs <- runMCMCChains(
  X,
  n_chains,
  R,
  thin,
  batch_vec,
  type
)

# Note that in this toy example we have not applied a burn in
psm <- createSimilarityMat(mcmc_outputs[[1]]$samples)
VI.lb(mcmc_outputs[[1]]$samples[1, ], psm)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{sampleMVN}
\alias{sampleMVN}
\title{Sample mixture of multivariate normal distributions with batch effects}
\usage{
sampleMVN(
  X,
  K,
  B,
  labels,
  batch_vec,
  mu_proposal_window,
  cov_proposal_window,
  m_proposal_window,
  S_proposal_window,
  R,
  thin,
  concentration,
  m_scale,
  rho,
  theta,
  initial_mu,
  initial_cov,
  initial_m,
  initial_S,
  mu_initialised,
  cov_initialised,
  m_initialised,
  S_initialised,
  sample_m_scale
)
}
\arguments{
\item{X}{The data matrix to perform clustering upon (items to cluster in rows).}

\item{K}{The number of components to model (upper limit on the number of 
clusters found).}

\item{B}{The number of batches to model.}

\item{labels}{Vector item labels to initialise from.}

\item{batch_vec}{Observed batch labels.}

\item{mu_proposal_window}{The standard deviation for the Gaussian proposal
density of the cluster means.}

\item{cov_proposal_window}{The degrees of freedom for the Wishart proposal
density of the cluster covariances.}

\item{m_proposal_window}{The standard deviation for the Gaussian proposal
density of the batch mean effects.}

\item{S_proposal_window}{The rate for the Gamma proposal density of the 
batch scale.}

\item{R}{The number of iterations to run for.}

\item{thin}{thinning factor for samples recorded.}

\item{concentration}{Vector of concentrations for mixture weights
(recommended to be symmetric).}

\item{m_scale}{The scale hyperparameter for the batch shift prior 
distribution.}

\item{rho}{The shape of the prior distribution for the batch scale.}

\item{theta}{The scale of the prior distribution for the batch scale.}

\item{initial_mu}{A P x K matrix of initial values for the class means.}

\item{initial_cov}{A P x P x K cube of initial values for the class 
covariance matrices.}

\item{initial_m}{A P x B matrix of initial values for the batch shift 
effects.}

\item{initial_S}{A P x B matrix of initial values for the batch scales.}

\item{mu_initialised}{Bool indicating if the class means are initialised by
the user. If ``false`` then initial values are drawn from the prior 
distribution.}

\item{cov_initialised}{Bool indicating if the class covariance matrices are 
initialised by the user. If ``false`` then initial values are drawn from the
prior distribution.}

\item{m_initialised}{Bool indicating if the batch shift effects are 
initialised by the user. If ``false`` then initial values are drawn from the
prior distribution.}

\item{S_initialised}{Bool indicating if the batch scales are initialised by 
the user. If ``false`` then initial values are drawn from the prior 
distribution.}

\item{sample_m_scale}{Bool indicating if the hyperparameter on the batch 
shift effect is sampled or given as fixed.}
}
\value{
Named list of the different quantities drawn by the sampler.
}
\description{
Performs MCMC sampling for a mixture model with batch effects.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotAcceptanceRates.R
\name{plotAcceptanceRates}
\alias{plotAcceptanceRates}
\title{Plot acceptance rates}
\usage{
plotAcceptanceRates(mcmc_lst)
}
\arguments{
\item{mcmc_lst}{The output of the ``runMCMCChains`` function.}
}
\value{
A ggplot object of the boxplots of acceptance rates for each
parameter across chains.
}
\description{
Plot the acceptance rates for the parameters sampled in a
Metropolis-Hastings step. Aiming for acceptance rates in [0.2, 0.5] for the
class means, the batch effect on location and scale is a good rule of thumb.
The class covariance should be in [0.35, 0.8] based on the authors'
experience. The class degree of freedom appears to be prone to high
acceptance rates, but aim to keep this above 0.2 at a minimum.
}
\examples{
# Data in a matrix format
X <- matrix(c(rnorm(100, 0, 1), rnorm(100, 3, 1)), ncol = 2, byrow = TRUE)

# Initial labelling
labels <- c(
  rep(1, 10),
  sample(c(1, 2), size = 40, replace = TRUE),
  rep(2, 10),
  sample(c(1, 2), size = 40, replace = TRUE)
)

fixed <- c(rep(1, 10), rep(0, 40), rep(1, 10), rep(0, 40))

# Batch
batch_vec <- sample(seq(1, 5), replace = TRUE, size = 100)

# Sampling parameters
R <- 500
thin <- 10
n_chains <- 4

# MCMC samples and BIC vector
mcmc_lst <- runMCMCChains(X, n_chains, R, thin, batch_vec, "MVN",
  initial_labels = labels,
  fixed = fixed
)

# Plot the acceptance rate of each parameter in the 4 chains
plotAcceptanceRates(mcmc_lst)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotSampledBatchScales.R
\name{plotSampledBatchScales}
\alias{plotSampledBatchScales}
\title{Plot sampled batch scales}
\usage{
plotSampledBatchScales(samples, burn_in = 0)
}
\arguments{
\item{samples}{The output of the ``batchUnsupervisedMixtureModel`` or
``batchSemiSupervisedMixtureModel`` functions.}

\item{burn_in}{The samples at the beginning of the chain to drop. Defaults to 0.}
}
\value{
A ggplot object of the values in each sampled batch mean per iteration.
}
\description{
Plot the sampled values for the batch scale in each
dimension from the output of the mixture model functions. Not recommended
for large B or P.
}
\examples{
# Data in matrix format
X <- matrix(c(rnorm(100, 0, 1), rnorm(100, 3, 1)), ncol = 2, byrow = TRUE)

# Observed batches represented by integers
batch_vec <- sample(seq(1, 5), size = 100, replace = TRUE)

# MCMC iterations (this is too low for real use)
R <- 100
thin <- 5

# MCMC samples and BIC vector
samples <- runBatchMix(X, R, thin, batch_vec, "MVN")

# Plot the sampled value of the batch scales against MCMC iteration
plotSampledBatchScales(samples)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generateInitialLabels.R
\name{generateInitialLabels}
\alias{generateInitialLabels}
\title{Generate initial labels}
\usage{
generateInitialLabels(alpha, K, fixed, labels = NULL)
}
\arguments{
\item{alpha}{The mass in the stick breaking prior}

\item{K}{The number of classes available.}

\item{fixed}{The vector of 0s and 1s indicating which labels are to be held
fixed.}

\item{labels}{The initial labelling. Defaults to NULL.}
}
\value{
An N vector of labels.
}
\description{
For simulated data, generates an initial labelling for sampling.
}
\examples{
N <- 500
P <- 2
K <- 2
B <- 5
mean_dist <- 4
batch_dist <- 0.3
cluster_means <- seq(1, K) * mean_dist
batch_shift <- rnorm(B, mean = batch_dist, sd = batch_dist)
std_dev <- rep(2, K)
batch_var <- rep(1.2, B)
cluster_weights <- rep(1 / K, K)
batch_weights <- rep(1 / B, B)

my_data <- generateBatchData(
  N,
  P,
  cluster_means,
  std_dev,
  batch_shift,
  batch_var,
  cluster_weights,
  batch_weights
)

initial_labels <- generateInitialLabels(1, K, my_data$fixed)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/batchmix-package.R
\docType{package}
\name{batchmix-package}
\alias{batchmix-package}
\alias{batchmix}
\title{Bayesian Mixture Modelling for Joint Model-Based
Clustering/Classification and Batch Correction}
\description{
Semi-supervised and unsupervised Bayesian mixture models that
simultaneously infer the cluster/class structure and a batch correction.
Densities available are the multivariate normal and the multivariate t.
The model sampler is implemented in C++. This package is aimed at analysis of
low-dimensional data generated across several batches. See
(Coleman et al. (2022))[https://doi.org/10.1101/2022.01.14.476352] for
details of the model.
}
\examples{

# Data in a matrix format
X <- matrix(c(rnorm(100, 0, 1), rnorm(100, 3, 1)), ncol = 2, byrow = TRUE)

# Initial labelling
labels <- c(
  rep(1, 10),
  sample(c(1, 2), size = 40, replace = TRUE),
  rep(2, 10),
  sample(c(1, 2), size = 40, replace = TRUE)
)

# Which labels are observed
fixed <- c(rep(1, 10), rep(0, 40), rep(1, 10), rep(0, 40))

# Batch
batch_vec <- sample(seq(1, 5), replace = TRUE, size = 100)

# Sampling parameters
R <- 1000
thin <- 50

# Classification
samples <- runBatchMix(X,
  R,
  thin,
  batch_vec,
  "MVN",
  initial_labels = labels,
  fixed = fixed,
)

# Clustering
samples <- runBatchMix(X, R, thin, batch_vec, "MVT")

}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/stcolema/batchmix}
  \item Report bugs at \url{https://github.com/stcolema/batchmix/issues}
}

}
\author{
Stephen Coleman <stcolema@tcd.ie>, Paul D.W. Kirk, Chris Wallace
}
\keyword{package}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{wishartLogLikelihood}
\alias{wishartLogLikelihood}
\title{Wishart log-likelihood}
\usage{
wishartLogLikelihood(X, V, n, P)
}
\arguments{
\item{X}{- matrix; the matrix to calculate the likelihood of.}

\item{V}{- matrix; the scale of the Wishart distribution.}

\item{n}{- double; the degrees of freedom for the Wishart distribution.}

\item{P}{- unsigned integer; the dimension of X.}
}
\value{
the unnormalised log-likelihood of X in a Wishart with parameters V 
and n.
}
\description{
Used in calculating model probability in Metropolis-Hastings 
algorithm when proposals are from the Wishart distribution.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkDataGenerationInputs.R
\name{checkDataGenerationInputs}
\alias{checkDataGenerationInputs}
\title{Check data generation inputs}
\usage{
checkDataGenerationInputs(
  N,
  P,
  group_means,
  group_std_devs,
  batch_shift,
  batch_scale,
  group_weights,
  batch_weights,
  type,
  group_dfs,
  frac_known,
  permute_variables,
  scale_data
)
}
\arguments{
\item{N}{The number of items (rows) to generate.}

\item{P}{The number of columns in the generated dataset.}

\item{group_means}{A vector of the group means for a column.}

\item{group_std_devs}{A vector of group standard deviations for a column.}

\item{batch_shift}{A vector of batch means in a column.}

\item{batch_scale}{A vector of batch standard deviations within a column.}

\item{group_weights}{A K x B matrix of the expected proportion of N in each group in each batch.}

\item{batch_weights}{A vector of the expected proportion of N in each batch.}

\item{type}{A string indicating if data should be generated from
multivariate normal ("MVN") or multivariate t ("MVT") densities.}

\item{group_dfs}{A K-vector of the group specific degrees of freedom.}

\item{frac_known}{The number of items with known labels.}

\item{permute_variables}{Logical indicating if group and batch means and
standard deviations should be permuted in each column or not.}

\item{scale_data}{Logical indicating if data should be mean centred and
standardised.}
}
\value{
No return value, called for side effects.
}
\description{
Checks that the inputs for the ``generateBatchData`` function
are correct. For internal use only.
}
\examples{
N <- 500
P <- 2
K <- 2
B <- 5
mean_dist <- 4
batch_dist <- 0.3
group_means <- seq(1, K) * mean_dist
batch_shift <- rnorm(B, mean = batch_dist, sd = batch_dist)
group_std_devs <- rep(2, K)
batch_scale <- rep(1.2, B)
group_weights <- rep(1 / K, K)
batch_weights <- rep(1 / B, B)
type <- "MVT"
group_dfs <- c(4, 7)
frac_known <- 0.3
permute_variables <- TRUE
scale_data <- FALSE

checkDataGenerationInputs(
  N,
  P,
  group_means,
  group_std_devs,
  batch_shift,
  batch_scale,
  group_weights,
  batch_weights,
  type,
  group_dfs,
  frac_known,
  permute_variables,
  scale_data
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prepareInitialParameters.R
\name{prepareInitialParameters}
\alias{prepareInitialParameters}
\title{Prepare initial values}
\usage{
prepareInitialParameters(
  initial_class_means,
  initial_class_covariance,
  initial_batch_shift,
  initial_batch_scale,
  initial_class_df,
  P,
  K,
  B,
  type
)
}
\arguments{
\item{initial_class_means}{A $P x K$ matrix of initial values for the class
means. Defaults to draws from the prior distribution.}

\item{initial_class_covariance}{A $P x P x K$ array of initial values for the
class covariance matrices. Defaults to draws from the prior distribution.}

\item{initial_batch_shift}{A $P x B$ matrix of initial values for the batch
shift effect Defaults to draws from the prior distribution.}

\item{initial_batch_scale}{A $P x B$ matrix of initial values for the batch
scales Defaults to draws from the prior distribution.}

\item{initial_class_df}{A $K$ vector of initial values for the class degrees
of freedom. Defaults to draws from the prior distribution.}

\item{P}{Integer. The number of measurements for each sample in the dataset
being modelled.}

\item{K}{The number of classes/clusters being modelled.}

\item{B}{The number of batches being modelled.}

\item{type}{The type of mixture model used; one of "MVN" or "MVT".}
}
\value{
A named list containing the different parameters.
}
\description{
Prepares user given values for input into the C++ function.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/runBatchMix.R
\name{runBatchMix}
\alias{runBatchMix}
\title{Run Batch Mixture Model}
\usage{
runBatchMix(
  X,
  R,
  thin,
  batch_vec,
  type,
  K_max = NULL,
  initial_labels = NULL,
  fixed = NULL,
  alpha = 1,
  mu_proposal_window = 0.5^2,
  cov_proposal_window = 0.002,
  m_proposal_window = 0.3^2,
  S_proposal_window = 0.01,
  t_df_proposal_window = 0.015,
  m_scale = NULL,
  rho = 3,
  theta = 1,
  initial_class_means = NULL,
  initial_class_covariance = NULL,
  initial_batch_shift = NULL,
  initial_batch_scale = NULL,
  initial_class_df = NULL,
  verbose = TRUE
)
}
\arguments{
\item{X}{Data to cluster as a matrix with the items to cluster held in rows.}

\item{R}{The number of iterations in the sampler.}

\item{thin}{The factor by which the samples generated are thinned, e.g. if
``thin=50`` only every 50th sample is kept.}

\item{batch_vec}{Labels identifying which batch each item being clustered is
from.}

\item{type}{Character indicating density type to use. One of 'MVN'
(multivariate normal distribution) or 'MVT' (multivariate t distribution).}

\item{K_max}{The number of components to include (the upper bound on the
number of clusters in each sample). Defaults to the number of unique labels
in ``initial_labels``.}

\item{initial_labels}{Initial clustering.}

\item{fixed}{Which items are fixed in their initial label. If not given,
defaults to a vector of 0 meaning the model is run unsupervised.}

\item{alpha}{The concentration parameter for the stick-breaking prior and the
weights in the model.}

\item{mu_proposal_window}{The proposal window for the cluster mean proposal
kernel. Making this smaller will normally increase the acceptance rate for
the proposed values in the Metropolis-Hastings sampler. The proposal density
is a Gaussian distribution, the window is the variance.}

\item{cov_proposal_window}{The proposal window for the cluster covariance
proposal kernel. The proposal density is a Wishart distribution, this
argument is the reciprocal of the degree of freedom. It is recommended to
set this aiming for accpetance rates of greater than 0.5 for the covariance
matrices (e.g., between 2e-03 and 1e-04 is a good range to consider
initially). As the entire covariance matrix is sampled at once exploration is
difficult.}

\item{m_proposal_window}{The proposal window for the batch mean proposal
 kernel. The proposal density is a Gaussian distribution, the window is the
variance.}

\item{S_proposal_window}{The proposal window for the batch standard deviation
 proposal kernel. The proposal density is a Gamma distribution, this
argument is the reciprocal of the rate. Recommended range to initially
consider is 0.015 to 2e-03, though smaller values might be necessary
particularly in higher dimensional data.}

\item{t_df_proposal_window}{The proposal window for the degrees of freedom
for the multivariate t distribution (not used if type is not 'MVT'). The
proposal density is a Gamma distribution, this argument is the reciprocal of
the rate. If the data is more Gaussian than the degrees of freedom might have
high acceptance rates regardless of the value chosen.}

\item{m_scale}{The scale hyperparameter for the batch shift prior
distribution. This defines the scale of the batch effect upon the mean and
should be in (0, 1].}

\item{rho}{The shape of the prior distribution for the batch scale.}

\item{theta}{The scale of the prior distribution for the batch scale.}

\item{initial_class_means}{A $P x K$ matrix of initial values for the class
means. Defaults to draws from the prior distribution.}

\item{initial_class_covariance}{A $P x P x K$ array of initial values for
the class covariance matrices. Defaults to draws from the prior distribution.}

\item{initial_batch_shift}{A $P x B$ matrix of initial values for the batch
shift effect Defaults to draws from the prior distribution.}

\item{initial_batch_scale}{A $P x B$ matrix of initial values for the batch
scales Defaults to draws from the prior distribution.}

\item{initial_class_df}{A $K$ vector of initial values for the class degrees
of freedom. Defaults to draws from the prior distribution.}

\item{verbose}{Logiccal indicating if warning about proposal windows should
be printed.}
}
\value{
A named list containing the sampled partitions, cluster and batch
parameters, model fit measures and some details on the model call.
}
\description{
Runs a MCMC chain for a Bayesian mixture model which models
both batch effects and class/cluster structure.
}
\examples{

# Data in a matrix format
X <- matrix(c(rnorm(100, 0, 1), rnorm(100, 3, 1)), ncol = 2, byrow = TRUE)

# Initial labelling
labels <- c(
  rep(1, 10),
  sample(c(1, 2), size = 40, replace = TRUE),
  rep(2, 10),
  sample(c(1, 2), size = 40, replace = TRUE)
)

fixed <- c(rep(1, 10), rep(0, 40), rep(1, 10), rep(0, 40))

# Batch
batch_vec <- sample(seq(1, 5), replace = TRUE, size = 100)

# Density choice
type <- "MVN"

# Sampling parameters
R <- 1000
thin <- 50

# MCMC samples
mcmc_out <- runBatchMix(
  X,
  R,
  thin,
  batch_vec,
  type,
  initial_labels = labels,
  fixed = fixed
)

# Given an initial value for the parameters
initial_class_means <- matrix(c(1, 1, 3, 4), nrow = 2)
initial_class_covariance <- array(c(1, 0, 0, 1, 1, 0, 0, 1),
  dim = c(2, 2, 2)
)

# We can use values from a previous chain
initial_batch_shift <- mcmc_out$batch_shift[, , R / thin]
initial_batch_scale <- matrix(
  c(1.2, 1.3, 1.7, 1.1, 1.4, 1.3, 1.2, 1.2, 1.1, 2.0),
  nrow = 2
)

mcmc_out <- runBatchMix(X,
  R,
  thin,
  batch_vec,
  type,
  initial_labels = labels,
  fixed = fixed,
  initial_class_means = initial_class_means,
  initial_class_covariance = initial_class_covariance,
  initial_batch_shift = initial_batch_shift,
  initial_batch_scale = initial_batch_scale
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/continueChain.R
\name{continueChain}
\alias{continueChain}
\title{Continue chain}
\usage{
continueChain(mcmc_output, X, fixed, batch_vec, R, keep_old_samples = TRUE)
}
\arguments{
\item{mcmc_output}{Chain to be continued.}

\item{X}{Data to cluster as a matrix with the items to cluster held in rows.}

\item{fixed}{The indicator vector for which labels are observed.}

\item{batch_vec}{The vector of the batch labels for the data.}

\item{R}{The number of iterations to run in this continuation (thinning
factor is the same as initial chain).}

\item{keep_old_samples}{Logical indicating if the original samples should be
kept or only the new samples returned. Defaults to TRUE.}
}
\value{
A named list containing the sampled partitions, cluster and batch
parameters, model fit measures and some details on the model call.
}
\description{
Continues sampling from a previous position for a given chain.
}
\examples{

# Data in a matrix format
X <- matrix(c(rnorm(100, 0, 1), rnorm(100, 3, 1)), ncol = 2, byrow = TRUE)

# Initial labelling
labels <- c(
  rep(1, 10),
  sample(c(1, 2), size = 40, replace = TRUE),
  rep(2, 10),
  sample(c(1, 2), size = 40, replace = TRUE)
)

fixed <- c(rep(1, 10), rep(0, 40), rep(1, 10), rep(0, 40))

# Batch
batch_vec <- sample(seq(1, 5), replace = TRUE, size = 100)

# Density choice
type <- "MVT"

# Sampling parameters
R <- 1000
thin <- 50

# MCMC samples and BIC vector
mcmc_output <- runBatchMix(
  X,
  R,
  thin,
  batch_vec,
  type,
  initial_labels = labels,
  fixed = fixed
)

# Given an initial value for the parameters
mcmc_output <- continueChain(
  mcmc_output,
  X,
  fixed,
  batch_vec,
  R,
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/minVI.R
\name{minVI}
\alias{minVI}
\title{Minimium VI}
\usage{
minVI(psm, cls.draw = NULL, method = "avg", max.k = NULL)
}
\arguments{
\item{psm}{The posterior similarity matrix for a set of clustering MCMC
samples such as is returned by the `createSimilarityMat` function.}

\item{cls.draw}{The set of clustering MCMC samples used to generate `psm`.
Only required if `method` is one of `'draws'` or `'all'`.}

\item{method}{String indicating which method is used to find the point
estimate clustering. Must be one of `'avg'`, `'comp'`, `'draws'` or `'all'`.
Defaults to `'avg'`. If `'all'` is passed the three methods are all applied
to return different choices of point clustering.}

\item{max.k}{The maximum number of clusters to consider. Only used by the
`'comp'` and `'avg'` methods. Defaults to one-quarter the number of data
points rounded up.}
}
\value{
If `method` is `'all'` returns a matrix of four clusterings, one for
each method and a repeat of that which performs best based on minimising the
Variation of Information between the clustering and the PSM. Otherwise
returns a vector. This is annotated with the attribute `"info"`, a named list
describing:

* `.$loss`: the loss score used (Variation of Information)

* `.$maxNClusters`: the `max.k` value used by the `'comp'` and `'avg'` methods

* `.$expectedLoss`: the estimated minimum Variation of Information for the point
clustering(s)

* `.$method`: the point method used to infer the clustering(s)

Names are due to legacy reasons - this function is replacing the
`salso::salso` function and name choices are to minimise workflow damage.
}
\description{
Local implementation of S. Wade's `minVI` function from their
`mcclust.ext` package (available from github).
Reimplemented here to avoid dependency on a non-CRAN package and we have
dropped the `greedy` method. Finds the optimal partition by minimising the
lower bound to the Variation of Information obtained from Jensen's inequality
where the expectation and log are reversed. For full details please see the
aforementioned package and Wade and Ghahramani, 2018, 'Bayesian Cluster
Analysis: Point Estimation and Credible Balls (with Discussion)'.
}
\examples{
\dontrun{
# MCMC samples and BIC vector
mcmc_outputs <- runMCMCChains(
  X,
  n_chains,
  R,
  thin,
  batch_vec,
  type
)

# Note that in this toy example we have not applied a burn in
psm <- createSimilarityMat(mcmc_outputs[[1]]$samples)
cl_est <- minVI(psm, mcmc_outputs[[1]]$samples)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkProposalWindows.R
\name{checkProposalWindows}
\alias{checkProposalWindows}
\title{Check proposal windows}
\usage{
checkProposalWindows(
  mu_proposal_window,
  cov_proposal_window,
  m_proposal_window,
  S_proposal_window,
  t_df_proposal_window,
  verbose = TRUE
)
}
\arguments{
\item{mu_proposal_window}{The proposal window for the cluster mean proposal
kernel. The proposal density is a Gaussian distribution, the window is the
variance.}

\item{cov_proposal_window}{The proposal window for the cluster covariance
proposal kernel. The proposal density is a Wishart distribution, this
argument is the reciprocal of the degree of freedom.}

\item{m_proposal_window}{The proposal window for the batch mean proposal
 kernel. The proposal density is a Gaussian distribution, the window is the
variance.}

\item{S_proposal_window}{The proposal window for the batch standard deviation
 proposal kernel. The proposal density is a Gamma distribution, this
argument is the reciprocal of the rate.}

\item{t_df_proposal_window}{The proposal window for the degrees of freedom
for the multivariate t distribution (not used if type is not 'MVT'). The
proposal density is a Gamma distribution, this argument is the reciprocal of
the rate.}

\item{verbose}{Logical indicating if a warning should be printed if proposal
windows are outside their expected scale.}
}
\value{
No return value, called for side effects
}
\description{
Checks the proposal windows are acceptable.
}
\examples{
checkProposalWindows(0.1, 0.2, 0.3, 0.1, 0.4, 0.3)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{invWishartLogLikelihood}
\alias{invWishartLogLikelihood}
\title{Inverse-Wishart log-likelihood}
\usage{
invWishartLogLikelihood(X, Psi, nu, P)
}
\arguments{
\item{X}{- matrix; the matrix to calculate the likelihood of.}

\item{Psi}{- matrix; the scale of the inverse-Wishart distribution.}

\item{nu}{- double; the degrees of freedom for the inverse-Wishart distribution.}

\item{P}{- unsigned integer; the dimension of X.}
}
\value{
the unnormalised log-likelihood of X in a inverse-Wishart with parameters Psi 
and nu.
}
\description{
Used in calculating model probability in Metropolis-Hastings 
algorithm when proposals are from the Wishart distribution.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generateGroupIDsInSimulator.R
\name{generateGroupIDsInSimulator}
\alias{generateGroupIDsInSimulator}
\title{Generate group IDs}
\usage{
generateGroupIDsInSimulator(
  N,
  K,
  B,
  batch_IDs,
  group_weights,
  varying_group_within_batch
)
}
\arguments{
\item{N}{The number of items (rows) to generate.}

\item{K}{The number of groups to genetare.}

\item{B}{The number of batches present in ``batch_IDs``.}

\item{batch_IDs}{The batch membership of each item.}

\item{group_weights}{One of either a K x B matrix of the expected proportion
of each batch in each group or a K-vector of the expected proportion of the
entire dataset in each group.}

\item{varying_group_within_batch}{Flag indicating if the groups are vvarying
across batches.}
}
\value{
A N-vector of group membership.
}
\description{
Generate group IDs within ``generateBatchData``.
}
\examples{
N <- 500
K <- 2
B <- 5
group_weights <- rep(1 / K, K)
batch_weights <- rep(1 / B, B)
batch_IDs <- sample(seq(1, B), N, replace = TRUE, prob = batch_weights)
varying_group_within_batch <- FALSE
group_IDs <- generateGroupIDsInSimulator(
  N,
  K,
  B,
  batch_IDs,
  group_weights,
  varying_group_within_batch
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotSampledParameter.R
\name{plotSampledParameter}
\alias{plotSampledParameter}
\title{Plot sampled vector parameter}
\usage{
plotSampledParameter(samples, parameter, R = NULL, thin = 1, burn_in = 0)
}
\arguments{
\item{samples}{The output of the ``mixtureModel`` function.}

\item{parameter}{The name of the parameter to be plotted (a string).}

\item{R}{The number of iterations run. Defaults to the number of samples for
the cluster membership.}

\item{thin}{The thinning factor of the sampler. Defaults to 1.}

\item{burn_in}{The samples at the beginning of the chain to drop. Defaults to 0.}
}
\value{
A ggplot object of the values in each sampled batch mean per iteration.
}
\description{
Plot the sampled values for a sampled vector from the output of
the ``mixtureModel`` function. Not recommended for large B or P.
}
\examples{
# Data in matrix format
X <- matrix(c(rnorm(100, 0, 1), rnorm(100, 3, 1)), ncol = 2, byrow = TRUE)

# Observed batches represented by integers
batch_vec <- sample(seq(1, 3), size = 100, replace = TRUE)

# MCMC iterations (this is too low for real use)
R <- 50
thin <- 1

# MCMC samples and BIC vector
samples <- runBatchMix(X, R, thin, batch_vec, "MVN", K_max = 8)

# Plot the sampled value of the cluster means against MCMC iteration
parameter <- "means"
plotSampledParameter(samples, parameter, R, thin)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotLikelihoods.R
\name{plotLikelihoods}
\alias{plotLikelihoods}
\title{Plot likelihoods}
\usage{
plotLikelihoods(
  mcmc_outputs,
  choice = "complete_likelihood",
  colour_by_chain = TRUE
)
}
\arguments{
\item{mcmc_outputs}{The output from ``runMCMCChains``.}

\item{choice}{The model fit score to use. Must be one of
``'observed_likelihood'``, ``'complete_likelihood'`` or ``'BIC'``. Defaults
to ``'complete_likelihood'``.}

\item{colour_by_chain}{Logical indcating if plots should be coloured by chain
or all the same colour. Defaults to ``TRUE``.}
}
\value{
A ggplot2 object. Line plot of likelihood across iteration.
}
\description{
Plots the model fit for multiple chains.
}
\examples{

# Data in a matrix format
X <- matrix(c(rnorm(100, 0, 1), rnorm(100, 3, 1)), ncol = 2, byrow = TRUE)

# Initial labelling
labels <- c(
  rep(1, 10),
  sample(c(1, 2), size = 40, replace = TRUE),
  rep(2, 10),
  sample(c(1, 2), size = 40, replace = TRUE)
)

fixed <- c(rep(1, 10), rep(0, 40), rep(1, 10), rep(0, 40))

# Batch
batch_vec <- sample(seq(1, 5), replace = TRUE, size = 100)

# Sampling parameters
R <- 1000
thin <- 50
n_chains <- 4

# MCMC samples
samples <- runMCMCChains(X, n_chains, R, thin, batch_vec, "MVN",
  initial_labels = labels,
  fixed = fixed
)

p <- plotLikelihoods(samples)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getSampledBatchScale.R
\name{getSampledBatchScale}
\alias{getSampledBatchScale}
\title{Get sampled batch shift}
\usage{
getSampledBatchScale(
  sampled_batch_scale,
  B = dim(sampled_batch_scale)[2],
  P = dim(sampled_batch_scale)[1],
  R = dim(sampled_batch_scale)[3],
  thin = 1
)
}
\arguments{
\item{sampled_batch_scale}{A 3D array of sampled batch mean shifts.}

\item{B}{The number of batches present. Defaults to the number of columns in
the batch mean matrix from the first sample.}

\item{P}{The dimension of the batch mean shifts. Defaults to the number of
rows in the batch mean matrix from the first sample.}

\item{R}{The number of iterations run. Defaults to the number of slices in
the sampled batch mean array.}

\item{thin}{The thinning factor of the sampler. Defaults to 1.}
}
\value{
A data.frame of three columns; the parameter, the sampled value and the iteration.
}
\description{
Given an array of sampled batch scales from the
``mixtureModel`` function, acquire a tidy version ready for ``ggplot2`` use.
}
\examples{

# Data in matrix format
X <- matrix(c(rnorm(100, 0, 1), rnorm(100, 3, 1)), ncol = 2, byrow = TRUE)

# Observed batches represented by integers
batch_vec <- sample(seq(1, 5), size = 100, replace = TRUE)

# MCMC iterations (this is too low for real use)
R <- 100
thin <- 5

# MCMC samples
samples <- runBatchMix(X, R, thin, batch_vec, "MVN")

batch_scale_df <- getSampledBatchShift(samples$batch_scale, R = R, thin = thin)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{createSimilarityMat}
\alias{createSimilarityMat}
\title{Create Similarity Matrix}
\usage{
createSimilarityMat(allocations)
}
\arguments{
\item{allocations}{Matrix of sampled partitions. Columns correspond to 
items/samples being clustered, each row is a sampled partition.//'}
}
\value{
A symmetric n x n matrix (for n rows in cluster record) describing 
the fraction of iterations for which each pairwise combination of points are
assigned the same label.
}
\description{
Constructs a similarity matrix of the pairwise coclustering 
rate.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/collectAcceptanceRates.R
\name{collectAcceptanceRates}
\alias{collectAcceptanceRates}
\title{Collect acceptance rate}
\usage{
collectAcceptanceRates(samples)
}
\arguments{
\item{samples}{The output of ``runBatchMix``.}
}
\value{
A wide data.frame of all the sampled parameters and the iteration.
}
\description{
Collects the acceptance rates for each parameter into a data.frame
}
\examples{

# Data in a matrix format
X <- matrix(c(rnorm(100, 0, 1), rnorm(100, 3, 1)), ncol = 2, byrow = TRUE)

# Initial labelling
labels <- c(
  rep(1, 10),
  sample(c(1, 2), size = 40, replace = TRUE),
  rep(2, 10),
  sample(c(1, 2), size = 40, replace = TRUE)
)

fixed <- c(rep(1, 10), rep(0, 40), rep(1, 10), rep(0, 40))

# Batch
batch_vec <- sample(seq(1, 5), replace = TRUE, size = 100)

# Sampling parameters
R <- 1000
thin <- 50

# MCMC samples
samples <- runBatchMix(X, R, thin, batch_vec, "MVN",
  initial_labels = labels,
  fixed = fixed
)

# Acceptance rates
collectAcceptanceRates(samples)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predictFromMultipleChains.R
\name{predictFromMultipleChains}
\alias{predictFromMultipleChains}
\title{Predict from multiple MCMC chains}
\usage{
predictFromMultipleChains(
  mcmc_outputs,
  burn,
  point_estimate_method = "median",
  chains_already_processed = FALSE,
  nCores = 1L
)
}
\arguments{
\item{mcmc_outputs}{Output from ``runMCMCChains``}

\item{burn}{The number of MCMC samples to drop as part of a burn in.}

\item{point_estimate_method}{Summary statistic used to define the point
estimate. Must be ``'mean'`` or ``'median'``. ``'median'`` is the default.}

\item{chains_already_processed}{Logical indicating if the chains have already
had a burn-in applied.}

\item{nCores}{Integer used by salso to parallelize point estimate prediction.}
}
\value{
A named list of quantities related to prediction/clustering:

 * ``allocation_probability``: List with an $(N x K)$ matrix if the model is
   semi-supervised. The point estimate of the allocation probabilities for
   each data point to each class.

 * ``prob``: $N$ vector of the point estimate of the probability of being
   allocated to the class with the highest probability.

 * ``pred``: $N$ vector of the predicted class for each sample. If the model
   is unsupervised then the ``salso`` function from Dahl et al. (2021) is
   used on the sampled partitions using the default settings.

 * ``samples``: List of sampled allocations for each view. Columns
   correspond to items being clustered, rows to MCMC samples.
}
\description{
Applies a burn in to and finds a point estimate by combining
multiple chains of ``callMDI``.
}
\examples{
\donttest{
# Data dimensions
N <- 600
P <- 4
K <- 5
B <- 7

# Generating model parameters
mean_dist <- 2.25
batch_dist <- 0.3
group_means <- seq(1, K) * mean_dist
batch_shift <- rnorm(B, mean = batch_dist, sd = batch_dist)
std_dev <- rep(2, K)
batch_var <- rep(1.2, B)
group_weights <- rep(1 / K, K)
batch_weights <- rep(1 / B, B)
dfs <- c(4, 7, 15, 60, 120)

my_data <- generateBatchData(
  N,
  P,
  group_means,
  std_dev,
  batch_shift,
  batch_var,
  group_weights,
  batch_weights,
  type = "MVT",
  group_dfs = dfs
)


X <- my_data$observed_data

true_labels <- my_data$group_IDs
fixed <- my_data$fixed
batch_vec <- my_data$batch_IDs

alpha <- 1
initial_labels <- generateInitialLabels(alpha, K, fixed, true_labels)

# Sampling parameters
R <- 1000
thin <- 25
burn <- 100
n_chains <- 2

# Density choice
type <- "MVT"

# MCMC samples and BIC vector
mcmc_outputs <- runMCMCChains(
  X,
  n_chains,
  R,
  thin,
  batch_vec,
  type,
  initial_labels = initial_labels,
  fixed = fixed
)
ensemble_mod <- predictFromMultipleChains(mcmc_outputs, burn)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generateBatchDataVaryingRepresentation.R
\name{generateBatchDataVaryingRepresentation}
\alias{generateBatchDataVaryingRepresentation}
\title{Generate batch data}
\usage{
generateBatchDataVaryingRepresentation(
  N,
  P,
  group_means,
  group_std_dev,
  batch_shift,
  batch_scale,
  group_weights,
  batch_weights,
  frac_known = 0.2
)
}
\arguments{
\item{N}{The number of items (rows) to generate.}

\item{P}{The number of columns in the generated dataset.}

\item{group_means}{A vector of the group means for a column.}

\item{group_std_dev}{A vector of group standard deviations for a column.}

\item{batch_shift}{A vector of batch means in a column.}

\item{batch_scale}{A vector of batch standard deviations within a column.}

\item{group_weights}{A K x B matrix of the expected proportion of N in each group in each batch.}

\item{batch_weights}{A vector of the expected proportion of N in each batch.}

\item{frac_known}{The expected fraction of observed labels. Used to generate
a ``fixed`` vector to feed into the ``batchSemiSupervisedMixtureModel`` function.}
}
\value{
A list of 4 objects; the data generated from the groups with and
without batch effects, the label indicating the generating group and the
batch label.
}
\description{
Generate data from groups across batches. Assumes independence
across columns. In each column the parameters are randomly permuted for both
the groups and batches.
}
\examples{
N <- 500
P <- 2
K <- 2
B <- 5
mean_dist <- 4
batch_dist <- 0.3
group_means <- seq(1, K) * mean_dist
batch_shift <- rnorm(B, mean = batch_dist, sd = batch_dist)
std_dev <- rep(2, K)
batch_var <- rep(1.2, B)
group_weights <- matrix(
  c(
    0.8, 0.6, 0.4, 0.2, 0.2,
    0.2, 0.4, 0.6, 0.8, 0.8
  ),
  nrow = K, ncol = B, byrow = TRUE
)
batch_weights <- rep(1 / B, B)

my_data <- generateBatchDataVaryingRepresentation(
  N,
  P,
  group_means,
  std_dev,
  batch_shift,
  batch_var,
  group_weights,
  batch_weights
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/processMCMCChain.R
\name{processMCMCChain}
\alias{processMCMCChain}
\title{Process MCMC chain}
\usage{
processMCMCChain(mcmc_output, burn, point_estimate_method = "median")
}
\arguments{
\item{mcmc_output}{Output from ``batchSemiSupervisedMixtureModel``}

\item{burn}{The number of MCMC samples to drop as part of a burn in.}

\item{point_estimate_method}{Summary statistic used to define the point
estimate. Must be ``'mean'`` or ``'median'``. ``'median'`` is the default.}
}
\value{
A named list similar to the output of
``batchSemiSupervisedMixtureModel`` with some additional entries:

 * ``mean_est``: $(P x K)$ matrix. The point estimate of the cluster
 means with columns  corresponding to clusters.

 * ``cov_est``: $(P x P x K)$ array. The point estimate of the
 cluster covariance matrices with slices corresponding to clusters.

 * ``shift_est``: $(P x B)$ matrix. The point estimate of the batch
 shift effect with columns  corresponding to batches.

 * ``scale_est``: $(P x B)$ matrix. The point estimate of the batch
 scale effects. The $bth$ column contains the diagonal entries of the scaling
 matrix for the $bth£ batch.

 * ``mean_sum_est``: $(P x K x B)$ array. The point estimate of the
 sum of the cluster  means and the batch shift effect with columns
 corresponding to clusters and slices to batches.

 * ``cov_comb_est``: List of length $B$, with each entry being a
 $(P x P x K)$ array. The point estimate of the combination of the
 cluster covariance matrices and the batch scale effect with list entries
 corresponding to batches and slices of each array corresponding to clusters.

 * ``inferred_dataset``: $(N x P)$ matrix. The inferred ``batch-free''
 dataset.

 * ``allocation_probability``: $(N x K)$ matrix. The point estimate of
 the allocation probabilities for each data point to each class.

 * ``prob``: $N$ vector. The point estimate of the probability of being
 allocated to the class with the highest probability.

 * ``pred``: $N$ vector. The predicted class for each sample.
}
\description{
Applies a burn in to and finds a point estimate for the output
of ``batchSemiSupervisedMixtureModel``.
}
\examples{

# Data in a matrix format
X <- matrix(c(rnorm(100, 0, 1), rnorm(100, 3, 1)), ncol = 2, byrow = TRUE)

# Initial labelling
labels <- c(
  rep(1, 10),
  sample(c(1, 2), size = 40, replace = TRUE),
  rep(2, 10),
  sample(c(1, 2), size = 40, replace = TRUE)
)

fixed <- c(rep(1, 10), rep(0, 40), rep(1, 10), rep(0, 40))

# Batch
batch_vec <- sample(seq(1, 5), replace = TRUE, size = 100)

# Sampling parameters
R <- 1000
burn <- 250
thin <- 50

# MCMC samples
samples <- runBatchMix(X, R, thin, batch_vec, "MVN",
  initial_labels = labels,
  fixed = fixed
)

# Process the MCMC samples
processed_samples <- processMCMCChain(samples, burn)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/batchSemiSupervisedMixtureModel.R
\name{batchSemiSupervisedMixtureModel}
\alias{batchSemiSupervisedMixtureModel}
\title{Batch semisupervised mixture model}
\usage{
batchSemiSupervisedMixtureModel(
  X,
  R,
  thin,
  initial_labels,
  fixed,
  batch_vec,
  type,
  K_max = length(unique(initial_labels)),
  alpha = NULL,
  concentration = NULL,
  mu_proposal_window = 0.5^2,
  cov_proposal_window = 0.002,
  m_proposal_window = 0.3^2,
  S_proposal_window = 0.01,
  t_df_proposal_window = 0.015,
  m_scale = NULL,
  rho = 3,
  theta = 1,
  initial_class_means = NULL,
  initial_class_covariance = NULL,
  initial_batch_shift = NULL,
  initial_batch_scale = NULL,
  initial_class_df = NULL,
  verbose = TRUE
)
}
\arguments{
\item{X}{Data to cluster as a matrix with the items to cluster held in rows.}

\item{R}{The number of iterations in the sampler.}

\item{thin}{The factor by which the samples generated are thinned, e.g. if
``thin=50`` only every 50th sample is kept.}

\item{initial_labels}{Initial clustering.}

\item{fixed}{Which items are fixed in their initial label.}

\item{batch_vec}{Labels identifying which batch each item being clustered is
from.}

\item{type}{Character indicating density type to use. One of 'MVN'
(multivariate normal distribution) or 'MVT' (multivariate t distribution).}

\item{K_max}{The number of components to include (the upper bound on the
number of clusters in each sample). Defaults to the number of unique labels
in ``initial_labels``.}

\item{alpha}{The concentration parameter for the stick-breaking prior and the
weights in the model.}

\item{concentration}{Initial concentration vector for component weights.}

\item{mu_proposal_window}{The proposal window for the cluster mean proposal
kernel. The proposal density is a Gaussian distribution, the window is the
variance.}

\item{cov_proposal_window}{The proposal window for the cluster covariance
proposal kernel. The proposal density is a Wishart distribution, this
argument is the reciprocal of the degree of freedom.}

\item{m_proposal_window}{The proposal window for the batch mean proposal
 kernel. The proposal density is a Gaussian distribution, the window is the
variance.}

\item{S_proposal_window}{The proposal window for the batch standard deviation
 proposal kernel. The proposal density is a Gamma distribution, this
argument is the reciprocal of the rate.}

\item{t_df_proposal_window}{The proposal window for the degrees of freedom
for the multivariate t distribution (not used if type is not 'MVT'). The
proposal density is a Gamma distribution, this argument is the reciprocal of
the rate.}

\item{m_scale}{The scale hyperparameter for the batch shift prior
distribution. This defines the scale of the batch effect upon the mean and
should be in (0, 1].}

\item{rho}{The shape of the prior distribution for the batch scale.}

\item{theta}{The scale of the prior distribution for the batch scale.}

\item{initial_class_means}{A $P x K$ matrix of initial values for the class
means. Defaults to draws from the prior distribution.}

\item{initial_class_covariance}{A $P x P x K$ array of initial values for
the class covariance matrices. Defaults to draws from the prior distribution.}

\item{initial_batch_shift}{A $P x B$ matrix of initial values for the batch
shift effect Defaults to draws from the prior distribution.}

\item{initial_batch_scale}{A $P x B$ matrix of initial values for the batch
scales Defaults to draws from the prior distribution.}

\item{initial_class_df}{A $K$ vector of initial values for the class degrees
of freedom. Defaults to draws from the prior distribution.}

\item{verbose}{Logiccal indicating if warning about proposal windows should
be printed.}
}
\value{
A named list containing the sampled partitions, cluster and batch
parameters, model fit measures and some details on the model call.
}
\description{
A Bayesian mixture model with batch effects.
}
\examples{

# Data in a matrix format
X <- matrix(c(rnorm(100, 0, 1), rnorm(100, 3, 1)), ncol = 2, byrow = TRUE)

# Initial labelling
labels <- c(
  rep(1, 10),
  sample(c(1, 2), size = 40, replace = TRUE),
  rep(2, 10),
  sample(c(1, 2), size = 40, replace = TRUE)
)

fixed <- c(rep(1, 10), rep(0, 40), rep(1, 10), rep(0, 40))

# Batch
batch_vec <- sample(seq(1, 5), replace = TRUE, size = 100)

# Density choice
type <- "MVN"

# Sampling parameters
R <- 1000
thin <- 50

# MCMC samples and BIC vector
samples <- batchSemiSupervisedMixtureModel(
  X,
  R,
  thin,
  labels,
  fixed,
  batch_vec,
  type
)

# Given an initial value for the parameters
initial_class_means <- matrix(c(1, 1, 3, 4), nrow = 2)
initial_class_covariance <- array(c(1, 0, 0, 1, 1, 0, 0, 1),
  dim = c(2, 2, 2)
)

# We can use values from a previous chain
initial_batch_shift <- samples$batch_shift[, , R / thin]
initial_batch_scale <- matrix(
  c(1.2, 1.3, 1.7, 1.1, 1.4, 1.3, 1.2, 1.2, 1.1, 2.0),
  nrow = 2
)

samples <- batchSemiSupervisedMixtureModel(X,
  R,
  thin,
  labels,
  fixed,
  batch_vec,
  type,
  initial_class_means = initial_class_means,
  initial_class_covariance = initial_class_covariance,
  initial_batch_shift = initial_batch_shift,
  initial_batch_scale = initial_batch_scale
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generateBatchDataLogPoisson.R
\name{generateBatchDataLogPoisson}
\alias{generateBatchDataLogPoisson}
\title{Generate batch data}
\usage{
generateBatchDataLogPoisson(
  N,
  P,
  group_rates,
  batch_rates,
  group_weights,
  batch_weights,
  frac_known = 0.2,
  permute_variables = TRUE,
  scale_data = FALSE
)
}
\arguments{
\item{N}{The number of items (rows) to generate.}

\item{P}{The number of columns in the generated dataset.}

\item{group_rates}{A vector of the group rates for the classes within a column.}

\item{batch_rates}{A vector of the batch rates for the classes within a column.
This is used to create a variable which has the sum of the appropriate batch
and class rate, it might be better interpreted as the batch effect on the
observed rate.}

\item{group_weights}{One of either a K x B matrix of the expected proportion
of each batch in each group or a K-vector of the expected proportion of the
entire dataset in each group.}

\item{batch_weights}{A vector of the expected proportion of N in each batch.}

\item{frac_known}{The number of items with known labels.}

\item{permute_variables}{Logical indicating if group and batch means and
standard deviations should be permuted in each column or not (defaults to
``TRUE``).}

\item{scale_data}{Logical indicating if data should be mean centred and
standardised (defaults to ``FALSE``).}
}
\value{
A list of 5 objects; the data generated from the groups with and
without batch effects, the label indicating the generating group, the
batch label and the vector indicating training versus test.
}
\description{
Generate data from K multivaraite normal or multivariate t
distributions with additional noise from batches. Assumes independence across
columns. In each column the parameters are randomly permuted for both the
groups and batches.
}
